@@ -77,11 +77,15 @@ static av_always_inline type bytestream2_get_ ## name(GetByteContext *g)       \
     }                                                                          \
     return bytestream2_get_ ## name ## u(g);                                   \
 }                                                                              \
+static av_always_inline type bytestream2_peek_ ## name ## u(GetByteContext *g) \
+{                                                                              \
+    return read(g->buffer);                                                    \
+}                                                                              \
 static av_always_inline type bytestream2_peek_ ## name(GetByteContext *g)      \
 {                                                                              \
     if (g->buffer_end - g->buffer < bytes)                                     \
         return 0;                                                              \
-    return read(g->buffer);                                                    \
+    return bytestream2_peek_ ## name ## u(g);                                  \
 }
 
 DEF(uint64_t,     le64, 8, AV_RL64, AV_WL64)
@@ -176,7 +180,7 @@ static av_always_inline void bytestream2_skipu(GetByteContext *g,
 static av_always_inline void bytestream2_skip_p(PutByteContext *p,
                                                 unsigned int size)
 {
-    int size2;
+    unsigned int size2;
     if (p->eof)
         return;
     size2 = FFMIN(p->buffer_end - p->buffer, size);
@@ -264,7 +268,7 @@ static av_always_inline unsigned int bytestream2_get_buffer(GetByteContext *g,
                                                             uint8_t *dst,
                                                             unsigned int size)
 {
-    int size2 = FFMIN(g->buffer_end - g->buffer, size);
+    unsigned int size2 = FFMIN(g->buffer_end - g->buffer, size);
     memcpy(dst, g->buffer, size2);
     g->buffer += size2;
     return size2;
@@ -283,7 +287,7 @@ static av_always_inline unsigned int bytestream2_put_buffer(PutByteContext *p,
                                                             const uint8_t *src,
                                                             unsigned int size)
 {
-    int size2;
+    unsigned int size2;
     if (p->eof)
         return 0;
     size2 = FFMIN(p->buffer_end - p->buffer, size);
@@ -307,7 +311,7 @@ static av_always_inline void bytestream2_set_buffer(PutByteContext *p,
                                                     const uint8_t c,
                                                     unsigned int size)
 {
-    int size2;
+    unsigned int size2;
     if (p->eof)
         return;
     size2 = FFMIN(p->buffer_end - p->buffer, size);
@@ -344,7 +348,7 @@ static av_always_inline unsigned int bytestream2_copy_buffer(PutByteContext *p,
                                                              GetByteContext *g,
                                                              unsigned int size)
 {
-    int size2;
+    unsigned int size2;
 
     if (p->eof)
         return 0;
