@@ -21,9 +21,11 @@
 #ifndef AVCODEC_DCADEC_H
 #define AVCODEC_DCADEC_H
 
-#include "libavutil/common.h"
+#include <stdint.h>
+
 #include "libavutil/crc.h"
 #include "libavutil/float_dsp.h"
+#include "libavutil/log.h"
 
 #include "avcodec.h"
 #include "get_bits.h"
@@ -43,6 +45,11 @@
 #define DCA_PACKET_RECOVERY     0x10    ///< Sync error recovery flag
 #define DCA_PACKET_RESIDUAL     0x20    ///< Core valid for residual decoding
 
+enum DCAOutputChannelOrder {
+    CHANNEL_ORDER_DEFAULT,
+    CHANNEL_ORDER_CODED,
+};
+
 typedef struct DCAContext {
     const AVClass   *class;       ///< class for AVOptions
     AVCodecContext  *avctx;
@@ -63,6 +70,8 @@ typedef struct DCAContext {
 
     int     request_channel_layout; ///< Converted from avctx.request_channel_layout
     int     core_only;              ///< Core only decoding flag
+    int     output_channel_order;
+    AVChannelLayout downmix_layout;
 } DCAContext;
 
 int ff_dca_set_channel_layout(AVCodecContext *avctx, int *ch_remap, int dca_mask);
