@@ -27,14 +27,14 @@
 
 #include <limits.h>
 
-#include "libavutil/internal.h"
+#include "libavutil/mem.h"
 #include "avcodec.h"
 #include "error_resilience.h"
 #include "me_cmp.h"
 #include "mpegutils.h"
 #include "mpegvideo.h"
-#include "rectangle.h"
 #include "threadframe.h"
+#include "threadprogress.h"
 
 /**
  * @param stride the number of MVs to get to the next row
@@ -410,8 +410,12 @@ static void guess_mv(ERContext *s)
     set_mv_strides(s, &mot_step, &mot_stride);
 
     num_avail = 0;
-    if (s->last_pic.motion_val[0])
-        ff_thread_await_progress(s->last_pic.tf, mb_height-1, 0);
+    if (s->last_pic.motion_val[0]) {
+        if (s->last_pic.tf)
+            ff_thread_await_progress(s->last_pic.tf, mb_height-1, 0);
+        else
+            ff_thread_progress_await(s->last_pic.progress, mb_height - 1);
+    }
     for (i = 0; i < mb_width * mb_height; i++) {
         const int mb_xy = s->mb_index2xy[i];
         int f = 0;
@@ -764,7 +768,7 @@ static int is_intra_more_likely(ERContext *s)
                 if (s->avctx->codec_id == AV_CODEC_ID_H264) {
                     // FIXME
                 } else {
-                    ff_thread_await_progress(s->last_pic.tf, mb_y, 0);
+                    ff_thread_progress_await(s->last_pic.progress, mb_y);
                 }
                 is_intra_likely += s->sad(NULL, last_mb_ptr, mb_ptr,
                                           linesize[0], 16);
@@ -898,6 +902,7 @@ void ff_er_frame_end(ERContext *s, int *decode_error_flags)
     int threshold = 50;
     int is_intra_likely;
     int size = s->b8_stride * 2 * s->mb_height;
+    int guessed_mb_type;
 
     /* We do not support ER of field pictures yet,
      * though it should not crash if enabled. */
@@ -949,19 +954,10 @@ void ff_er_frame_end(ERContext *s, int *decode_error_flags)
             s->ref_index[i]       = av_calloc(s->mb_stride * s->mb_height, 4 * sizeof(uint8_t));
             s->motion_val_base[i] = av_calloc(size + 4, 2 * sizeof(uint16_t));
             if (!s->ref_index[i] || !s->motion_val_base[i])
-                break;
+                goto cleanup;
             s->cur_pic.ref_index[i]  = s->ref_index[i];
             s->cur_pic.motion_val[i] = s->motion_val_base[i] + 4;
         }
-        if (i < 2) {
-            for (i = 0; i < 2; i++) {
-                av_freep(&s->ref_index[i]);
-                av_freep(&s->motion_val_base[i]);
-                s->cur_pic.ref_index[i]  = NULL;
-                s->cur_pic.motion_val[i] = NULL;
-            }
-            return;
-        }
     }
 
     if (s->avctx->debug & FF_DEBUG_ER) {
@@ -1122,16 +1118,15 @@ void ff_er_frame_end(ERContext *s, int *decode_error_flags)
     is_intra_likely = is_intra_more_likely(s);
 
     /* set unknown mb-type to most likely */
+    guessed_mb_type = is_intra_likely ? MB_TYPE_INTRA4x4 :
+                         (MB_TYPE_16x16 | (s->avctx->codec_id == AV_CODEC_ID_H264 ? MB_TYPE_L0 : MB_TYPE_FORWARD_MV));
     for (i = 0; i < s->mb_num; i++) {
         const int mb_xy = s->mb_index2xy[i];
         int error = s->error_status_table[mb_xy];
         if (!((error & ER_DC_ERROR) && (error & ER_MV_ERROR)))
             continue;
 
-        if (is_intra_likely)
-            s->cur_pic.mb_type[mb_xy] = MB_TYPE_INTRA4x4;
-        else
-            s->cur_pic.mb_type[mb_xy] = MB_TYPE_16x16 | MB_TYPE_L0;
+        s->cur_pic.mb_type[mb_xy] = guessed_mb_type;
     }
 
     // change inter to intra blocks if no reference frames are available
@@ -1208,7 +1203,7 @@ void ff_er_frame_end(ERContext *s, int *decode_error_flags)
                     int time_pb = s->pb_time;
 
                     av_assert0(s->avctx->codec_id != AV_CODEC_ID_H264);
-                    ff_thread_await_progress(s->next_pic.tf, mb_y, 0);
+                    ff_thread_progress_await(s->next_pic.progress, mb_y);
 
                     s->mv[0][0][0] = s->next_pic.motion_val[0][xy][0] *  time_pb            / time_pp;
                     s->mv[0][0][1] = s->next_pic.motion_val[0][xy][1] *  time_pb            / time_pp;
@@ -1345,14 +1340,15 @@ void ff_er_frame_end(ERContext *s, int *decode_error_flags)
             s->mbintra_table[mb_xy] = 1;
     }
 
+    memset(&s->cur_pic, 0, sizeof(ERPicture));
+    memset(&s->last_pic, 0, sizeof(ERPicture));
+    memset(&s->next_pic, 0, sizeof(ERPicture));
+
+cleanup:
     for (i = 0; i < 2; i++) {
         av_freep(&s->ref_index[i]);
         av_freep(&s->motion_val_base[i]);
         s->cur_pic.ref_index[i]  = NULL;
         s->cur_pic.motion_val[i] = NULL;
     }
-
-    memset(&s->cur_pic, 0, sizeof(ERPicture));
-    memset(&s->last_pic, 0, sizeof(ERPicture));
-    memset(&s->next_pic, 0, sizeof(ERPicture));
 }
