@@ -27,11 +27,11 @@
 
 #include "libavutil/attributes.h"
 #include "libavutil/avassert.h"
+#include "libavutil/mem.h"
 
 #include "avcodec.h"
-#include "rangecoder.h"
 #include "ffv1.h"
-#include "threadframe.h"
+#include "refstruct.h"
 
 av_cold int ff_ffv1_common_init(AVCodecContext *avctx)
 {
@@ -53,16 +53,33 @@ av_cold int ff_ffv1_common_init(AVCodecContext *avctx)
     return 0;
 }
 
-av_cold int ff_ffv1_init_slice_state(const FFV1Context *f, FFV1Context *fs)
+static void planes_free(FFRefStructOpaque opaque, void *obj)
+{
+    PlaneContext *planes = obj;
+
+    for (int i = 0; i < MAX_PLANES; i++) {
+        PlaneContext *p = &planes[i];
+
+        av_freep(&p->state);
+        av_freep(&p->vlc_state);
+    }
+}
+
+PlaneContext* ff_ffv1_planes_alloc(void)
+{
+    return ff_refstruct_alloc_ext(sizeof(PlaneContext) * MAX_PLANES,
+                                  0, NULL, planes_free);
+}
+
+av_cold int ff_ffv1_init_slice_state(const FFV1Context *f,
+                                     FFV1SliceContext *sc)
 {
     int j, i;
 
-    fs->plane_count  = f->plane_count;
-    fs->transparency = f->transparency;
     for (j = 0; j < f->plane_count; j++) {
-        PlaneContext *const p = &fs->plane[j];
+        PlaneContext *const p = &sc->plane[j];
 
-        if (fs->ac != AC_GOLOMB_RICE) {
+        if (f->ac != AC_GOLOMB_RICE) {
             if (!p->state)
                 p->state = av_calloc(p->context_count, CONTEXT_SIZE *
                                      sizeof(uint8_t));
@@ -81,11 +98,11 @@ av_cold int ff_ffv1_init_slice_state(const FFV1Context *f, FFV1Context *fs)
         }
     }
 
-    if (fs->ac == AC_RANGE_CUSTOM_TAB) {
+    if (f->ac == AC_RANGE_CUSTOM_TAB) {
         //FIXME only redo if state_transition changed
         for (j = 1; j < 256; j++) {
-            fs->c. one_state[      j] = f->state_transition[j];
-            fs->c.zero_state[256 - j] = 256 - fs->c.one_state[j];
+            sc->c. one_state[      j] = f->state_transition[j];
+            sc->c.zero_state[256 - j] = 256 - sc->c.one_state[j];
         }
     }
 
@@ -96,8 +113,7 @@ av_cold int ff_ffv1_init_slices_state(FFV1Context *f)
 {
     int i, ret;
     for (i = 0; i < f->max_slice_count; i++) {
-        FFV1Context *fs = f->slice_context[i];
-        if ((ret = ff_ffv1_init_slice_state(f, fs)) < 0)
+        if ((ret = ff_ffv1_init_slice_state(f, &f->slices[i])) < 0)
             return AVERROR(ENOMEM);
     }
     return 0;
@@ -112,44 +128,45 @@ int ff_need_new_slices(int width, int num_h_slices, int chroma_shift) {
 
 av_cold int ff_ffv1_init_slice_contexts(FFV1Context *f)
 {
-    int i, max_slice_count = f->num_h_slices * f->num_v_slices;
+    int max_slice_count = f->num_h_slices * f->num_v_slices;
 
     av_assert0(max_slice_count > 0);
 
-    for (i = 0; i < max_slice_count;) {
+    f->slices = av_calloc(max_slice_count, sizeof(*f->slices));
+    if (!f->slices)
+        return AVERROR(ENOMEM);
+
+    f->max_slice_count = max_slice_count;
+
+    for (int i = 0; i < max_slice_count; i++) {
+        FFV1SliceContext *sc = &f->slices[i];
         int sx          = i % f->num_h_slices;
         int sy          = i / f->num_h_slices;
         int sxs         = f->avctx->width  *  sx      / f->num_h_slices;
         int sxe         = f->avctx->width  * (sx + 1) / f->num_h_slices;
         int sys         = f->avctx->height *  sy      / f->num_v_slices;
         int sye         = f->avctx->height * (sy + 1) / f->num_v_slices;
-        FFV1Context *fs = av_mallocz(sizeof(*fs));
 
-        if (!fs)
-            goto memfail;
+        sc->slice_width  = sxe - sxs;
+        sc->slice_height = sye - sys;
+        sc->slice_x      = sxs;
+        sc->slice_y      = sys;
+        sc->sx           = sx;
+        sc->sy           = sy;
 
-        f->slice_context[i++] = fs;
-        memcpy(fs, f, sizeof(*fs));
-        memset(fs->rc_stat2, 0, sizeof(fs->rc_stat2));
+        sc->sample_buffer = av_malloc_array((f->width + 6), 3 * MAX_PLANES *
+                                            sizeof(*sc->sample_buffer));
+        sc->sample_buffer32 = av_malloc_array((f->width + 6), 3 * MAX_PLANES *
+                                              sizeof(*sc->sample_buffer32));
+        if (!sc->sample_buffer || !sc->sample_buffer32)
+            return AVERROR(ENOMEM);
 
-        fs->slice_width  = sxe - sxs;
-        fs->slice_height = sye - sys;
-        fs->slice_x      = sxs;
-        fs->slice_y      = sys;
-
-        fs->sample_buffer = av_malloc_array((fs->width + 6), 3 * MAX_PLANES *
-                                      sizeof(*fs->sample_buffer));
-        fs->sample_buffer32 = av_malloc_array((fs->width + 6), 3 * MAX_PLANES *
-                                        sizeof(*fs->sample_buffer32));
-        if (!fs->sample_buffer || !fs->sample_buffer32)
-            goto memfail;
+        sc->plane = ff_ffv1_planes_alloc();
+        if (!sc->plane)
+            return AVERROR(ENOMEM);
     }
-    f->max_slice_count = max_slice_count;
-    return 0;
 
-memfail:
-    f->max_slice_count = i;
-    return AVERROR(ENOMEM);
+    return 0;
 }
 
 int ff_ffv1_allocate_initial_states(FFV1Context *f)
@@ -167,17 +184,14 @@ int ff_ffv1_allocate_initial_states(FFV1Context *f)
     return 0;
 }
 
-void ff_ffv1_clear_slice_state(const FFV1Context *f, FFV1Context *fs)
+void ff_ffv1_clear_slice_state(const FFV1Context *f, FFV1SliceContext *sc)
 {
     int i, j;
 
     for (i = 0; i < f->plane_count; i++) {
-        PlaneContext *p = &fs->plane[i];
+        PlaneContext *p = &sc->plane[i];
 
-        p->interlace_bit_state[0] = 128;
-        p->interlace_bit_state[1] = 128;
-
-        if (fs->ac != AC_GOLOMB_RICE) {
+        if (f->ac != AC_GOLOMB_RICE) {
             if (f->initial_states[p->quant_table_index]) {
                 memcpy(p->state, f->initial_states[p->quant_table_index],
                        CONTEXT_SIZE * p->context_count);
@@ -201,29 +215,27 @@ av_cold int ff_ffv1_close(AVCodecContext *avctx)
     int i, j;
 
     for (j = 0; j < s->max_slice_count; j++) {
-        FFV1Context *fs = s->slice_context[j];
-        for (i = 0; i < s->plane_count; i++) {
-            PlaneContext *p = &fs->plane[i];
+        FFV1SliceContext *sc = &s->slices[j];
 
-            av_freep(&p->state);
-            av_freep(&p->vlc_state);
-        }
-        av_freep(&fs->sample_buffer);
-        av_freep(&fs->sample_buffer32);
+        av_freep(&sc->sample_buffer);
+        av_freep(&sc->sample_buffer32);
+
+        ff_refstruct_unref(&sc->plane);
     }
 
+    ff_refstruct_unref(&s->slice_damaged);
+
     av_freep(&avctx->stats_out);
     for (j = 0; j < s->quant_table_count; j++) {
         av_freep(&s->initial_states[j]);
         for (i = 0; i < s->max_slice_count; i++) {
-            FFV1Context *sf = s->slice_context[i];
-            av_freep(&sf->rc_stat2[j]);
+            FFV1SliceContext *sc = &s->slices[i];
+            av_freep(&sc->rc_stat2[j]);
         }
         av_freep(&s->rc_stat2[j]);
     }
 
-    for (i = 0; i < s->max_slice_count; i++)
-        av_freep(&s->slice_context[i]);
+    av_freep(&s->slices);
 
     return 0;
 }
