@@ -33,10 +33,9 @@
 #include "acelp_vectors.h"
 #include "avcodec.h"
 #include "celp_filters.h"
-#include "celp_math.h"
 #include "codec_internal.h"
+#include "decode.h"
 #include "get_bits.h"
-#include "internal.h"
 #include "g723_1.h"
 
 #define CNG_RANDOM_SEED 12345
@@ -1113,13 +1112,16 @@ static const AVClass g723_1dec_class = {
 
 const FFCodec ff_g723_1_decoder = {
     .p.name         = "g723_1",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("G.723.1"),
+    CODEC_LONG_NAME("G.723.1"),
     .p.type         = AVMEDIA_TYPE_AUDIO,
     .p.id           = AV_CODEC_ID_G723_1,
     .priv_data_size = sizeof(G723_1_Context),
     .init           = g723_1_decode_init,
     FF_CODEC_DECODE_CB(g723_1_decode_frame),
-    .p.capabilities = AV_CODEC_CAP_SUBFRAMES | AV_CODEC_CAP_DR1,
+    .p.capabilities =
+#if FF_API_SUBFRAMES
+                      AV_CODEC_CAP_SUBFRAMES |
+#endif
+                      AV_CODEC_CAP_DR1,
     .p.priv_class   = &g723_1dec_class,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE,
 };
