@@ -685,7 +685,7 @@ static av_cold int decode_init(AVCodecContext *avctx)
 
 const FFCodec ff_indeo5_decoder = {
     .p.name         = "indeo5",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Intel Indeo Video Interactive 5"),
+    CODEC_LONG_NAME("Intel Indeo Video Interactive 5"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_INDEO5,
     .priv_data_size = sizeof(IVI45DecContext),
@@ -693,5 +693,5 @@ const FFCodec ff_indeo5_decoder = {
     .close          = ff_ivi_decode_close,
     FF_CODEC_DECODE_CB(ff_ivi_decode_frame),
     .p.capabilities = AV_CODEC_CAP_DR1,
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE | FF_CODEC_CAP_INIT_CLEANUP,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
 };
