@@ -32,11 +32,10 @@
 
 #include <stdint.h>
 #include "libavutil/intreadwrite.h"
-#include "avcodec.h"
 #include "get_bits.h"
 
 typedef struct lag_rac {
-    AVCodecContext *avctx;
+    void    *logctx;
     unsigned low;
     unsigned range;
     unsigned scale;             /**< Number of bits of precision in range. */
