@@ -23,7 +23,7 @@
 #include "avcodec.h"
 #include "bytestream.h"
 #include "codec_internal.h"
-#include "internal.h"
+#include "decode.h"
 
 typedef struct OggVorbisDecContext {
     vorbis_info vi;                     /**< vorbis_info used during init   */
@@ -218,10 +218,11 @@ static int oggvorbis_decode_close(AVCodecContext *avccontext) {
 
 const FFCodec ff_libvorbis_decoder = {
     .p.name         = "libvorbis",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("libvorbis"),
+    CODEC_LONG_NAME("libvorbis"),
     .p.type         = AVMEDIA_TYPE_AUDIO,
     .p.id           = AV_CODEC_ID_VORBIS,
     .p.capabilities = AV_CODEC_CAP_DELAY | AV_CODEC_CAP_CHANNEL_CONF,
+    .caps_internal  = FF_CODEC_CAP_NOT_INIT_THREADSAFE,
     .priv_data_size = sizeof(OggVorbisDecContext),
     .init           = oggvorbis_decode_init,
     FF_CODEC_DECODE_CB(oggvorbis_decode_frame),
