@@ -156,7 +156,7 @@ static const int8_t bilinear_filters_msa[7][2] = {
                  out0, out1, out2, out3);                                     \
 }
 
-static void common_hz_6t_4x4_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_6t_4x4_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -184,7 +184,7 @@ static void common_hz_6t_4x4_msa(uint8_t *src, int32_t src_stride,
     ST_W4(out, 0, 1, 2, 3, dst, dst_stride);
 }
 
-static void common_hz_6t_4x8_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_6t_4x8_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -220,7 +220,7 @@ static void common_hz_6t_4x8_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_epel4_h6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     const int8_t *filter = subpel_filters_msa[mx - 1];
@@ -233,7 +233,7 @@ void ff_put_vp8_epel4_h6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_h6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -281,7 +281,7 @@ void ff_put_vp8_epel8_h6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel16_h6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                              uint8_t *src, ptrdiff_t src_stride,
+                              const uint8_t *src, ptrdiff_t src_stride,
                               int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -330,7 +330,7 @@ void ff_put_vp8_epel16_h6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel4_v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -377,7 +377,7 @@ void ff_put_vp8_epel4_v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -427,7 +427,7 @@ void ff_put_vp8_epel8_v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel16_v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                              uint8_t *src, ptrdiff_t src_stride,
+                              const uint8_t *src, ptrdiff_t src_stride,
                               int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -501,7 +501,7 @@ void ff_put_vp8_epel16_v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel4_h6v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -576,7 +576,7 @@ void ff_put_vp8_epel4_h6v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_h6v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -663,7 +663,7 @@ void ff_put_vp8_epel8_h6v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 
 
 void ff_put_vp8_epel16_h6v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     int32_t multiple8_cnt;
@@ -677,7 +677,7 @@ void ff_put_vp8_epel16_h6v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void common_hz_4t_4x4_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_4t_4x4_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -704,7 +704,7 @@ static void common_hz_4t_4x4_msa(uint8_t *src, int32_t src_stride,
     ST_W4(out, 0, 1, 2, 3, dst, dst_stride);
 }
 
-static void common_hz_4t_4x8_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_4t_4x8_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -739,7 +739,7 @@ static void common_hz_4t_4x8_msa(uint8_t *src, int32_t src_stride,
     ST_W4(out, 0, 1, 2, 3, dst + 4 * dst_stride, dst_stride);
 }
 
-static void common_hz_4t_4x16_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_4t_4x16_msa(const uint8_t *src, int32_t src_stride,
                                   uint8_t *dst, int32_t dst_stride,
                                   const int8_t *filter)
 {
@@ -790,7 +790,7 @@ static void common_hz_4t_4x16_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_epel4_h4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     const int8_t *filter = subpel_filters_msa[mx - 1];
@@ -805,7 +805,7 @@ void ff_put_vp8_epel4_h4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_h4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -840,7 +840,7 @@ void ff_put_vp8_epel8_h4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel16_h4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                              uint8_t *src, ptrdiff_t src_stride,
+                              const uint8_t *src, ptrdiff_t src_stride,
                               int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -889,7 +889,7 @@ void ff_put_vp8_epel16_h4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel4_v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -936,7 +936,7 @@ void ff_put_vp8_epel4_v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                             uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                              int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -982,7 +982,7 @@ void ff_put_vp8_epel8_v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel16_v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                              uint8_t *src, ptrdiff_t src_stride,
+                              const uint8_t *src, ptrdiff_t src_stride,
                               int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1041,7 +1041,7 @@ void ff_put_vp8_epel16_v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel4_h4v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1100,7 +1100,7 @@ void ff_put_vp8_epel4_h4v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_h4v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1167,7 +1167,7 @@ void ff_put_vp8_epel8_h4v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel16_h4v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                 int height, int mx, int my)
 {
     int32_t multiple8_cnt;
@@ -1182,7 +1182,7 @@ void ff_put_vp8_epel16_h4v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel4_h6v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1248,7 +1248,7 @@ void ff_put_vp8_epel4_h6v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_h6v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1321,7 +1321,7 @@ void ff_put_vp8_epel8_h6v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel16_h6v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     int32_t multiple8_cnt;
@@ -1336,7 +1336,7 @@ void ff_put_vp8_epel16_h6v4_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel4_h4v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1400,7 +1400,7 @@ void ff_put_vp8_epel4_h4v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel8_h4v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                               uint8_t *src, ptrdiff_t src_stride,
+                               const uint8_t *src, ptrdiff_t src_stride,
                                int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1475,7 +1475,7 @@ void ff_put_vp8_epel8_h4v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_epel16_h4v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                 int height, int mx, int my)
 {
     int32_t multiple8_cnt;
@@ -1489,7 +1489,7 @@ void ff_put_vp8_epel16_h4v6_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void common_hz_2t_4x4_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_2t_4x4_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -1512,7 +1512,7 @@ static void common_hz_2t_4x4_msa(uint8_t *src, int32_t src_stride,
     ST_W2(res1, 0, 1, dst + 2 * dst_stride, dst_stride);
 }
 
-static void common_hz_2t_4x8_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_2t_4x8_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -1542,7 +1542,7 @@ static void common_hz_2t_4x8_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_bilinear4_h_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                 int height, int mx, int my)
 {
     const int8_t *filter = bilinear_filters_msa[mx - 1];
@@ -1554,7 +1554,7 @@ void ff_put_vp8_bilinear4_h_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void common_hz_2t_8x4_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_2t_8x4_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -1578,7 +1578,7 @@ static void common_hz_2t_8x4_msa(uint8_t *src, int32_t src_stride,
     ST_D4(src0, src1, 0, 1, 0, 1, dst, dst_stride);
 }
 
-static void common_hz_2t_8x8mult_msa(uint8_t *src, int32_t src_stride,
+static void common_hz_2t_8x8mult_msa(const uint8_t *src, int32_t src_stride,
                                      uint8_t *dst, int32_t dst_stride,
                                      const int8_t *filter, int32_t height)
 {
@@ -1642,7 +1642,7 @@ static void common_hz_2t_8x8mult_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_bilinear8_h_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                 int height, int mx, int my)
 {
     const int8_t *filter = bilinear_filters_msa[mx - 1];
@@ -1656,7 +1656,7 @@ void ff_put_vp8_bilinear8_h_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_bilinear16_h_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                 uint8_t *src, ptrdiff_t src_stride,
+                                 const uint8_t *src, ptrdiff_t src_stride,
                                  int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1722,7 +1722,7 @@ void ff_put_vp8_bilinear16_h_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void common_vt_2t_4x4_msa(uint8_t *src, int32_t src_stride,
+static void common_vt_2t_4x4_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -1748,7 +1748,7 @@ static void common_vt_2t_4x4_msa(uint8_t *src, int32_t src_stride,
     ST_W4(src2110, 0, 1, 2, 3, dst, dst_stride);
 }
 
-static void common_vt_2t_4x8_msa(uint8_t *src, int32_t src_stride,
+static void common_vt_2t_4x8_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -1783,7 +1783,7 @@ static void common_vt_2t_4x8_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_bilinear4_v_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                 int height, int mx, int my)
 {
     const int8_t *filter = bilinear_filters_msa[my - 1];
@@ -1795,7 +1795,7 @@ void ff_put_vp8_bilinear4_v_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void common_vt_2t_8x4_msa(uint8_t *src, int32_t src_stride,
+static void common_vt_2t_8x4_msa(const uint8_t *src, int32_t src_stride,
                                  uint8_t *dst, int32_t dst_stride,
                                  const int8_t *filter)
 {
@@ -1819,7 +1819,7 @@ static void common_vt_2t_8x4_msa(uint8_t *src, int32_t src_stride,
     ST_D4(out0, out1, 0, 1, 0, 1, dst, dst_stride);
 }
 
-static void common_vt_2t_8x8mult_msa(uint8_t *src, int32_t src_stride,
+static void common_vt_2t_8x8mult_msa(const uint8_t *src, int32_t src_stride,
                                      uint8_t *dst, int32_t dst_stride,
                                      const int8_t *filter, int32_t height)
 {
@@ -1865,7 +1865,7 @@ static void common_vt_2t_8x8mult_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_bilinear8_v_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                uint8_t *src, ptrdiff_t src_stride,
+                                const uint8_t *src, ptrdiff_t src_stride,
                                 int height, int mx, int my)
 {
     const int8_t *filter = bilinear_filters_msa[my - 1];
@@ -1879,7 +1879,7 @@ void ff_put_vp8_bilinear8_v_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_bilinear16_v_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                 uint8_t *src, ptrdiff_t src_stride,
+                                 const uint8_t *src, ptrdiff_t src_stride,
                                  int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -1932,7 +1932,7 @@ void ff_put_vp8_bilinear16_v_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void common_hv_2ht_2vt_4x4_msa(uint8_t *src, int32_t src_stride,
+static void common_hv_2ht_2vt_4x4_msa(const uint8_t *src, int32_t src_stride,
                                       uint8_t *dst, int32_t dst_stride,
                                       const int8_t *filter_horiz,
                                       const int8_t *filter_vert)
@@ -1966,7 +1966,7 @@ static void common_hv_2ht_2vt_4x4_msa(uint8_t *src, int32_t src_stride,
     ST_W2(res1, 0, 1, dst + 2 * dst_stride, dst_stride);
 }
 
-static void common_hv_2ht_2vt_4x8_msa(uint8_t *src, int32_t src_stride,
+static void common_hv_2ht_2vt_4x8_msa(const uint8_t *src, int32_t src_stride,
                                       uint8_t *dst, int32_t dst_stride,
                                       const int8_t *filter_horiz,
                                       const int8_t *filter_vert)
@@ -2014,7 +2014,7 @@ static void common_hv_2ht_2vt_4x8_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_bilinear4_hv_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                 uint8_t *src, ptrdiff_t src_stride,
+                                 const uint8_t *src, ptrdiff_t src_stride,
                                  int height, int mx, int my)
 {
     const int8_t *filter_horiz = bilinear_filters_msa[mx - 1];
@@ -2029,7 +2029,7 @@ void ff_put_vp8_bilinear4_hv_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void common_hv_2ht_2vt_8x4_msa(uint8_t *src, int32_t src_stride,
+static void common_hv_2ht_2vt_8x4_msa(const uint8_t *src, int32_t src_stride,
                                       uint8_t *dst, int32_t dst_stride,
                                       const int8_t *filter_horiz,
                                       const int8_t *filter_vert)
@@ -2073,7 +2073,7 @@ static void common_hv_2ht_2vt_8x4_msa(uint8_t *src, int32_t src_stride,
     ST_D4(out0, out1, 0, 1, 0, 1, dst, dst_stride);
 }
 
-static void common_hv_2ht_2vt_8x8mult_msa(uint8_t *src, int32_t src_stride,
+static void common_hv_2ht_2vt_8x8mult_msa(const uint8_t *src, int32_t src_stride,
                                           uint8_t *dst, int32_t dst_stride,
                                           const int8_t *filter_horiz,
                                           const int8_t *filter_vert,
@@ -2154,7 +2154,7 @@ static void common_hv_2ht_2vt_8x8mult_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_bilinear8_hv_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                 uint8_t *src, ptrdiff_t src_stride,
+                                 const uint8_t *src, ptrdiff_t src_stride,
                                  int height, int mx, int my)
 {
     const int8_t *filter_horiz = bilinear_filters_msa[mx - 1];
@@ -2170,7 +2170,7 @@ void ff_put_vp8_bilinear8_hv_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_bilinear16_hv_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                                  uint8_t *src, ptrdiff_t src_stride,
+                                  const uint8_t *src, ptrdiff_t src_stride,
                                   int height, int mx, int my)
 {
     uint32_t loop_cnt;
@@ -2241,7 +2241,7 @@ void ff_put_vp8_bilinear16_hv_msa(uint8_t *dst, ptrdiff_t dst_stride,
 }
 
 void ff_put_vp8_pixels8_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                            uint8_t *src, ptrdiff_t src_stride,
+                            const uint8_t *src, ptrdiff_t src_stride,
                             int height, int mx, int my)
 {
     int32_t cnt;
@@ -2283,16 +2283,16 @@ void ff_put_vp8_pixels8_msa(uint8_t *dst, ptrdiff_t dst_stride,
     }
 }
 
-static void copy_16multx8mult_msa(uint8_t *src, int32_t src_stride,
+static void copy_16multx8mult_msa(const uint8_t *src, int32_t src_stride,
                                   uint8_t *dst, int32_t dst_stride,
                                   int32_t height, int32_t width)
 {
     int32_t cnt, loop_cnt;
-    uint8_t *src_tmp, *dst_tmp;
+    uint8_t *dst_tmp;
     v16u8 src0, src1, src2, src3, src4, src5, src6, src7;
 
     for (cnt = (width >> 4); cnt--;) {
-        src_tmp = src;
+        const uint8_t *src_tmp = src;
         dst_tmp = dst;
 
         for (loop_cnt = (height >> 3); loop_cnt--;) {
@@ -2311,7 +2311,7 @@ static void copy_16multx8mult_msa(uint8_t *src, int32_t src_stride,
 }
 
 void ff_put_vp8_pixels16_msa(uint8_t *dst, ptrdiff_t dst_stride,
-                            uint8_t *src, ptrdiff_t src_stride,
+                             const uint8_t *src, ptrdiff_t src_stride,
                             int height, int mx, int my)
 {
     int32_t cnt;
