@@ -116,9 +116,9 @@ static av_cold void build_vlc(VLC *vlc, int nb_bits, int nb_codes,
 {
     vlc->table           = &qdm2_table[*offset];
     vlc->table_allocated = FF_ARRAY_ELEMS(qdm2_table) - *offset;
-    ff_init_vlc_from_lengths(vlc, nb_bits, nb_codes,
+    ff_vlc_init_from_lengths(vlc, nb_bits, nb_codes,
                              &tab[0][1], 2, &tab[0][0], 2, 1,
-                             -1, INIT_VLC_STATIC_OVERLONG | INIT_VLC_LE, NULL);
+                             -1, VLC_INIT_STATIC_OVERLONG | VLC_INIT_LE, NULL);
     *offset += vlc->table_size;
 }
 
