@@ -23,6 +23,7 @@
 
 #include "libavutil/avassert.h"
 #include "libavutil/common.h"
+#include "libavutil/mem.h"
 #include "libavutil/pixdesc.h"
 #include "libavutil/opt.h"
 #include "libavutil/mastering_display_metadata.h"
@@ -31,14 +32,14 @@
 #include "avcodec.h"
 #include "cbs.h"
 #include "cbs_h265.h"
+#include "hw_base_encode_h265.h"
 #include "codec_internal.h"
 #include "h2645data.h"
 #include "h265_profile_level.h"
-#include "hevc.h"
-#include "hevc_sei.h"
-#include "put_bits.h"
 #include "vaapi_encode.h"
 
+#include "hevc/hevc.h"
+
 enum {
     SEI_MASTERING_DISPLAY       = 0x08,
     SEI_CONTENT_LIGHT_LEVEL     = 0x10,
@@ -69,20 +70,17 @@ typedef struct VAAPIEncodeH265Context {
     int qp;
     int aud;
     int profile;
-    int tier;
     int level;
     int sei;
 
     // Derived settings.
-    int fixed_qp_idr;
     int fixed_qp_p;
     int fixed_qp_b;
 
     // Writer structures.
+    FFHWBaseEncodeH265 units;
+    FFHWBaseEncodeH265Opts unit_opts;
     H265RawAUD   raw_aud;
-    H265RawVPS   raw_vps;
-    H265RawSPS   raw_sps;
-    H265RawPPS   raw_pps;
     H265RawSlice raw_slice;
 
     SEIRawMasteringDisplayColourVolume sei_mastering_display;
@@ -155,15 +153,15 @@ static int vaapi_encode_h265_write_sequence_header(AVCodecContext *avctx,
         priv->aud_needed = 0;
     }
 
-    err = vaapi_encode_h265_add_nal(avctx, au, &priv->raw_vps);
+    err = vaapi_encode_h265_add_nal(avctx, au, &priv->units.raw_vps);
     if (err < 0)
         goto fail;
 
-    err = vaapi_encode_h265_add_nal(avctx, au, &priv->raw_sps);
+    err = vaapi_encode_h265_add_nal(avctx, au, &priv->units.raw_sps);
     if (err < 0)
         goto fail;
 
-    err = vaapi_encode_h265_add_nal(avctx, au, &priv->raw_pps);
+    err = vaapi_encode_h265_add_nal(avctx, au, &priv->units.raw_pps);
     if (err < 0)
         goto fail;
 
@@ -200,7 +198,7 @@ fail:
 }
 
 static int vaapi_encode_h265_write_extra_header(AVCodecContext *avctx,
-                                                VAAPIEncodePicture *pic,
+                                                FFHWBaseEncodePicture *base,
                                                 int index, int *type,
                                                 char *data, size_t *data_len)
 {
@@ -260,210 +258,36 @@ fail:
 
 static int vaapi_encode_h265_init_sequence_params(AVCodecContext *avctx)
 {
+    FFHWBaseEncodeContext        *base_ctx = avctx->priv_data;
     VAAPIEncodeContext                *ctx = avctx->priv_data;
     VAAPIEncodeH265Context           *priv = avctx->priv_data;
-    H265RawVPS                        *vps = &priv->raw_vps;
-    H265RawSPS                        *sps = &priv->raw_sps;
-    H265RawPPS                        *pps = &priv->raw_pps;
-    H265RawProfileTierLevel           *ptl = &vps->profile_tier_level;
-    H265RawVUI                        *vui = &sps->vui;
+    H265RawVPS                        *vps = &priv->units.raw_vps;
+    H265RawSPS                        *sps = &priv->units.raw_sps;
+    H265RawPPS                        *pps = &priv->units.raw_pps;
     VAEncSequenceParameterBufferHEVC *vseq = ctx->codec_sequence_params;
     VAEncPictureParameterBufferHEVC  *vpic = ctx->codec_picture_params;
-    const AVPixFmtDescriptor *desc;
-    int chroma_format, bit_depth;
-    int i;
+    int i, err;
 
-    memset(vps, 0, sizeof(*vps));
-    memset(sps, 0, sizeof(*sps));
-    memset(pps, 0, sizeof(*pps));
+    // priv->unit_opts.tier already set
+    // priv->unit_opts.fixed_qp_idr already set
+    priv->unit_opts.cu_qp_delta_enabled_flag = (ctx->va_rc_mode != VA_RC_CQP);
+    priv->unit_opts.tile_rows = ctx->tile_rows;
+    priv->unit_opts.tile_cols = ctx->tile_cols;
+    priv->unit_opts.nb_slices = ctx->nb_slices;
+    priv->unit_opts.slice_block_rows = ctx->slice_block_rows;
+    priv->unit_opts.slice_block_cols = ctx->slice_block_cols;
+    memcpy(priv->unit_opts.col_width, ctx->col_width,
+           ctx->tile_rows*sizeof(*priv->unit_opts.col_width));
+    memcpy(priv->unit_opts.row_height, ctx->row_height,
+           ctx->tile_cols*sizeof(*priv->unit_opts.row_height));
 
+    err = ff_hw_base_encode_init_params_h265(base_ctx, avctx,
+                                             &priv->units, &priv->unit_opts);
+    if (err < 0)
+        return err;
 
-    desc = av_pix_fmt_desc_get(priv->common.input_frames->sw_format);
-    av_assert0(desc);
-    if (desc->nb_components == 1) {
-        chroma_format = 0;
-    } else {
-        if (desc->log2_chroma_w == 1 && desc->log2_chroma_h == 1) {
-            chroma_format = 1;
-        } else if (desc->log2_chroma_w == 1 && desc->log2_chroma_h == 0) {
-            chroma_format = 2;
-        } else if (desc->log2_chroma_w == 0 && desc->log2_chroma_h == 0) {
-            chroma_format = 3;
-        } else {
-            av_log(avctx, AV_LOG_ERROR, "Chroma format of input pixel format "
-                   "%s is not supported.\n", desc->name);
-            return AVERROR(EINVAL);
-        }
-    }
-    bit_depth = desc->comp[0].depth;
-
-
-    // VPS
-
-    vps->nal_unit_header = (H265RawNALUnitHeader) {
-        .nal_unit_type         = HEVC_NAL_VPS,
-        .nuh_layer_id          = 0,
-        .nuh_temporal_id_plus1 = 1,
-    };
-
-    vps->vps_video_parameter_set_id = 0;
-
-    vps->vps_base_layer_internal_flag  = 1;
-    vps->vps_base_layer_available_flag = 1;
-    vps->vps_max_layers_minus1         = 0;
-    vps->vps_max_sub_layers_minus1     = 0;
-    vps->vps_temporal_id_nesting_flag  = 1;
-
-    ptl->general_profile_space = 0;
-    ptl->general_profile_idc   = avctx->profile;
-    ptl->general_tier_flag     = priv->tier;
-
-    ptl->general_profile_compatibility_flag[ptl->general_profile_idc] = 1;
-
-    if (ptl->general_profile_compatibility_flag[1])
-        ptl->general_profile_compatibility_flag[2] = 1;
-    if (ptl->general_profile_compatibility_flag[3]) {
-        ptl->general_profile_compatibility_flag[1] = 1;
-        ptl->general_profile_compatibility_flag[2] = 1;
-    }
-
-    ptl->general_progressive_source_flag    = 1;
-    ptl->general_interlaced_source_flag     = 0;
-    ptl->general_non_packed_constraint_flag = 1;
-    ptl->general_frame_only_constraint_flag = 1;
-
-    ptl->general_max_14bit_constraint_flag = bit_depth <= 14;
-    ptl->general_max_12bit_constraint_flag = bit_depth <= 12;
-    ptl->general_max_10bit_constraint_flag = bit_depth <= 10;
-    ptl->general_max_8bit_constraint_flag  = bit_depth ==  8;
-
-    ptl->general_max_422chroma_constraint_flag  = chroma_format <= 2;
-    ptl->general_max_420chroma_constraint_flag  = chroma_format <= 1;
-    ptl->general_max_monochrome_constraint_flag = chroma_format == 0;
-
-    ptl->general_intra_constraint_flag = ctx->gop_size == 1;
-    ptl->general_one_picture_only_constraint_flag = 0;
-
-    ptl->general_lower_bit_rate_constraint_flag = 1;
-
-    if (avctx->level != AV_LEVEL_UNKNOWN) {
-        ptl->general_level_idc = avctx->level;
-    } else {
-        const H265LevelDescriptor *level;
-
-        level = ff_h265_guess_level(ptl, avctx->bit_rate,
-                                    ctx->surface_width, ctx->surface_height,
-                                    ctx->nb_slices, ctx->tile_rows, ctx->tile_cols,
-                                    (ctx->b_per_p > 0) + 1);
-        if (level) {
-            av_log(avctx, AV_LOG_VERBOSE, "Using level %s.\n", level->name);
-            ptl->general_level_idc = level->level_idc;
-        } else {
-            av_log(avctx, AV_LOG_VERBOSE, "Stream will not conform to "
-                   "any normal level; using level 8.5.\n");
-            ptl->general_level_idc = 255;
-            // The tier flag must be set in level 8.5.
-            ptl->general_tier_flag = 1;
-        }
-    }
-
-    vps->vps_sub_layer_ordering_info_present_flag = 0;
-    vps->vps_max_dec_pic_buffering_minus1[0]      = ctx->max_b_depth + 1;
-    vps->vps_max_num_reorder_pics[0]              = ctx->max_b_depth;
-    vps->vps_max_latency_increase_plus1[0]        = 0;
-
-    vps->vps_max_layer_id             = 0;
-    vps->vps_num_layer_sets_minus1    = 0;
-    vps->layer_id_included_flag[0][0] = 1;
-
-    vps->vps_timing_info_present_flag = 1;
-    if (avctx->framerate.num > 0 && avctx->framerate.den > 0) {
-        vps->vps_num_units_in_tick  = avctx->framerate.den;
-        vps->vps_time_scale         = avctx->framerate.num;
-        vps->vps_poc_proportional_to_timing_flag = 1;
-        vps->vps_num_ticks_poc_diff_one_minus1   = 0;
-    } else {
-        vps->vps_num_units_in_tick  = avctx->time_base.num;
-        vps->vps_time_scale         = avctx->time_base.den;
-        vps->vps_poc_proportional_to_timing_flag = 0;
-    }
-    vps->vps_num_hrd_parameters = 0;
-
-
-    // SPS
-
-    sps->nal_unit_header = (H265RawNALUnitHeader) {
-        .nal_unit_type         = HEVC_NAL_SPS,
-        .nuh_layer_id          = 0,
-        .nuh_temporal_id_plus1 = 1,
-    };
-
-    sps->sps_video_parameter_set_id = vps->vps_video_parameter_set_id;
-
-    sps->sps_max_sub_layers_minus1    = vps->vps_max_sub_layers_minus1;
-    sps->sps_temporal_id_nesting_flag = vps->vps_temporal_id_nesting_flag;
-
-    sps->profile_tier_level = vps->profile_tier_level;
-
-    sps->sps_seq_parameter_set_id = 0;
-
-    sps->chroma_format_idc          = chroma_format;
-    sps->separate_colour_plane_flag = 0;
-
-    sps->pic_width_in_luma_samples  = ctx->surface_width;
-    sps->pic_height_in_luma_samples = ctx->surface_height;
-
-    if (avctx->width  != ctx->surface_width ||
-        avctx->height != ctx->surface_height) {
-        sps->conformance_window_flag = 1;
-        sps->conf_win_left_offset   = 0;
-        sps->conf_win_right_offset  =
-            (ctx->surface_width - avctx->width) >> desc->log2_chroma_w;
-        sps->conf_win_top_offset    = 0;
-        sps->conf_win_bottom_offset =
-            (ctx->surface_height - avctx->height) >> desc->log2_chroma_h;
-    } else {
-        sps->conformance_window_flag = 0;
-    }
-
-    sps->bit_depth_luma_minus8   = bit_depth - 8;
-    sps->bit_depth_chroma_minus8 = bit_depth - 8;
-
-    sps->log2_max_pic_order_cnt_lsb_minus4 = 8;
-
-    sps->sps_sub_layer_ordering_info_present_flag =
-        vps->vps_sub_layer_ordering_info_present_flag;
-    for (i = 0; i <= sps->sps_max_sub_layers_minus1; i++) {
-        sps->sps_max_dec_pic_buffering_minus1[i] =
-            vps->vps_max_dec_pic_buffering_minus1[i];
-        sps->sps_max_num_reorder_pics[i] =
-            vps->vps_max_num_reorder_pics[i];
-        sps->sps_max_latency_increase_plus1[i] =
-            vps->vps_max_latency_increase_plus1[i];
-    }
-
-    // These values come from the capabilities of the first encoder
-    // implementation in the i965 driver on Intel Skylake.  They may
-    // fail badly with other platforms or drivers.
-    // CTB size from 8x8 to 32x32.
-    sps->log2_min_luma_coding_block_size_minus3   = 0;
-    sps->log2_diff_max_min_luma_coding_block_size = 2;
-    // Transform size from 4x4 to 32x32.
-    sps->log2_min_luma_transform_block_size_minus2   = 0;
-    sps->log2_diff_max_min_luma_transform_block_size = 3;
-    // Full transform hierarchy allowed (2-5).
-    sps->max_transform_hierarchy_depth_inter = 3;
-    sps->max_transform_hierarchy_depth_intra = 3;
-    // AMP works.
-    sps->amp_enabled_flag = 1;
-    // SAO and temporal MVP do not work.
-    sps->sample_adaptive_offset_enabled_flag = 0;
-    sps->sps_temporal_mvp_enabled_flag       = 0;
-
-    sps->pcm_enabled_flag = 0;
-
-// update sps setting according to queried result
 #if VA_CHECK_VERSION(1, 13, 0)
+    // update sps setting according to queried result
     if (priv->va_features) {
         VAConfigAttribValEncHEVCFeatures features = { .value = priv->va_features };
 
@@ -496,95 +320,8 @@ static int vaapi_encode_h265_init_sequence_params(AVCodecContext *avctx)
         sps->max_transform_hierarchy_depth_intra =
             bs.bits.max_max_transform_hierarchy_depth_intra;
     }
-#endif
 
-    // STRPSs should ideally be here rather than defined individually in
-    // each slice, but the structure isn't completely fixed so for now
-    // don't bother.
-    sps->num_short_term_ref_pic_sets     = 0;
-    sps->long_term_ref_pics_present_flag = 0;
-
-    sps->vui_parameters_present_flag = 1;
-
-    if (avctx->sample_aspect_ratio.num != 0 &&
-        avctx->sample_aspect_ratio.den != 0) {
-        int num, den, i;
-        av_reduce(&num, &den, avctx->sample_aspect_ratio.num,
-                  avctx->sample_aspect_ratio.den, 65535);
-        for (i = 0; i < FF_ARRAY_ELEMS(ff_h2645_pixel_aspect); i++) {
-            if (num == ff_h2645_pixel_aspect[i].num &&
-                den == ff_h2645_pixel_aspect[i].den) {
-                vui->aspect_ratio_idc = i;
-                break;
-            }
-        }
-        if (i >= FF_ARRAY_ELEMS(ff_h2645_pixel_aspect)) {
-            vui->aspect_ratio_idc = 255;
-            vui->sar_width  = num;
-            vui->sar_height = den;
-        }
-        vui->aspect_ratio_info_present_flag = 1;
-    }
-
-    // Unspecified video format, from table E-2.
-    vui->video_format             = 5;
-    vui->video_full_range_flag    =
-        avctx->color_range == AVCOL_RANGE_JPEG;
-    vui->colour_primaries         = avctx->color_primaries;
-    vui->transfer_characteristics = avctx->color_trc;
-    vui->matrix_coefficients      = avctx->colorspace;
-    if (avctx->color_primaries != AVCOL_PRI_UNSPECIFIED ||
-        avctx->color_trc       != AVCOL_TRC_UNSPECIFIED ||
-        avctx->colorspace      != AVCOL_SPC_UNSPECIFIED)
-        vui->colour_description_present_flag = 1;
-    if (avctx->color_range     != AVCOL_RANGE_UNSPECIFIED ||
-        vui->colour_description_present_flag)
-        vui->video_signal_type_present_flag = 1;
-
-    if (avctx->chroma_sample_location != AVCHROMA_LOC_UNSPECIFIED) {
-        vui->chroma_loc_info_present_flag = 1;
-        vui->chroma_sample_loc_type_top_field    =
-        vui->chroma_sample_loc_type_bottom_field =
-            avctx->chroma_sample_location - 1;
-    }
-
-    vui->vui_timing_info_present_flag        = 1;
-    vui->vui_num_units_in_tick               = vps->vps_num_units_in_tick;
-    vui->vui_time_scale                      = vps->vps_time_scale;
-    vui->vui_poc_proportional_to_timing_flag = vps->vps_poc_proportional_to_timing_flag;
-    vui->vui_num_ticks_poc_diff_one_minus1   = vps->vps_num_ticks_poc_diff_one_minus1;
-    vui->vui_hrd_parameters_present_flag     = 0;
-
-    vui->bitstream_restriction_flag    = 1;
-    vui->motion_vectors_over_pic_boundaries_flag = 1;
-    vui->restricted_ref_pic_lists_flag = 1;
-    vui->max_bytes_per_pic_denom       = 0;
-    vui->max_bits_per_min_cu_denom     = 0;
-    vui->log2_max_mv_length_horizontal = 15;
-    vui->log2_max_mv_length_vertical   = 15;
-
-
-    // PPS
-
-    pps->nal_unit_header = (H265RawNALUnitHeader) {
-        .nal_unit_type         = HEVC_NAL_PPS,
-        .nuh_layer_id          = 0,
-        .nuh_temporal_id_plus1 = 1,
-    };
-
-    pps->pps_pic_parameter_set_id = 0;
-    pps->pps_seq_parameter_set_id = sps->sps_seq_parameter_set_id;
-
-    pps->num_ref_idx_l0_default_active_minus1 = 0;
-    pps->num_ref_idx_l1_default_active_minus1 = 0;
-
-    pps->init_qp_minus26 = priv->fixed_qp_idr - 26;
-
-    pps->cu_qp_delta_enabled_flag = (ctx->va_rc_mode != VA_RC_CQP);
-    pps->diff_cu_qp_delta_depth   = 0;
-
-// update pps setting according to queried result
-#if VA_CHECK_VERSION(1, 13, 0)
+    // update pps setting according to queried result
     if (priv->va_features) {
         VAConfigAttribValEncHEVCFeatures features = { .value = priv->va_features };
         if (ctx->va_rc_mode != VA_RC_CQP)
@@ -600,42 +337,6 @@ static int vaapi_encode_h265_init_sequence_params(AVCodecContext *avctx)
     }
 #endif
 
-    if (ctx->tile_rows && ctx->tile_cols) {
-        int uniform_spacing;
-
-        pps->tiles_enabled_flag      = 1;
-        pps->num_tile_columns_minus1 = ctx->tile_cols - 1;
-        pps->num_tile_rows_minus1    = ctx->tile_rows - 1;
-
-        // Test whether the spacing provided matches the H.265 uniform
-        // spacing, and set the flag if it does.
-        uniform_spacing = 1;
-        for (i = 0; i <= pps->num_tile_columns_minus1 &&
-                    uniform_spacing; i++) {
-            if (ctx->col_width[i] !=
-                (i + 1) * ctx->slice_block_cols / ctx->tile_cols -
-                 i      * ctx->slice_block_cols / ctx->tile_cols)
-                uniform_spacing = 0;
-        }
-        for (i = 0; i <= pps->num_tile_rows_minus1 &&
-                    uniform_spacing; i++) {
-            if (ctx->row_height[i] !=
-                (i + 1) * ctx->slice_block_rows / ctx->tile_rows -
-                 i      * ctx->slice_block_rows / ctx->tile_rows)
-                uniform_spacing = 0;
-        }
-        pps->uniform_spacing_flag = uniform_spacing;
-
-        for (i = 0; i <= pps->num_tile_columns_minus1; i++)
-            pps->column_width_minus1[i] = ctx->col_width[i] - 1;
-        for (i = 0; i <= pps->num_tile_rows_minus1; i++)
-            pps->row_height_minus1[i]   = ctx->row_height[i] - 1;
-
-        pps->loop_filter_across_tiles_enabled_flag = 1;
-    }
-
-    pps->pps_loop_filter_across_slices_enabled_flag = 1;
-
     // Fill VAAPI parameter buffers.
 
     *vseq = (VAEncSequenceParameterBufferHEVC) {
@@ -643,9 +344,9 @@ static int vaapi_encode_h265_init_sequence_params(AVCodecContext *avctx)
         .general_level_idc   = vps->profile_tier_level.general_level_idc,
         .general_tier_flag   = vps->profile_tier_level.general_tier_flag,
 
-        .intra_period     = ctx->gop_size,
-        .intra_idr_period = ctx->gop_size,
-        .ip_period        = ctx->b_per_p + 1,
+        .intra_period     = base_ctx->gop_size,
+        .intra_idr_period = base_ctx->gop_size,
+        .ip_period        = base_ctx->b_per_p + 1,
         .bits_per_second  = ctx->va_bit_rate,
 
         .pic_width_in_luma_samples  = sps->pic_width_in_luma_samples,
@@ -756,17 +457,18 @@ static int vaapi_encode_h265_init_sequence_params(AVCodecContext *avctx)
 }
 
 static int vaapi_encode_h265_init_picture_params(AVCodecContext *avctx,
-                                                 VAAPIEncodePicture *pic)
+                                                 FFHWBaseEncodePicture *pic)
 {
-    VAAPIEncodeContext               *ctx = avctx->priv_data;
+    FFHWBaseEncodeContext       *base_ctx = avctx->priv_data;
     VAAPIEncodeH265Context          *priv = avctx->priv_data;
-    VAAPIEncodeH265Picture          *hpic = pic->priv_data;
-    VAAPIEncodePicture              *prev = pic->prev;
-    VAAPIEncodeH265Picture         *hprev = prev ? prev->priv_data : NULL;
-    VAEncPictureParameterBufferHEVC *vpic = pic->codec_picture_params;
+    VAAPIEncodePicture         *vaapi_pic = pic->priv;
+    VAAPIEncodeH265Picture          *hpic = pic->codec_priv;
+    FFHWBaseEncodePicture           *prev = pic->prev;
+    VAAPIEncodeH265Picture         *hprev = prev ? prev->codec_priv : NULL;
+    VAEncPictureParameterBufferHEVC *vpic = vaapi_pic->codec_picture_params;
     int i, j = 0;
 
-    if (pic->type == PICTURE_TYPE_IDR) {
+    if (pic->type == FF_HW_PICTURE_TYPE_IDR) {
         av_assert0(pic->display_order == pic->encode_order);
 
         hpic->last_idr_frame = pic->display_order;
@@ -778,23 +480,23 @@ static int vaapi_encode_h265_init_picture_params(AVCodecContext *avctx,
         av_assert0(prev);
         hpic->last_idr_frame = hprev->last_idr_frame;
 
-        if (pic->type == PICTURE_TYPE_I) {
+        if (pic->type == FF_HW_PICTURE_TYPE_I) {
             hpic->slice_nal_unit = HEVC_NAL_CRA_NUT;
             hpic->slice_type     = HEVC_SLICE_I;
             hpic->pic_type       = 0;
-        } else if (pic->type == PICTURE_TYPE_P) {
+        } else if (pic->type == FF_HW_PICTURE_TYPE_P) {
             av_assert0(pic->refs[0]);
             hpic->slice_nal_unit = HEVC_NAL_TRAIL_R;
             hpic->slice_type     = HEVC_SLICE_P;
             hpic->pic_type       = 1;
         } else {
-            VAAPIEncodePicture *irap_ref;
+            FFHWBaseEncodePicture *irap_ref;
             av_assert0(pic->refs[0][0] && pic->refs[1][0]);
             for (irap_ref = pic; irap_ref; irap_ref = irap_ref->refs[1][0]) {
-                if (irap_ref->type == PICTURE_TYPE_I)
+                if (irap_ref->type == FF_HW_PICTURE_TYPE_I)
                     break;
             }
-            if (pic->b_depth == ctx->max_b_depth) {
+            if (pic->b_depth == base_ctx->max_b_depth) {
                 hpic->slice_nal_unit = irap_ref ? HEVC_NAL_RASL_N
                                                 : HEVC_NAL_TRAIL_N;
             } else {
@@ -827,7 +529,7 @@ static int vaapi_encode_h265_init_picture_params(AVCodecContext *avctx,
     // may force an IDR frame on the output where the medadata gets
     // changed on the input frame.
     if ((priv->sei & SEI_MASTERING_DISPLAY) &&
-        (pic->type == PICTURE_TYPE_I || pic->type == PICTURE_TYPE_IDR)) {
+        (pic->type == FF_HW_PICTURE_TYPE_I || pic->type == FF_HW_PICTURE_TYPE_IDR)) {
         AVFrameSideData *sd =
             av_frame_get_side_data(pic->input_image,
                                    AV_FRAME_DATA_MASTERING_DISPLAY_METADATA);
@@ -875,7 +577,7 @@ static int vaapi_encode_h265_init_picture_params(AVCodecContext *avctx,
     }
 
     if ((priv->sei & SEI_CONTENT_LIGHT_LEVEL) &&
-        (pic->type == PICTURE_TYPE_I || pic->type == PICTURE_TYPE_IDR)) {
+        (pic->type == FF_HW_PICTURE_TYPE_I || pic->type == FF_HW_PICTURE_TYPE_IDR)) {
         AVFrameSideData *sd =
             av_frame_get_side_data(pic->input_image,
                                    AV_FRAME_DATA_CONTENT_LIGHT_LEVEL);
@@ -910,21 +612,21 @@ static int vaapi_encode_h265_init_picture_params(AVCodecContext *avctx,
     }
 
     vpic->decoded_curr_pic = (VAPictureHEVC) {
-        .picture_id    = pic->recon_surface,
+        .picture_id    = vaapi_pic->recon_surface,
         .pic_order_cnt = hpic->pic_order_cnt,
         .flags         = 0,
     };
 
     for (int k = 0; k < MAX_REFERENCE_LIST_NUM; k++) {
         for (i = 0; i < pic->nb_refs[k]; i++) {
-            VAAPIEncodePicture      *ref = pic->refs[k][i];
+            FFHWBaseEncodePicture *ref = pic->refs[k][i];
             VAAPIEncodeH265Picture *href;
 
             av_assert0(ref && ref->encode_order < pic->encode_order);
-            href = ref->priv_data;
+            href = ref->codec_priv;
 
             vpic->reference_frames[j++] = (VAPictureHEVC) {
-                .picture_id    = ref->recon_surface,
+                .picture_id    = ((VAAPIEncodePicture *)ref->priv)->recon_surface,
                 .pic_order_cnt = href->pic_order_cnt,
                 .flags = (ref->display_order < pic->display_order ?
                           VA_PICTURE_HEVC_RPS_ST_CURR_BEFORE : 0) |
@@ -941,30 +643,27 @@ static int vaapi_encode_h265_init_picture_params(AVCodecContext *avctx,
         };
     }
 
-    vpic->coded_buf = pic->output_buffer;
+    vpic->coded_buf = vaapi_pic->output_buffer;
 
     vpic->nal_unit_type = hpic->slice_nal_unit;
 
+    vpic->pic_fields.bits.reference_pic_flag = pic->is_reference;
     switch (pic->type) {
-    case PICTURE_TYPE_IDR:
+    case FF_HW_PICTURE_TYPE_IDR:
         vpic->pic_fields.bits.idr_pic_flag       = 1;
         vpic->pic_fields.bits.coding_type        = 1;
-        vpic->pic_fields.bits.reference_pic_flag = 1;
         break;
-    case PICTURE_TYPE_I:
+    case FF_HW_PICTURE_TYPE_I:
         vpic->pic_fields.bits.idr_pic_flag       = 0;
         vpic->pic_fields.bits.coding_type        = 1;
-        vpic->pic_fields.bits.reference_pic_flag = 1;
         break;
-    case PICTURE_TYPE_P:
+    case FF_HW_PICTURE_TYPE_P:
         vpic->pic_fields.bits.idr_pic_flag       = 0;
         vpic->pic_fields.bits.coding_type        = 2;
-        vpic->pic_fields.bits.reference_pic_flag = 1;
         break;
-    case PICTURE_TYPE_B:
+    case FF_HW_PICTURE_TYPE_B:
         vpic->pic_fields.bits.idr_pic_flag       = 0;
         vpic->pic_fields.bits.coding_type        = 3;
-        vpic->pic_fields.bits.reference_pic_flag = 0;
         break;
     default:
         av_assert0(0 && "invalid picture type");
@@ -974,16 +673,17 @@ static int vaapi_encode_h265_init_picture_params(AVCodecContext *avctx,
 }
 
 static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
-                                               VAAPIEncodePicture *pic,
+                                               FFHWBaseEncodePicture *pic,
                                                VAAPIEncodeSlice *slice)
 {
-    VAAPIEncodeContext                *ctx = avctx->priv_data;
+    FFHWBaseEncodeContext        *base_ctx = avctx->priv_data;
     VAAPIEncodeH265Context           *priv = avctx->priv_data;
-    VAAPIEncodeH265Picture           *hpic = pic->priv_data;
-    const H265RawSPS                  *sps = &priv->raw_sps;
-    const H265RawPPS                  *pps = &priv->raw_pps;
+    VAAPIEncodePicture          *vaapi_pic = pic->priv;
+    VAAPIEncodeH265Picture           *hpic = pic->codec_priv;
+    const H265RawSPS                  *sps = &priv->units.raw_sps;
+    const H265RawPPS                  *pps = &priv->units.raw_pps;
     H265RawSliceHeader                 *sh = &priv->raw_slice.header;
-    VAEncPictureParameterBufferHEVC  *vpic = pic->codec_picture_params;
+    VAEncPictureParameterBufferHEVC  *vpic = vaapi_pic->codec_picture_params;
     VAEncSliceParameterBufferHEVC  *vslice = slice->codec_slice_params;
     int i;
 
@@ -1000,13 +700,13 @@ static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
 
     sh->slice_type = hpic->slice_type;
 
-    if (sh->slice_type == HEVC_SLICE_P && ctx->p_to_gpb)
+    if (sh->slice_type == HEVC_SLICE_P && base_ctx->p_to_gpb)
         sh->slice_type = HEVC_SLICE_B;
 
     sh->slice_pic_order_cnt_lsb = hpic->pic_order_cnt &
         (1 << (sps->log2_max_pic_order_cnt_lsb_minus4 + 4)) - 1;
 
-    if (pic->type != PICTURE_TYPE_IDR) {
+    if (pic->type != FF_HW_PICTURE_TYPE_IDR) {
         H265RawSTRefPicSet *rps;
         const VAAPIEncodeH265Picture *strp;
         int rps_poc[MAX_DPB_SIZE];
@@ -1021,7 +721,7 @@ static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
         rps_pics = 0;
         for (i = 0; i < MAX_REFERENCE_LIST_NUM; i++) {
             for (j = 0; j < pic->nb_refs[i]; j++) {
-                strp = pic->refs[i][j]->priv_data;
+                strp = pic->refs[i][j]->codec_priv;
                 rps_poc[rps_pics]  = strp->pic_order_cnt;
                 rps_used[rps_pics] = 1;
                 ++rps_pics;
@@ -1046,7 +746,7 @@ static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
             if (j < pic->nb_refs[1])
                 continue;
 
-            strp = pic->dpb[i]->priv_data;
+            strp = pic->dpb[i]->codec_priv;
             rps_poc[rps_pics]  = strp->pic_order_cnt;
             rps_used[rps_pics] = 0;
             ++rps_pics;
@@ -1113,12 +813,12 @@ static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
     sh->slice_sao_luma_flag = sh->slice_sao_chroma_flag =
         sps->sample_adaptive_offset_enabled_flag;
 
-    if (pic->type == PICTURE_TYPE_B)
+    if (pic->type == FF_HW_PICTURE_TYPE_B)
         sh->slice_qp_delta = priv->fixed_qp_b - (pps->init_qp_minus26 + 26);
-    else if (pic->type == PICTURE_TYPE_P)
+    else if (pic->type == FF_HW_PICTURE_TYPE_P)
         sh->slice_qp_delta = priv->fixed_qp_p - (pps->init_qp_minus26 + 26);
     else
-        sh->slice_qp_delta = priv->fixed_qp_idr - (pps->init_qp_minus26 + 26);
+        sh->slice_qp_delta = priv->unit_opts.fixed_qp_idr - (pps->init_qp_minus26 + 26);
 
 
     *vslice = (VAEncSliceParameterBufferHEVC) {
@@ -1144,7 +844,7 @@ static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
         .slice_tc_offset_div2   = sh->slice_tc_offset_div2,
 
         .slice_fields.bits = {
-            .last_slice_of_pic_flag       = slice->index == pic->nb_slices - 1,
+            .last_slice_of_pic_flag       = slice->index == vaapi_pic->nb_slices - 1,
             .dependent_slice_segment_flag = sh->dependent_slice_segment_flag,
             .colour_plane_id              = sh->colour_plane_id,
             .slice_temporal_mvp_enabled_flag =
@@ -1172,20 +872,20 @@ static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
 
     if (pic->nb_refs[0]) {
         // Backward reference for P- or B-frame.
-        av_assert0(pic->type == PICTURE_TYPE_P ||
-                   pic->type == PICTURE_TYPE_B);
+        av_assert0(pic->type == FF_HW_PICTURE_TYPE_P ||
+                   pic->type == FF_HW_PICTURE_TYPE_B);
         vslice->ref_pic_list0[0] = vpic->reference_frames[0];
-        if (ctx->p_to_gpb && pic->type == PICTURE_TYPE_P)
+        if (base_ctx->p_to_gpb && pic->type == FF_HW_PICTURE_TYPE_P)
             // Reference for GPB B-frame, L0 == L1
             vslice->ref_pic_list1[0] = vpic->reference_frames[0];
     }
     if (pic->nb_refs[1]) {
         // Forward reference for B-frame.
-        av_assert0(pic->type == PICTURE_TYPE_B);
+        av_assert0(pic->type == FF_HW_PICTURE_TYPE_B);
         vslice->ref_pic_list1[0] = vpic->reference_frames[1];
     }
 
-    if (pic->type == PICTURE_TYPE_P && ctx->p_to_gpb) {
+    if (pic->type == FF_HW_PICTURE_TYPE_P && base_ctx->p_to_gpb) {
         vslice->slice_type = HEVC_SLICE_B;
         for (i = 0; i < FF_ARRAY_ELEMS(vslice->ref_pic_list0); i++) {
             vslice->ref_pic_list1[i].picture_id = vslice->ref_pic_list0[i].picture_id;
@@ -1198,11 +898,12 @@ static int vaapi_encode_h265_init_slice_params(AVCodecContext *avctx,
 
 static av_cold int vaapi_encode_h265_get_encoder_caps(AVCodecContext *avctx)
 {
-    VAAPIEncodeContext      *ctx = avctx->priv_data;
-    VAAPIEncodeH265Context *priv = avctx->priv_data;
+    FFHWBaseEncodeContext *base_ctx = avctx->priv_data;
+    VAAPIEncodeH265Context    *priv = avctx->priv_data;
 
 #if VA_CHECK_VERSION(1, 13, 0)
     {
+        VAAPIEncodeContext *ctx = avctx->priv_data;
         VAConfigAttribValEncHEVCBlockSizes block_size;
         VAConfigAttrib attr;
         VAStatus vas;
@@ -1250,10 +951,10 @@ static av_cold int vaapi_encode_h265_get_encoder_caps(AVCodecContext *avctx)
            "min CB size %dx%d.\n", priv->ctu_size, priv->ctu_size,
            priv->min_cb_size, priv->min_cb_size);
 
-    ctx->surface_width  = FFALIGN(avctx->width,  priv->min_cb_size);
-    ctx->surface_height = FFALIGN(avctx->height, priv->min_cb_size);
+    base_ctx->surface_width  = FFALIGN(avctx->width,  priv->min_cb_size);
+    base_ctx->surface_height = FFALIGN(avctx->height, priv->min_cb_size);
 
-    ctx->slice_block_width = ctx->slice_block_height = priv->ctu_size;
+    base_ctx->slice_block_width = base_ctx->slice_block_height = priv->ctu_size;
 
     return 0;
 }
@@ -1275,11 +976,11 @@ static av_cold int vaapi_encode_h265_configure(AVCodecContext *avctx)
 
         priv->fixed_qp_p = av_clip(ctx->rc_quality, 1, 51);
         if (avctx->i_quant_factor > 0.0)
-            priv->fixed_qp_idr =
+            priv->unit_opts.fixed_qp_idr =
                 av_clip((avctx->i_quant_factor * priv->fixed_qp_p +
                          avctx->i_quant_offset) + 0.5, 1, 51);
         else
-            priv->fixed_qp_idr = priv->fixed_qp_p;
+            priv->unit_opts.fixed_qp_idr = priv->fixed_qp_p;
         if (avctx->b_quant_factor > 0.0)
             priv->fixed_qp_b =
                 av_clip((avctx->b_quant_factor * priv->fixed_qp_p +
@@ -1289,11 +990,11 @@ static av_cold int vaapi_encode_h265_configure(AVCodecContext *avctx)
 
         av_log(avctx, AV_LOG_DEBUG, "Using fixed QP = "
                "%d / %d / %d for IDR- / P- / B-frames.\n",
-               priv->fixed_qp_idr, priv->fixed_qp_p, priv->fixed_qp_b);
+               priv->unit_opts.fixed_qp_idr, priv->fixed_qp_p, priv->fixed_qp_b);
 
     } else {
         // These still need to be set for init_qp/slice_qp_delta.
-        priv->fixed_qp_idr = 30;
+        priv->unit_opts.fixed_qp_idr = 30;
         priv->fixed_qp_p   = 30;
         priv->fixed_qp_b   = 30;
     }
@@ -1325,10 +1026,10 @@ static const VAAPIEncodeProfile vaapi_encode_h265_profiles[] = {
 static const VAAPIEncodeType vaapi_encode_type_h265 = {
     .profiles              = vaapi_encode_h265_profiles,
 
-    .flags                 = FLAG_SLICE_CONTROL |
-                             FLAG_B_PICTURES |
-                             FLAG_B_PICTURE_REFERENCES |
-                             FLAG_NON_IDR_KEY_PICTURES,
+    .flags                 = FF_HW_FLAG_SLICE_CONTROL |
+                             FF_HW_FLAG_B_PICTURES |
+                             FF_HW_FLAG_B_PICTURE_REFERENCES |
+                             FF_HW_FLAG_NON_IDR_KEY_PICTURES,
 
     .default_quality       = 25,
 
@@ -1398,6 +1099,7 @@ static av_cold int vaapi_encode_h265_close(AVCodecContext *avctx)
 #define OFFSET(x) offsetof(VAAPIEncodeH265Context, x)
 #define FLAGS (AV_OPT_FLAG_VIDEO_PARAM | AV_OPT_FLAG_ENCODING_PARAM)
 static const AVOption vaapi_encode_h265_options[] = {
+    HW_BASE_ENCODE_COMMON_OPTIONS,
     VAAPI_ENCODE_COMMON_OPTIONS,
     VAAPI_ENCODE_RC_OPTIONS,
 
@@ -1409,29 +1111,29 @@ static const AVOption vaapi_encode_h265_options[] = {
 
     { "profile", "Set profile (general_profile_idc)",
       OFFSET(profile), AV_OPT_TYPE_INT,
-      { .i64 = AV_PROFILE_UNKNOWN }, AV_PROFILE_UNKNOWN, 0xff, FLAGS, "profile" },
+      { .i64 = AV_PROFILE_UNKNOWN }, AV_PROFILE_UNKNOWN, 0xff, FLAGS, .unit = "profile" },
 
 #define PROFILE(name, value)  name, NULL, 0, AV_OPT_TYPE_CONST, \
-      { .i64 = value }, 0, 0, FLAGS, "profile"
+      { .i64 = value }, 0, 0, FLAGS, .unit = "profile"
     { PROFILE("main",               AV_PROFILE_HEVC_MAIN) },
     { PROFILE("main10",             AV_PROFILE_HEVC_MAIN_10) },
     { PROFILE("rext",               AV_PROFILE_HEVC_REXT) },
 #undef PROFILE
 
     { "tier", "Set tier (general_tier_flag)",
-      OFFSET(tier), AV_OPT_TYPE_INT,
-      { .i64 = 0 }, 0, 1, FLAGS, "tier" },
+      OFFSET(unit_opts.tier), AV_OPT_TYPE_INT,
+      { .i64 = 0 }, 0, 1, FLAGS, .unit = "tier" },
     { "main", NULL, 0, AV_OPT_TYPE_CONST,
-      { .i64 = 0 }, 0, 0, FLAGS, "tier" },
+      { .i64 = 0 }, 0, 0, FLAGS, .unit = "tier" },
     { "high", NULL, 0, AV_OPT_TYPE_CONST,
-      { .i64 = 1 }, 0, 0, FLAGS, "tier" },
+      { .i64 = 1 }, 0, 0, FLAGS, .unit = "tier" },
 
     { "level", "Set level (general_level_idc)",
       OFFSET(level), AV_OPT_TYPE_INT,
-      { .i64 = AV_LEVEL_UNKNOWN }, AV_LEVEL_UNKNOWN, 0xff, FLAGS, "level" },
+      { .i64 = AV_LEVEL_UNKNOWN }, AV_LEVEL_UNKNOWN, 0xff, FLAGS, .unit = "level" },
 
 #define LEVEL(name, value) name, NULL, 0, AV_OPT_TYPE_CONST, \
-      { .i64 = value }, 0, 0, FLAGS, "level"
+      { .i64 = value }, 0, 0, FLAGS, .unit = "level"
     { LEVEL("1",    30) },
     { LEVEL("2",    60) },
     { LEVEL("2.1",  63) },
@@ -1450,18 +1152,18 @@ static const AVOption vaapi_encode_h265_options[] = {
     { "sei", "Set SEI to include",
       OFFSET(sei), AV_OPT_TYPE_FLAGS,
       { .i64 = SEI_MASTERING_DISPLAY | SEI_CONTENT_LIGHT_LEVEL | SEI_A53_CC },
-      0, INT_MAX, FLAGS, "sei" },
+      0, INT_MAX, FLAGS, .unit = "sei" },
     { "hdr",
       "Include HDR metadata for mastering display colour volume "
       "and content light level information",
       0, AV_OPT_TYPE_CONST,
       { .i64 = SEI_MASTERING_DISPLAY | SEI_CONTENT_LIGHT_LEVEL },
-      INT_MIN, INT_MAX, FLAGS, "sei" },
+      INT_MIN, INT_MAX, FLAGS, .unit = "sei" },
     { "a53_cc",
       "Include A/53 caption data",
       0, AV_OPT_TYPE_CONST,
       { .i64 = SEI_A53_CC },
-      INT_MIN, INT_MAX, FLAGS, "sei" },
+      INT_MIN, INT_MAX, FLAGS, .unit = "sei" },
 
     { "tiles", "Tile columns x rows",
       OFFSET(common.tile_cols), AV_OPT_TYPE_IMAGE_SIZE,
@@ -1509,6 +1211,7 @@ const FFCodec ff_hevc_vaapi_encoder = {
         AV_PIX_FMT_VAAPI,
         AV_PIX_FMT_NONE,
     },
+    .color_ranges   = AVCOL_RANGE_MPEG | AVCOL_RANGE_JPEG,
     .hw_configs     = ff_vaapi_encode_hw_configs,
     .p.wrapper_name = "vaapi",
 };
