@@ -29,41 +29,6 @@ wrap(avcodec_open2(AVCodecContext *avctx,
     testxmmclobbers(avcodec_open2, avctx, codec, options);
 }
 
-#if FF_API_OLD_ENCDEC
-wrap(avcodec_decode_audio4(AVCodecContext *avctx,
-                           AVFrame *frame,
-                           int *got_frame_ptr,
-                           AVPacket *avpkt))
-{
-    testxmmclobbers(avcodec_decode_audio4, avctx, frame,
-                    got_frame_ptr, avpkt);
-}
-
-wrap(avcodec_decode_video2(AVCodecContext *avctx,
-                           AVFrame *picture,
-                           int *got_picture_ptr,
-                           AVPacket *avpkt))
-{
-    testxmmclobbers(avcodec_decode_video2, avctx, picture,
-                    got_picture_ptr, avpkt);
-}
-
-wrap(avcodec_encode_audio2(AVCodecContext *avctx,
-                           AVPacket *avpkt,
-                           const AVFrame *frame,
-                           int *got_packet_ptr))
-{
-    testxmmclobbers(avcodec_encode_audio2, avctx, avpkt, frame,
-                    got_packet_ptr);
-}
-
-wrap(avcodec_encode_video2(AVCodecContext *avctx, AVPacket *avpkt,
-                           const AVFrame *frame, int *got_packet_ptr))
-{
-    testxmmclobbers(avcodec_encode_video2, avctx, avpkt, frame, got_packet_ptr);
-}
-#endif
-
 wrap(avcodec_decode_subtitle2(AVCodecContext *avctx,
                               AVSubtitle *sub,
                               int *got_sub_ptr,
