@@ -26,7 +26,7 @@
 #include "avcodec.h"
 #include "bytestream.h"
 #include "codec_internal.h"
-#include "internal.h"
+#include "decode.h"
 
 typedef struct XanContext {
     AVCodecContext *avctx;
@@ -435,7 +435,7 @@ static int xan_decode_frame(AVCodecContext *avctx, AVFrame *rframe,
 
 const FFCodec ff_xan_wc4_decoder = {
     .p.name         = "xan_wc4",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Wing Commander IV / Xxan"),
+    CODEC_LONG_NAME("Wing Commander IV / Xxan"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_XAN_WC4,
     .priv_data_size = sizeof(XanContext),
@@ -443,5 +443,5 @@ const FFCodec ff_xan_wc4_decoder = {
     .close          = xan_decode_end,
     FF_CODEC_DECODE_CB(xan_decode_frame),
     .p.capabilities = AV_CODEC_CAP_DR1,
-    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP | FF_CODEC_CAP_INIT_THREADSAFE,
+    .caps_internal  = FF_CODEC_CAP_INIT_CLEANUP,
 };
