@@ -19,13 +19,12 @@
  */
 
 #include "libavutil/avstring.h"
-#include "libavutil/eval.h"
+#include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "libavutil/samplefmt.h"
 #include "avfilter.h"
 #include "audio.h"
 #include "filters.h"
-#include "internal.h"
 
 typedef struct ChanDelay {
     int64_t delay;
