@@ -29,13 +29,14 @@
 #include "libavutil/channel_layout.h"
 #include "libavutil/common.h"
 #include "libavutil/mathematics.h"
+#include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "libavutil/samplefmt.h"
 
 #include "audio.h"
 #include "avfilter.h"
+#include "filters.h"
 #include "formats.h"
-#include "internal.h"
 
 struct ChannelMap {
     int in_channel;
@@ -54,15 +55,15 @@ enum MappingMode {
     MAP_PAIR_STR_STR
 };
 
-#define MAX_CH 64
 typedef struct ChannelMapContext {
     const AVClass *class;
     char *mapping_str;
-    char *channel_layout_str;
     AVChannelLayout output_layout;
-    struct ChannelMap map[MAX_CH];
+    struct ChannelMap *map;
     int nch;
     enum MappingMode mode;
+
+    uint8_t **source_planes;
 } ChannelMapContext;
 
 #define OFFSET(x) offsetof(ChannelMapContext, x)
@@ -72,12 +73,19 @@ static const AVOption channelmap_options[] = {
     { "map", "A comma-separated list of input channel numbers in output order.",
           OFFSET(mapping_str),        AV_OPT_TYPE_STRING, .flags = A|F },
     { "channel_layout", "Output channel layout.",
-          OFFSET(channel_layout_str), AV_OPT_TYPE_STRING, .flags = A|F },
+          OFFSET(output_layout),      AV_OPT_TYPE_CHLAYOUT, .flags = A|F },
     { NULL }
 };
 
 AVFILTER_DEFINE_CLASS(channelmap);
 
+static void channelmap_uninit(AVFilterContext *ctx)
+{
+    ChannelMapContext *s = ctx->priv;
+    av_freep(&s->map);
+    av_freep(&s->source_planes);
+}
+
 static char* split(char *message, char delim) {
     char *next = strchr(message, delim);
     if (next)
@@ -85,7 +93,7 @@ static char* split(char *message, char delim) {
     return next;
 }
 
-static int get_channel_idx(char **map, int *ch, char delim, int max_nb_channels)
+static int get_channel_idx(char **map, int *ch, char delim)
 {
     char *next;
     int len;
@@ -99,7 +107,7 @@ static int get_channel_idx(char **map, int *ch, char delim, int max_nb_channels)
     sscanf(*map, "%d%n", ch, &n);
     if (n != len)
         return AVERROR(EINVAL);
-    if (*ch < 0 || *ch >= max_nb_channels)
+    if (*ch < 0)
         return AVERROR(EINVAL);
     *map = next;
     return 0;
@@ -117,14 +125,39 @@ static int get_channel(char **map, int *ch, char delim)
     return 0;
 }
 
+static int check_idx_and_id(AVFilterContext *ctx, int channel_idx, int channel, AVChannelLayout *ch_layout, const char *io)
+{
+    char channel_name[64];
+    char layout_name[256];
+    int nb_channels = ch_layout->nb_channels;
+
+    if (channel_idx < 0 || channel_idx >= nb_channels) {
+        av_channel_layout_describe(ch_layout, layout_name, sizeof(layout_name));
+        if (channel >= 0) {
+            av_channel_name(channel_name, sizeof(channel_name), channel);
+            av_log(ctx, AV_LOG_ERROR,
+                   "%sput channel '%s' not available from %sput layout '%s'\n",
+                   io, channel_name, io, layout_name);
+        } else {
+            av_log(ctx, AV_LOG_ERROR,
+                   "%sput channel #%d not available from %sput layout '%s'\n",
+                   io, channel_idx, io, layout_name);
+        }
+        return AVERROR(EINVAL);
+    }
+
+    return 0;
+}
+
 static av_cold int channelmap_init(AVFilterContext *ctx)
 {
     ChannelMapContext *s = ctx->priv;
     char *mapping, separator = '|';
     int map_entries = 0;
-    char buf[256];
     enum MappingMode mode;
-    uint64_t out_ch_mask = 0;
+    int64_t out_ch_mask = 0;
+    uint8_t *presence_map = NULL;
+    int ret = 0;
     int i;
 
     mapping = s->mapping_str;
@@ -158,20 +191,25 @@ static av_cold int channelmap_init(AVFilterContext *ctx)
             if (*++sep)  // Allow trailing comma
                 map_entries++;
         }
-    }
 
-    if (map_entries > MAX_CH) {
-        av_log(ctx, AV_LOG_ERROR, "Too many channels mapped: '%d'.\n", map_entries);
-        return AVERROR(EINVAL);
+        s->map = av_malloc_array(map_entries, sizeof(*s->map));
+        if (!s->map)
+            return AVERROR(ENOMEM);
     }
 
     for (i = 0; i < map_entries; i++) {
         int in_ch_idx = -1, out_ch_idx = -1;
         int in_ch = -1, out_ch = -1;
         static const char err[] = "Failed to parse channel map\n";
+
+        s->map[i].in_channel_idx  = -1;
+        s->map[i].out_channel_idx = -1;
+        s->map[i].in_channel      = -1;
+        s->map[i].out_channel     = -1;
+
         switch (mode) {
         case MAP_ONE_INT:
-            if (get_channel_idx(&mapping, &in_ch_idx, separator, MAX_CH) < 0) {
+            if (get_channel_idx(&mapping, &in_ch_idx, separator) < 0) {
                 av_log(ctx, AV_LOG_ERROR, err);
                 return AVERROR(EINVAL);
             }
@@ -187,8 +225,8 @@ static av_cold int channelmap_init(AVFilterContext *ctx)
             s->map[i].out_channel_idx = i;
             break;
         case MAP_PAIR_INT_INT:
-            if (get_channel_idx(&mapping, &in_ch_idx, '-', MAX_CH) < 0 ||
-                get_channel_idx(&mapping, &out_ch_idx, separator, MAX_CH) < 0) {
+            if (get_channel_idx(&mapping, &in_ch_idx, '-') < 0 ||
+                get_channel_idx(&mapping, &out_ch_idx, separator) < 0) {
                 av_log(ctx, AV_LOG_ERROR, err);
                 return AVERROR(EINVAL);
             }
@@ -196,19 +234,21 @@ static av_cold int channelmap_init(AVFilterContext *ctx)
             s->map[i].out_channel_idx = out_ch_idx;
             break;
         case MAP_PAIR_INT_STR:
-            if (get_channel_idx(&mapping, &in_ch_idx, '-', MAX_CH) < 0 ||
-                get_channel(&mapping, &out_ch, separator) < 0 ||
-                (1ULL << out_ch) & out_ch_mask) {
+            if (get_channel_idx(&mapping, &in_ch_idx, '-') < 0 ||
+                get_channel(&mapping, &out_ch, separator) < 0) {
                 av_log(ctx, AV_LOG_ERROR, err);
                 return AVERROR(EINVAL);
             }
             s->map[i].in_channel_idx  = in_ch_idx;
             s->map[i].out_channel     = out_ch;
-            out_ch_mask |= 1ULL << out_ch;
+            if (out_ch < 63)
+                out_ch_mask |= 1ULL << out_ch;
+            else
+                out_ch_mask = -1;
             break;
         case MAP_PAIR_STR_INT:
             if (get_channel(&mapping, &in_ch, '-') < 0 ||
-                get_channel_idx(&mapping, &out_ch_idx, separator, MAX_CH) < 0) {
+                get_channel_idx(&mapping, &out_ch_idx, separator) < 0) {
                 av_log(ctx, AV_LOG_ERROR, err);
                 return AVERROR(EINVAL);
             }
@@ -217,65 +257,49 @@ static av_cold int channelmap_init(AVFilterContext *ctx)
             break;
         case MAP_PAIR_STR_STR:
             if (get_channel(&mapping, &in_ch, '-') < 0 ||
-                get_channel(&mapping, &out_ch, separator) < 0 ||
-                (1ULL << out_ch) & out_ch_mask) {
+                get_channel(&mapping, &out_ch, separator) < 0) {
                 av_log(ctx, AV_LOG_ERROR, err);
                 return AVERROR(EINVAL);
             }
             s->map[i].in_channel = in_ch;
             s->map[i].out_channel = out_ch;
-            out_ch_mask |= 1ULL << out_ch;
+            if (out_ch < 63)
+                out_ch_mask |= 1ULL << out_ch;
+            else
+                out_ch_mask = -1;
             break;
         }
     }
     s->mode          = mode;
     s->nch           = map_entries;
-    if (out_ch_mask)
-        av_channel_layout_from_mask(&s->output_layout, out_ch_mask);
-    else
-        av_channel_layout_default(&s->output_layout, map_entries);
-
-    if (s->channel_layout_str) {
-        AVChannelLayout fmt = { 0 };
-        int ret;
-        if ((ret = av_channel_layout_from_string(&fmt, s->channel_layout_str)) < 0) {
-#if FF_API_OLD_CHANNEL_LAYOUT
-            uint64_t mask;
-FF_DISABLE_DEPRECATION_WARNINGS
-            if ((mask = av_get_channel_layout(s->channel_layout_str)) == 0) {
-#endif
-                av_log(ctx, AV_LOG_ERROR, "Error parsing channel layout: '%s'.\n",
-                       s->channel_layout_str);
-                return AVERROR(EINVAL);
-#if FF_API_OLD_CHANNEL_LAYOUT
-            }
-FF_ENABLE_DEPRECATION_WARNINGS
-            av_log(ctx, AV_LOG_WARNING, "Channel layout '%s' uses a deprecated syntax.\n",
-                   s->channel_layout_str);
-            av_channel_layout_from_mask(&fmt, mask);
-#endif
-        }
-        if (mode == MAP_NONE) {
-            int i;
-            s->nch = fmt.nb_channels;
-            for (i = 0; i < s->nch; i++) {
-                s->map[i].in_channel_idx  = i;
-                s->map[i].out_channel_idx = i;
-            }
-        } else if (out_ch_mask && av_channel_layout_compare(&s->output_layout, &fmt)) {
-            av_channel_layout_describe(&s->output_layout, buf, sizeof(buf));
-            av_log(ctx, AV_LOG_ERROR,
-                   "Output channel layout '%s' does not match the list of channel mapped: '%s'.\n",
-                   s->channel_layout_str, buf);
-            return AVERROR(EINVAL);
-        } else if (s->nch != fmt.nb_channels) {
-            av_log(ctx, AV_LOG_ERROR,
-                   "Output channel layout %s does not match the number of channels mapped %d.\n",
-                   s->channel_layout_str, s->nch);
-            return AVERROR(EINVAL);
-        }
-        s->output_layout = fmt;
+    if (s->output_layout.nb_channels == 0) {
+        if (out_ch_mask > 0)
+            av_channel_layout_from_mask(&s->output_layout, out_ch_mask);
+        else if (map_entries)
+            av_channel_layout_default(&s->output_layout, map_entries);
     }
+
+    if (mode == MAP_NONE) {
+        int i;
+        s->nch = s->output_layout.nb_channels;
+
+        s->map = av_malloc_array(s->nch, sizeof(*s->map));
+        if (!s->map)
+            return AVERROR(ENOMEM);
+
+        for (i = 0; i < s->nch; i++) {
+            s->map[i].in_channel_idx  = i;
+            s->map[i].out_channel_idx = i;
+        }
+    } else if (s->nch != s->output_layout.nb_channels) {
+        char buf[256];
+        av_channel_layout_describe(&s->output_layout, buf, sizeof(buf));
+        av_log(ctx, AV_LOG_ERROR,
+               "Output channel layout %s does not match the number of channels mapped %d.\n",
+               buf, s->nch);
+        return AVERROR(EINVAL);
+    }
+
     if (!s->output_layout.nb_channels) {
         av_log(ctx, AV_LOG_ERROR, "Output channel layout is not set and "
                "cannot be guessed from the maps.\n");
@@ -289,24 +313,51 @@ FF_ENABLE_DEPRECATION_WARNINGS
         }
     }
 
+    presence_map = av_calloc(s->nch, sizeof(*presence_map));
+    for (i = 0; i < s->nch; i++) {
+        const int out_idx = s->map[i].out_channel_idx;
+        ret = check_idx_and_id(ctx, out_idx, s->map[i].out_channel, &s->output_layout, "out");
+        if (ret < 0)
+            break;
+        if (presence_map[out_idx]) {
+            char layout_name[256];
+            av_channel_layout_describe(&s->output_layout, layout_name, sizeof(layout_name));
+            av_log(ctx, AV_LOG_ERROR, "Mapping %d assigns channel #%d twice in output layout '%s'.\n",
+                   i + 1, s->map[i].out_channel_idx, layout_name);
+            ret = AVERROR(EINVAL);
+            break;
+        }
+        presence_map[out_idx] = 1;
+    }
+    av_freep(&presence_map);
+    if (ret < 0)
+        return ret;
+
     return 0;
 }
 
-static int channelmap_query_formats(AVFilterContext *ctx)
+static int channelmap_query_formats(const AVFilterContext *ctx,
+                                    AVFilterFormatsConfig **cfg_in,
+                                    AVFilterFormatsConfig **cfg_out)
 {
-    ChannelMapContext *s = ctx->priv;
+    const ChannelMapContext *s = ctx->priv;
     AVFilterChannelLayouts *channel_layouts = NULL;
+
     int ret;
 
-    if ((ret = ff_set_common_formats    (ctx,  ff_planar_sample_fmts()))  < 0 ||
-        (ret = ff_set_common_all_samplerates(ctx                              )) < 0 ||
-        (ret = ff_add_channel_layout(&channel_layouts, &s->output_layout)) < 0 ||
-        (ret = ff_channel_layouts_ref(channel_layouts,
-                                      &ctx->outputs[0]->incfg.channel_layouts)) < 0)
+    ret = ff_set_common_formats2(ctx, cfg_in, cfg_out, ff_planar_sample_fmts());
+    if (ret < 0)
         return ret;
 
-    return ff_channel_layouts_ref(ff_all_channel_counts(),
-                                  &ctx->inputs[0]->outcfg.channel_layouts);
+    ret = ff_add_channel_layout(&channel_layouts, &s->output_layout);
+    if (ret < 0)
+        return ret;
+
+    ret = ff_channel_layouts_ref(channel_layouts, &cfg_out[0]->channel_layouts);
+    if (ret < 0)
+        return ret;
+
+    return 0;
 }
 
 static int channelmap_filter_frame(AVFilterLink *inlink, AVFrame *buf)
@@ -317,10 +368,9 @@ static int channelmap_filter_frame(AVFilterLink *inlink, AVFrame *buf)
     const int nch_in = inlink->ch_layout.nb_channels;
     const int nch_out = s->nch;
     int ch, ret;
-    uint8_t *source_planes[MAX_CH];
 
-    memcpy(source_planes, buf->extended_data,
-           nch_in * sizeof(source_planes[0]));
+    memcpy(s->source_planes, buf->extended_data,
+           nch_in * sizeof(s->source_planes[0]));
 
     if (nch_out > nch_in) {
         if (nch_out > FF_ARRAY_ELEMS(buf->data)) {
@@ -344,19 +394,13 @@ static int channelmap_filter_frame(AVFilterLink *inlink, AVFrame *buf)
 
     for (ch = 0; ch < nch_out; ch++) {
         buf->extended_data[s->map[ch].out_channel_idx] =
-            source_planes[s->map[ch].in_channel_idx];
+            s->source_planes[s->map[ch].in_channel_idx];
     }
 
     if (buf->data != buf->extended_data)
         memcpy(buf->data, buf->extended_data,
            FFMIN(FF_ARRAY_ELEMS(buf->data), nch_out) * sizeof(buf->data[0]));
 
-#if FF_API_OLD_CHANNEL_LAYOUT
-FF_DISABLE_DEPRECATION_WARNINGS
-    buf->channels = outlink->ch_layout.nb_channels;
-    buf->channel_layout = outlink->channel_layout;
-FF_ENABLE_DEPRECATION_WARNINGS
-#endif
     if ((ret = av_channel_layout_copy(&buf->ch_layout, &outlink->ch_layout)) < 0)
         return ret;
 
@@ -367,10 +411,7 @@ static int channelmap_config_input(AVFilterLink *inlink)
 {
     AVFilterContext *ctx = inlink->dst;
     ChannelMapContext *s = ctx->priv;
-    int nb_channels = inlink->ch_layout.nb_channels;
     int i, err = 0;
-    char channel_name[64];
-    char layout_name[256];
 
     for (i = 0; i < s->nch; i++) {
         struct ChannelMap *m = &s->map[i];
@@ -380,22 +421,16 @@ static int channelmap_config_input(AVFilterLink *inlink)
                 &inlink->ch_layout, m->in_channel);
         }
 
-        if (m->in_channel_idx < 0 || m->in_channel_idx >= nb_channels) {
-            av_channel_layout_describe(&inlink->ch_layout, layout_name, sizeof(layout_name));
-            if (m->in_channel >= 0) {
-                av_channel_name(channel_name, sizeof(channel_name), m->in_channel);
-                av_log(ctx, AV_LOG_ERROR,
-                       "input channel '%s' not available from input layout '%s'\n",
-                       channel_name, layout_name);
-            } else {
-                av_log(ctx, AV_LOG_ERROR,
-                       "input channel #%d not available from input layout '%s'\n",
-                       m->in_channel_idx, layout_name);
-            }
+        if (check_idx_and_id(ctx, m->in_channel_idx, m->in_channel, &inlink->ch_layout, "in") < 0)
             err = AVERROR(EINVAL);
-        }
     }
 
+    av_freep(&s->source_planes);
+    s->source_planes = av_calloc(inlink->ch_layout.nb_channels,
+                                 sizeof(*s->source_planes));
+    if (!s->source_planes)
+        return AVERROR(ENOMEM);
+
     return err;
 }
 
@@ -413,9 +448,10 @@ const AVFilter ff_af_channelmap = {
     .name          = "channelmap",
     .description   = NULL_IF_CONFIG_SMALL("Remap audio channels."),
     .init          = channelmap_init,
+    .uninit        = channelmap_uninit,
     .priv_size     = sizeof(ChannelMapContext),
     .priv_class    = &channelmap_class,
     FILTER_INPUTS(avfilter_af_channelmap_inputs),
     FILTER_OUTPUTS(ff_audio_default_filterpad),
-    FILTER_QUERY_FUNC(channelmap_query_formats),
+    FILTER_QUERY_FUNC2(channelmap_query_formats),
 };
