@@ -30,10 +30,11 @@
  */
 
 #include "libavutil/avstring.h"
+#include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "audio.h"
 #include "avfilter.h"
-#include "internal.h"
+#include "filters.h"
 #include "generate_wave_table.h"
 
 typedef struct ChorusContext {
