@@ -25,11 +25,12 @@
 
 #include <float.h> /* DBL_MAX */
 
+#include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "libavutil/timestamp.h"
 #include "audio.h"
 #include "avfilter.h"
-#include "internal.h"
+#include "filters.h"
 
 typedef struct SilenceDetectContext {
     const AVClass *class;
