@@ -32,7 +32,6 @@
 #include "formats.h"
 #include "audio.h"
 #include "video.h"
-#include "internal.h"
 
 enum VectorScopeMode {
     LISSAJOUS,
@@ -44,6 +43,7 @@ enum VectorScopeMode {
 enum VectorScopeDraw {
     DOT,
     LINE,
+    AALINE,
     DRAW_NB,
 };
 
@@ -78,11 +78,11 @@ typedef struct AudioVectorScopeContext {
 #define TFLAGS AV_OPT_FLAG_FILTERING_PARAM|AV_OPT_FLAG_VIDEO_PARAM|AV_OPT_FLAG_RUNTIME_PARAM
 
 static const AVOption avectorscope_options[] = {
-    { "mode", "set mode", OFFSET(mode), AV_OPT_TYPE_INT, {.i64=LISSAJOUS}, 0, MODE_NB-1, TFLAGS, "mode" },
-    { "m",    "set mode", OFFSET(mode), AV_OPT_TYPE_INT, {.i64=LISSAJOUS}, 0, MODE_NB-1, TFLAGS, "mode" },
-    { "lissajous",    "", 0, AV_OPT_TYPE_CONST, {.i64=LISSAJOUS},    0, 0, TFLAGS, "mode" },
-    { "lissajous_xy", "", 0, AV_OPT_TYPE_CONST, {.i64=LISSAJOUS_XY}, 0, 0, TFLAGS, "mode" },
-    { "polar",        "", 0, AV_OPT_TYPE_CONST, {.i64=POLAR},        0, 0, TFLAGS, "mode" },
+    { "mode", "set mode", OFFSET(mode), AV_OPT_TYPE_INT, {.i64=LISSAJOUS}, 0, MODE_NB-1, TFLAGS, .unit = "mode" },
+    { "m",    "set mode", OFFSET(mode), AV_OPT_TYPE_INT, {.i64=LISSAJOUS}, 0, MODE_NB-1, TFLAGS, .unit = "mode" },
+    { "lissajous",    "", 0, AV_OPT_TYPE_CONST, {.i64=LISSAJOUS},    0, 0, TFLAGS, .unit = "mode" },
+    { "lissajous_xy", "", 0, AV_OPT_TYPE_CONST, {.i64=LISSAJOUS_XY}, 0, 0, TFLAGS, .unit = "mode" },
+    { "polar",        "", 0, AV_OPT_TYPE_CONST, {.i64=POLAR},        0, 0, TFLAGS, .unit = "mode" },
     { "rate", "set video rate", OFFSET(frame_rate), AV_OPT_TYPE_VIDEO_RATE, {.str="25"}, 0, INT_MAX, FLAGS },
     { "r",    "set video rate", OFFSET(frame_rate), AV_OPT_TYPE_VIDEO_RATE, {.str="25"}, 0, INT_MAX, FLAGS },
     { "size", "set video size", OFFSET(w), AV_OPT_TYPE_IMAGE_SIZE, {.str="400x400"}, 0, 0, FLAGS },
@@ -96,28 +96,29 @@ static const AVOption avectorscope_options[] = {
     { "bf", "set blue fade",      OFFSET(fade[2]), AV_OPT_TYPE_INT, {.i64=5},  0, 255, TFLAGS },
     { "af", "set alpha fade",     OFFSET(fade[3]), AV_OPT_TYPE_INT, {.i64=5},  0, 255, TFLAGS },
     { "zoom", "set zoom factor",  OFFSET(zoom), AV_OPT_TYPE_DOUBLE, {.dbl=1},  0, 10, TFLAGS },
-    { "draw", "set draw mode", OFFSET(draw), AV_OPT_TYPE_INT, {.i64=DOT}, 0, DRAW_NB-1, TFLAGS, "draw" },
-    { "dot",   "", 0, AV_OPT_TYPE_CONST, {.i64=DOT} , 0, 0, TFLAGS, "draw" },
-    { "line",  "", 0, AV_OPT_TYPE_CONST, {.i64=LINE}, 0, 0, TFLAGS, "draw" },
-    { "scale", "set amplitude scale mode", OFFSET(scale), AV_OPT_TYPE_INT, {.i64=LIN}, 0, SCALE_NB-1, TFLAGS, "scale" },
-    { "lin",   "linear",      0, AV_OPT_TYPE_CONST, {.i64=LIN},  0, 0, TFLAGS, "scale" },
-    { "sqrt",  "square root", 0, AV_OPT_TYPE_CONST, {.i64=SQRT}, 0, 0, TFLAGS, "scale" },
-    { "cbrt",  "cube root",   0, AV_OPT_TYPE_CONST, {.i64=CBRT}, 0, 0, TFLAGS, "scale" },
-    { "log",   "logarithmic", 0, AV_OPT_TYPE_CONST, {.i64=LOG},  0, 0, TFLAGS, "scale" },
+    { "draw", "set draw mode", OFFSET(draw), AV_OPT_TYPE_INT, {.i64=DOT}, 0, DRAW_NB-1, TFLAGS, .unit = "draw" },
+    { "dot",   "draw dots",               0, AV_OPT_TYPE_CONST, {.i64=DOT} , 0, 0, TFLAGS, .unit = "draw" },
+    { "line",  "draw lines",              0, AV_OPT_TYPE_CONST, {.i64=LINE}, 0, 0, TFLAGS, .unit = "draw" },
+    { "aaline","draw anti-aliased lines", 0, AV_OPT_TYPE_CONST, {.i64=AALINE},0,0, TFLAGS, .unit = "draw" },
+    { "scale", "set amplitude scale mode", OFFSET(scale), AV_OPT_TYPE_INT, {.i64=LIN}, 0, SCALE_NB-1, TFLAGS, .unit = "scale" },
+    { "lin",   "linear",      0, AV_OPT_TYPE_CONST, {.i64=LIN},  0, 0, TFLAGS, .unit = "scale" },
+    { "sqrt",  "square root", 0, AV_OPT_TYPE_CONST, {.i64=SQRT}, 0, 0, TFLAGS, .unit = "scale" },
+    { "cbrt",  "cube root",   0, AV_OPT_TYPE_CONST, {.i64=CBRT}, 0, 0, TFLAGS, .unit = "scale" },
+    { "log",   "logarithmic", 0, AV_OPT_TYPE_CONST, {.i64=LOG},  0, 0, TFLAGS, .unit = "scale" },
     { "swap", "swap x axis with y axis", OFFSET(swap), AV_OPT_TYPE_BOOL, {.i64=1}, 0, 1, TFLAGS },
-    { "mirror", "mirror axis", OFFSET(mirror), AV_OPT_TYPE_INT, {.i64=0}, 0, 3, TFLAGS, "mirror" },
-    { "none",  "no mirror", 0, AV_OPT_TYPE_CONST, {.i64=0}, 0, 0, TFLAGS, "mirror" },
-    { "x",  "mirror x",     0, AV_OPT_TYPE_CONST, {.i64=1}, 0, 0, TFLAGS, "mirror" },
-    { "y",  "mirror y",     0, AV_OPT_TYPE_CONST, {.i64=2}, 0, 0, TFLAGS, "mirror" },
-    { "xy", "mirror both",  0, AV_OPT_TYPE_CONST, {.i64=3}, 0, 0, TFLAGS, "mirror" },
+    { "mirror", "mirror axis", OFFSET(mirror), AV_OPT_TYPE_INT, {.i64=0}, 0, 3, TFLAGS, .unit = "mirror" },
+    { "none",  "no mirror", 0, AV_OPT_TYPE_CONST, {.i64=0}, 0, 0, TFLAGS, .unit = "mirror" },
+    { "x",  "mirror x",     0, AV_OPT_TYPE_CONST, {.i64=1}, 0, 0, TFLAGS, .unit = "mirror" },
+    { "y",  "mirror y",     0, AV_OPT_TYPE_CONST, {.i64=2}, 0, 0, TFLAGS, .unit = "mirror" },
+    { "xy", "mirror both",  0, AV_OPT_TYPE_CONST, {.i64=3}, 0, 0, TFLAGS, .unit = "mirror" },
     { NULL }
 };
 
 AVFILTER_DEFINE_CLASS(avectorscope);
 
-static void draw_dot(AudioVectorScopeContext *s, unsigned x, unsigned y)
+static void draw_dot(AudioVectorScopeContext *s, unsigned x, unsigned y, int value)
 {
-    const int linesize = s->outpicref->linesize[0];
+    const ptrdiff_t linesize = s->outpicref->linesize[0];
     uint8_t *dst;
 
     if (s->zoom > 1) {
@@ -128,11 +129,11 @@ static void draw_dot(AudioVectorScopeContext *s, unsigned x, unsigned y)
         x = FFMIN(x, s->w - 1);
     }
 
-    dst = &s->outpicref->data[0][y * linesize + x * 4];
-    dst[0] = FFMIN(dst[0] + s->contrast[0], 255);
-    dst[1] = FFMIN(dst[1] + s->contrast[1], 255);
-    dst[2] = FFMIN(dst[2] + s->contrast[2], 255);
-    dst[3] = FFMIN(dst[3] + s->contrast[3], 255);
+    dst = s->outpicref->data[0] + y * linesize + x * 4;
+    dst[0] = FFMIN(dst[0] + s->contrast[0], value);
+    dst[1] = FFMIN(dst[1] + s->contrast[1], value);
+    dst[2] = FFMIN(dst[2] + s->contrast[2], value);
+    dst[3] = FFMIN(dst[3] + s->contrast[3], value);
 }
 
 static void draw_line(AudioVectorScopeContext *s, int x0, int y0, int x1, int y1)
@@ -142,7 +143,7 @@ static void draw_line(AudioVectorScopeContext *s, int x0, int y0, int x1, int y1
     int err = (dx>dy ? dx : -dy) / 2, e2;
 
     for (;;) {
-        draw_dot(s, x0, y0);
+        draw_dot(s, x0, y0, 255);
 
         if (x0 == x1 && y0 == y1)
             break;
@@ -161,6 +162,40 @@ static void draw_line(AudioVectorScopeContext *s, int x0, int y0, int x1, int y1
     }
 }
 
+static void draw_aaline(AudioVectorScopeContext *s, int x0, int y0, int x1, int y1)
+{
+    int sx = x0 < x1 ? 1 : -1, sy = y0 < y1 ? 1 : -1, x2;
+    int dx = FFABS(x1-x0), dy = FFABS(y1-y0), err = dx * dx + dy * dy;
+    int e2 = err == 0 ? 1 : 0xffffff / (dx + dy);
+
+    dx *= e2;
+    dy *= e2;
+    err = dx - dy;
+
+    for (;;) {
+        draw_dot(s, x0, y0, 255-(FFABS(err - dx + dy) >> 16));
+        e2 = err;
+        x2 = x0;
+        if (2 * e2 >= -dx) {
+            if (x0 == x1)
+                break;
+            if (e2 + dy < 0xff0000)
+                draw_dot(s, x0, y0 + sy, 255-((e2 + dy) >> 16));
+            err -= dy;
+            x0 += sx;
+        }
+
+        if (2 * e2 <= dy) {
+            if (y0 == y1)
+                break;
+            if (dx - e2 < 0xff0000)
+                draw_dot(s, x2 + sx, y0, 255-((dx - e2) >> 16));
+            err += dx;
+            y0 += sy;
+        }
+    }
+}
+
 static int fade(AVFilterContext *ctx, void *arg, int jobnr, int nb_jobs)
 {
     AudioVectorScopeContext *s = ctx->priv;
@@ -235,12 +270,13 @@ static int config_input(AVFilterLink *inlink)
 static int config_output(AVFilterLink *outlink)
 {
     AudioVectorScopeContext *s = outlink->src->priv;
+    FilterLink *l = ff_filter_link(outlink);
 
     outlink->w = s->w;
     outlink->h = s->h;
     outlink->sample_aspect_ratio = (AVRational){1,1};
-    outlink->frame_rate = s->frame_rate;
-    outlink->time_base = av_inv_q(outlink->frame_rate);
+    l->frame_rate = s->frame_rate;
+    outlink->time_base = av_inv_q(l->frame_rate);
 
     s->prev_x = s->hw = s->w / 2;
     s->prev_y = s->hh = s->mode == POLAR ? s->h - 1 : s->h / 2;
@@ -252,6 +288,8 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *insamples)
 {
     AVFilterContext *ctx = inlink->dst;
     AVFilterLink *outlink = ctx->outputs[0];
+    const int16_t *samples = (const int16_t *)insamples->data[0];
+    const float *samplesf = (const float *)insamples->data[0];
     AudioVectorScopeContext *s = ctx->priv;
     const int hw = s->hw;
     const int hh = s->hh;
@@ -259,7 +297,7 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *insamples)
     unsigned x, y;
     unsigned prev_x = s->prev_x, prev_y = s->prev_y;
     double zoom = s->zoom;
-    int i;
+    int ret;
 
     if (!s->outpicref || s->outpicref->width  != outlink->w ||
                          s->outpicref->height != outlink->h) {
@@ -271,34 +309,32 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *insamples)
         }
 
         s->outpicref->sample_aspect_ratio = (AVRational){1,1};
-        for (i = 0; i < outlink->h; i++)
+        for (int i = 0; i < outlink->h; i++)
             memset(s->outpicref->data[0] + i * s->outpicref->linesize[0], 0, outlink->w * 4);
     }
     s->outpicref->pts = av_rescale_q(insamples->pts, inlink->time_base, outlink->time_base);
+    s->outpicref->duration = 1;
 
-    av_frame_make_writable(s->outpicref);
+    ret = ff_inlink_make_frame_writable(outlink, &s->outpicref);
+    if (ret < 0) {
+        av_frame_free(&insamples);
+        return ret;
+    }
     ff_filter_execute(ctx, fade, NULL, NULL, FFMIN(outlink->h, ff_filter_get_nb_threads(ctx)));
 
     if (zoom < 1) {
         float max = 0;
 
         switch (insamples->format) {
-        case AV_SAMPLE_FMT_S16: {
-            int16_t *samples = (int16_t *)insamples->data[0];
-
-            for (i = 0; i < insamples->nb_samples * 2; i++) {
+        case AV_SAMPLE_FMT_S16:
+            for (int i = 0; i < insamples->nb_samples * 2; i++) {
                 float sample = samples[i] / (float)INT16_MAX;
                 max = FFMAX(FFABS(sample), max);
             }
-
-            }
             break;
-        case AV_SAMPLE_FMT_FLT: {
-            float *samples = (float *)insamples->data[0];
-
-            for (i = 0; i < insamples->nb_samples * 2; i++) {
-                max = FFMAX(FFABS(samples[i]), max);
-            }
+        case AV_SAMPLE_FMT_FLT:
+            for (int i = 0; i < insamples->nb_samples * 2; i++) {
+                max = FFMAX(FFABS(samplesf[i]), max);
             }
             break;
         default:
@@ -317,22 +353,21 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *insamples)
             break;
         }
 
-        zoom = 1. / max;
+        if (max > 0.f)
+            zoom = 1. / max;
     }
 
-    for (i = 0; i < insamples->nb_samples; i++) {
-        int16_t *samples = (int16_t *)insamples->data[0] + i * 2;
-        float *samplesf = (float *)insamples->data[0] + i * 2;
+    for (int i = 0; i < insamples->nb_samples; i++) {
         float src[2];
 
         switch (insamples->format) {
         case AV_SAMPLE_FMT_S16:
-            src[0] = samples[0] / (float)INT16_MAX;
-            src[1] = samples[1] / (float)INT16_MAX;
+            src[0] = samples[i*2+0] / (float)INT16_MAX;
+            src[1] = samples[i*2+1] / (float)INT16_MAX;
             break;
         case AV_SAMPLE_FMT_FLT:
-            src[0] = samplesf[0];
-            src[1] = samplesf[1];
+            src[0] = samplesf[i*2+0];
+            src[1] = samplesf[i*2+1];
             break;
         default:
             av_assert2(0);
@@ -380,9 +415,11 @@ static int filter_frame(AVFilterLink *inlink, AVFrame *insamples)
         }
 
         if (s->draw == DOT) {
-            draw_dot(s, x, y);
-        } else {
+            draw_dot(s, x, y, 255);
+        } else if (s->draw == LINE) {
             draw_line(s, x, y, prev_x, prev_y);
+        } else {
+            draw_aaline(s, x, y, prev_x, prev_y);
         }
         prev_x = x;
         prev_y = y;
