@@ -24,33 +24,36 @@
  * Calculate the VMAF between two input videos.
  */
 
+#include "config_components.h"
+
 #include <libvmaf.h>
 
 #include "libavutil/avstring.h"
+#include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "libavutil/pixdesc.h"
 #include "avfilter.h"
 #include "drawutils.h"
+#include "filters.h"
 #include "formats.h"
 #include "framesync.h"
-#include "internal.h"
 #include "video.h"
 
+#if CONFIG_LIBVMAF_CUDA_FILTER
+#include <libvmaf_cuda.h>
+
+#include "libavutil/hwcontext.h"
+#include "libavutil/hwcontext_cuda_internal.h"
+#endif
+
 typedef struct LIBVMAFContext {
     const AVClass *class;
     FFFrameSync fs;
-    char *model_path;
     char *log_path;
     char *log_fmt;
-    int enable_transform;
-    int phone_model;
-    int psnr;
-    int ssim;
-    int ms_ssim;
     char *pool;
     int n_threads;
     int n_subsample;
-    int enable_conf_interval;
     char *model_cfg;
     char *feature_cfg;
     VmafContext *vmaf;
@@ -58,24 +61,20 @@ typedef struct LIBVMAFContext {
     unsigned model_cnt;
     unsigned frame_cnt;
     unsigned bpc;
+#if CONFIG_LIBVMAF_CUDA_FILTER
+    VmafCudaState *cu_state;
+#endif
 } LIBVMAFContext;
 
 #define OFFSET(x) offsetof(LIBVMAFContext, x)
 #define FLAGS AV_OPT_FLAG_FILTERING_PARAM|AV_OPT_FLAG_VIDEO_PARAM
 
 static const AVOption libvmaf_options[] = {
-    {"model_path",  "use model='path=...'.",                                            OFFSET(model_path), AV_OPT_TYPE_STRING, {.str=NULL}, 0, 1, FLAGS|AV_OPT_FLAG_DEPRECATED},
     {"log_path",  "Set the file path to be used to write log.",                         OFFSET(log_path), AV_OPT_TYPE_STRING, {.str=NULL}, 0, 1, FLAGS},
     {"log_fmt",  "Set the format of the log (csv, json, xml, or sub).",                 OFFSET(log_fmt), AV_OPT_TYPE_STRING, {.str="xml"}, 0, 1, FLAGS},
-    {"enable_transform",  "use model='enable_transform=true'.",                         OFFSET(enable_transform), AV_OPT_TYPE_BOOL, {.i64=0}, 0, 1, FLAGS|AV_OPT_FLAG_DEPRECATED},
-    {"phone_model",  "use model='enable_transform=true'.",                              OFFSET(phone_model), AV_OPT_TYPE_BOOL, {.i64=0}, 0, 1, FLAGS|AV_OPT_FLAG_DEPRECATED},
-    {"psnr",  "use feature='name=psnr'.",                                               OFFSET(psnr), AV_OPT_TYPE_BOOL, {.i64=0}, 0, 1, FLAGS|AV_OPT_FLAG_DEPRECATED},
-    {"ssim",  "use feature='name=ssim'.",                                               OFFSET(ssim), AV_OPT_TYPE_BOOL, {.i64=0}, 0, 1, FLAGS|AV_OPT_FLAG_DEPRECATED},
-    {"ms_ssim",  "use feature='name=ms_ssim'.",                                         OFFSET(ms_ssim), AV_OPT_TYPE_BOOL, {.i64=0}, 0, 1, FLAGS|AV_OPT_FLAG_DEPRECATED},
     {"pool",  "Set the pool method to be used for computing vmaf.",                     OFFSET(pool), AV_OPT_TYPE_STRING, {.str=NULL}, 0, 1, FLAGS},
     {"n_threads", "Set number of threads to be used when computing vmaf.",              OFFSET(n_threads), AV_OPT_TYPE_INT, {.i64=0}, 0, UINT_MAX, FLAGS},
     {"n_subsample", "Set interval for frame subsampling used when computing vmaf.",     OFFSET(n_subsample), AV_OPT_TYPE_INT, {.i64=1}, 1, UINT_MAX, FLAGS},
-    {"enable_conf_interval",  "model='enable_conf_interval=true'.",                     OFFSET(enable_conf_interval), AV_OPT_TYPE_BOOL, {.i64=0}, 0, 1, FLAGS|AV_OPT_FLAG_DEPRECATED},
     {"model",  "Set the model to be used for computing vmaf.",                          OFFSET(model_cfg), AV_OPT_TYPE_STRING, {.str="version=vmaf_v0.6.1"}, 0, 1, FLAGS},
     {"feature",  "Set the feature to be used for computing vmaf.",                      OFFSET(feature_cfg), AV_OPT_TYPE_STRING, {.str=NULL}, 0, 1, FLAGS},
     { NULL }
@@ -108,6 +107,7 @@ static enum VmafPixelFormat pix_fmt_map(enum AVPixelFormat av_pix_fmt)
 
 static int copy_picture_data(AVFrame *src, VmafPicture *dst, unsigned bpc)
 {
+    const int bytes_per_value = bpc > 8 ? 2 : 1;
     int err = vmaf_picture_alloc(dst, pix_fmt_map(src->format), bpc,
                                  src->width, src->height);
     if (err)
@@ -117,7 +117,7 @@ static int copy_picture_data(AVFrame *src, VmafPicture *dst, unsigned bpc)
         uint8_t *src_data = src->data[i];
         uint8_t *dst_data = dst->data[i];
         for (unsigned j = 0; j < dst->h[i]; j++) {
-            memcpy(dst_data, src_data, sizeof(*dst_data) * dst->w[i]);
+            memcpy(dst_data, src_data, bytes_per_value * dst->w[i]);
             src_data += src->linesize[i];
             dst_data += dst->stride[i];
         }
@@ -140,6 +140,13 @@ static int do_vmaf(FFFrameSync *fs)
     if (ctx->is_disabled || !ref)
         return ff_filter_frame(ctx->outputs[0], dist);
 
+    if (dist->color_range != ref->color_range) {
+        av_log(ctx, AV_LOG_WARNING, "distorted and reference "
+               "frames use different color ranges (%s != %s)\n",
+               av_color_range_name(dist->color_range),
+               av_color_range_name(ref->color_range));
+    }
+
     err = copy_picture_data(ref, &pic_ref, s->bpc);
     if (err) {
         av_log(s, AV_LOG_ERROR, "problem during vmaf_picture_alloc.\n");
@@ -162,7 +169,6 @@ static int do_vmaf(FFFrameSync *fs)
     return ff_filter_frame(ctx->outputs[0], dist);
 }
 
-
 static AVDictionary **delimited_dict_parse(char *str, unsigned *cnt)
 {
     AVDictionary **dict = NULL;
@@ -235,10 +241,10 @@ static int parse_features(AVFilterContext *ctx)
     for (unsigned i = 0; i < dict_cnt; i++) {
         char *feature_name = NULL;
         VmafFeatureDictionary *feature_opts_dict = NULL;
-        AVDictionaryEntry *e = NULL;
+        const AVDictionaryEntry *e = NULL;
 
-        while (e = av_dict_get(dict[i], "", e, AV_DICT_IGNORE_SUFFIX)) {
-            if (av_stristr(e->key, "name")) {
+        while (e = av_dict_iterate(dict[i], e)) {
+            if (!strcmp(e->key, "name")) {
                 feature_name = e->value;
                 continue;
             }
@@ -294,34 +300,34 @@ static int parse_models(AVFilterContext *ctx)
 
     for (unsigned i = 0; i < dict_cnt; i++) {
         VmafModelConfig model_cfg = { 0 };
-        AVDictionaryEntry *e = NULL;
+        const AVDictionaryEntry *e = NULL;
         char *version = NULL;
         char  *path = NULL;
 
-        while (e = av_dict_get(dict[i], "", e, AV_DICT_IGNORE_SUFFIX)) {
-            if (av_stristr(e->key, "disable_clip")) {
-                model_cfg.flags |= av_stristr(e->value, "true") ?
+        while (e = av_dict_iterate(dict[i], e)) {
+            if (!strcmp(e->key, "disable_clip")) {
+                model_cfg.flags |= !strcmp(e->value, "true") ?
                     VMAF_MODEL_FLAG_DISABLE_CLIP : 0;
                 continue;
             }
 
-            if (av_stristr(e->key, "enable_transform")) {
-                model_cfg.flags |= av_stristr(e->value, "true") ?
+            if (!strcmp(e->key, "enable_transform")) {
+                model_cfg.flags |= !strcmp(e->value, "true") ?
                     VMAF_MODEL_FLAG_ENABLE_TRANSFORM : 0;
                 continue;
             }
 
-            if (av_stristr(e->key, "name")) {
+            if (!strcmp(e->key, "name")) {
                 model_cfg.name = e->value;
                 continue;
             }
 
-            if (av_stristr(e->key, "version")) {
+            if (!strcmp(e->key, "version")) {
                 version = e->value;
                 continue;
             }
 
-            if (av_stristr(e->key, "path")) {
+            if (!strcmp(e->key, "path")) {
                 path = e->value;
                 continue;
             }
@@ -354,7 +360,7 @@ static int parse_models(AVFilterContext *ctx)
             goto exit;
         }
 
-        while (e = av_dict_get(dict[i], "", e, AV_DICT_IGNORE_SUFFIX)) {
+        while (e = av_dict_iterate(dict[i], e)) {
             VmafFeatureDictionary *feature_opts_dict = NULL;
             char *feature_opt = NULL;
 
@@ -420,92 +426,6 @@ static enum VmafLogLevel log_level_map(int log_level)
     }
 }
 
-static int parse_deprecated_options(AVFilterContext *ctx)
-{
-    LIBVMAFContext *s = ctx->priv;
-    VmafModel *model = NULL;
-    VmafModelCollection *model_collection = NULL;
-    enum VmafModelFlags flags = VMAF_MODEL_FLAGS_DEFAULT;
-    int err = 0;
-
-    VmafModelConfig model_cfg = {
-        .name = "vmaf",
-        .flags = flags,
-    };
-
-    if (s->enable_transform || s->phone_model)
-        flags |= VMAF_MODEL_FLAG_ENABLE_TRANSFORM;
-
-    if (!s->model_path)
-        goto extra_metrics_only;
-
-    if (s->enable_conf_interval) {
-        err = vmaf_model_collection_load_from_path(&model, &model_collection,
-                                                   &model_cfg, s->model_path);
-        if (err) {
-            av_log(ctx, AV_LOG_ERROR,
-                   "problem loading model file: %s\n", s->model_path);
-            goto exit;
-        }
-
-        err = vmaf_use_features_from_model_collection(s->vmaf, model_collection);
-        if (err) {
-            av_log(ctx, AV_LOG_ERROR,
-                   "problem loading feature extractors from model file: %s\n",
-                   s->model_path);
-            goto exit;
-        }
-    } else {
-        err = vmaf_model_load_from_path(&model, &model_cfg, s->model_path);
-        if (err) {
-                av_log(ctx, AV_LOG_ERROR,
-                      "problem loading model file: %s\n", s->model_path);
-            goto exit;
-        }
-        err = vmaf_use_features_from_model(s->vmaf, model);
-        if (err) {
-            av_log(ctx, AV_LOG_ERROR,
-                   "problem loading feature extractors from model file: %s\n",
-                   s->model_path);
-            goto exit;
-        }
-    }
-
-extra_metrics_only:
-    if (s->psnr) {
-        VmafFeatureDictionary *d = NULL;
-        vmaf_feature_dictionary_set(&d, "enable_chroma", "false");
-
-        err = vmaf_use_feature(s->vmaf, "psnr", d);
-        if (err) {
-            av_log(ctx, AV_LOG_ERROR,
-                   "problem loading feature extractor: psnr\n");
-            goto exit;
-        }
-    }
-
-    if (s->ssim) {
-        err = vmaf_use_feature(s->vmaf, "float_ssim", NULL);
-        if (err) {
-            av_log(ctx, AV_LOG_ERROR,
-                   "problem loading feature extractor: ssim\n");
-            goto exit;
-        }
-    }
-
-    if (s->ms_ssim) {
-        err = vmaf_use_feature(s->vmaf, "float_ms_ssim", NULL);
-        if (err) {
-            av_log(ctx, AV_LOG_ERROR,
-                   "problem loading feature extractor: ms_ssim\n");
-            goto exit;
-        }
-    }
-
-exit:
-    return err;
-}
-
 static av_cold int init(AVFilterContext *ctx)
 {
     LIBVMAFContext *s = ctx->priv;
@@ -521,10 +441,6 @@ static av_cold int init(AVFilterContext *ctx)
     if (err)
         return AVERROR(EINVAL);
 
-    err = parse_deprecated_options(ctx);
-    if (err)
-        return err;
-
     err = parse_models(ctx);
     if (err)
         return err;
@@ -540,6 +456,8 @@ static av_cold int init(AVFilterContext *ctx)
 static const enum AVPixelFormat pix_fmts[] = {
     AV_PIX_FMT_YUV444P, AV_PIX_FMT_YUV422P, AV_PIX_FMT_YUV420P,
     AV_PIX_FMT_YUV444P10LE, AV_PIX_FMT_YUV422P10LE, AV_PIX_FMT_YUV420P10LE,
+    AV_PIX_FMT_YUV444P12LE, AV_PIX_FMT_YUV422P12LE, AV_PIX_FMT_YUV420P12LE,
+    AV_PIX_FMT_YUV444P16LE, AV_PIX_FMT_YUV422P16LE, AV_PIX_FMT_YUV420P16LE,
     AV_PIX_FMT_NONE
 };
 
@@ -579,6 +497,8 @@ static int config_output(AVFilterLink *outlink)
     AVFilterContext *ctx = outlink->src;
     LIBVMAFContext *s = ctx->priv;
     AVFilterLink *mainlink = ctx->inputs[0];
+    FilterLink *il = ff_filter_link(mainlink);
+    FilterLink *ol = ff_filter_link(outlink);
     int ret;
 
     ret = ff_framesync_init_dualinput(&s->fs, ctx);
@@ -588,7 +508,7 @@ static int config_output(AVFilterLink *outlink)
     outlink->h = mainlink->h;
     outlink->time_base = mainlink->time_base;
     outlink->sample_aspect_ratio = mainlink->sample_aspect_ratio;
-    outlink->frame_rate = mainlink->frame_rate;
+    ol->frame_rate = il->frame_rate;
     if ((ret = ff_framesync_configure(&s->fs)) < 0)
         return ret;
 
@@ -604,13 +524,13 @@ static int activate(AVFilterContext *ctx)
 static enum VmafOutputFormat log_fmt_map(const char *log_fmt)
 {
     if (log_fmt) {
-        if (av_stristr(log_fmt, "xml"))
+        if (!strcmp(log_fmt, "xml"))
             return VMAF_OUTPUT_FORMAT_XML;
-        if (av_stristr(log_fmt, "json"))
+        if (!strcmp(log_fmt, "json"))
             return VMAF_OUTPUT_FORMAT_JSON;
-        if (av_stristr(log_fmt, "csv"))
+        if (!strcmp(log_fmt, "csv"))
             return VMAF_OUTPUT_FORMAT_CSV;
-        if (av_stristr(log_fmt, "sub"))
+        if (!strcmp(log_fmt, "sub"))
             return VMAF_OUTPUT_FORMAT_SUB;
     }
 
@@ -620,11 +540,11 @@ static enum VmafOutputFormat log_fmt_map(const char *log_fmt)
 static enum VmafPoolingMethod pool_method_map(const char *pool_method)
 {
     if (pool_method) {
-        if (av_stristr(pool_method, "min"))
+        if (!strcmp(pool_method, "min"))
             return VMAF_POOL_METHOD_MIN;
-        if (av_stristr(pool_method, "mean"))
+        if (!strcmp(pool_method, "mean"))
             return VMAF_POOL_METHOD_MEAN;
-        if (av_stristr(pool_method, "harmonic_mean"))
+        if (!strcmp(pool_method, "harmonic_mean"))
             return VMAF_POOL_METHOD_HARMONIC_MEAN;
     }
 
@@ -681,7 +601,8 @@ static const AVFilterPad libvmaf_inputs[] = {
     {
         .name         = "main",
         .type         = AVMEDIA_TYPE_VIDEO,
-    },{
+    },
+    {
         .name         = "reference",
         .type         = AVMEDIA_TYPE_VIDEO,
         .config_props = config_input_ref,
@@ -709,3 +630,199 @@ const AVFilter ff_vf_libvmaf = {
     FILTER_OUTPUTS(libvmaf_outputs),
     FILTER_PIXFMTS_ARRAY(pix_fmts),
 };
+
+#if CONFIG_LIBVMAF_CUDA_FILTER
+static const enum AVPixelFormat supported_formats[] = {
+    AV_PIX_FMT_YUV420P,
+    AV_PIX_FMT_YUV444P16,
+};
+
+static int format_is_supported(enum AVPixelFormat fmt)
+{
+    int i;
+
+    for (i = 0; i < FF_ARRAY_ELEMS(supported_formats); i++)
+        if (supported_formats[i] == fmt)
+            return 1;
+    return 0;
+}
+
+static int config_props_cuda(AVFilterLink *outlink)
+{
+    int err;
+    AVFilterContext *ctx = outlink->src;
+    LIBVMAFContext *s = ctx->priv;
+    AVFilterLink *inlink = ctx->inputs[0];
+    FilterLink      *inl = ff_filter_link(inlink);
+    AVHWFramesContext *frames_ctx = (AVHWFramesContext*) inl->hw_frames_ctx->data;
+    AVCUDADeviceContext *device_hwctx = frames_ctx->device_ctx->hwctx;
+    CUcontext cu_ctx = device_hwctx->cuda_ctx;
+    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(frames_ctx->sw_format);
+
+    VmafConfiguration cfg = {
+        .log_level = log_level_map(av_log_get_level()),
+        .n_subsample = s->n_subsample,
+        .n_threads = s->n_threads,
+    };
+
+    VmafCudaPictureConfiguration cuda_pic_cfg = {
+        .pic_params = {
+            .bpc = desc->comp[0].depth,
+            .w = inlink->w,
+            .h = inlink->h,
+            .pix_fmt = pix_fmt_map(frames_ctx->sw_format),
+        },
+        .pic_prealloc_method = VMAF_CUDA_PICTURE_PREALLOCATION_METHOD_DEVICE,
+    };
+
+    VmafCudaConfiguration cuda_cfg = {
+        .cu_ctx = cu_ctx,
+    };
+
+    if (!format_is_supported(frames_ctx->sw_format)) {
+        av_log(s, AV_LOG_ERROR,
+               "Unsupported input format: %s\n", desc->name);
+        return AVERROR(EINVAL);
+    }
+
+    err = vmaf_init(&s->vmaf, cfg);
+    if (err)
+        return AVERROR(EINVAL);
+
+    err = vmaf_cuda_state_init(&s->cu_state, cuda_cfg);
+    if (err)
+        return AVERROR(EINVAL);
+
+    err = vmaf_cuda_import_state(s->vmaf, s->cu_state);
+    if (err)
+        return AVERROR(EINVAL);
+
+    err = vmaf_cuda_preallocate_pictures(s->vmaf, cuda_pic_cfg);
+    if (err < 0)
+        return err;
+
+    err = parse_models(ctx);
+    if (err)
+        return err;
+
+    err = parse_features(ctx);
+    if (err)
+        return err;
+
+    return config_output(outlink);
+}
+
+static int copy_picture_data_cuda(VmafContext* vmaf,
+                                  AVCUDADeviceContext* device_hwctx,
+                                  AVFrame* src, VmafPicture* dst,
+                                  enum AVPixelFormat pix_fmt)
+{
+    const AVPixFmtDescriptor *pix_desc = av_pix_fmt_desc_get(pix_fmt);
+    CudaFunctions *cu = device_hwctx->internal->cuda_dl;
+
+    CUDA_MEMCPY2D m = {
+        .srcMemoryType = CU_MEMORYTYPE_DEVICE,
+        .dstMemoryType = CU_MEMORYTYPE_DEVICE,
+    };
+
+    int err = vmaf_cuda_fetch_preallocated_picture(vmaf, dst);
+    if (err)
+        return AVERROR(ENOMEM);
+
+    err = cu->cuCtxPushCurrent(device_hwctx->cuda_ctx);
+    if (err)
+        return AVERROR_EXTERNAL;
+
+    for (unsigned i = 0; i < pix_desc->nb_components; i++) {
+        m.srcDevice = (CUdeviceptr) src->data[i];
+        m.srcPitch = src->linesize[i];
+        m.dstDevice = (CUdeviceptr) dst->data[i];
+        m.dstPitch = dst->stride[i];
+        m.WidthInBytes = dst->w[i] * ((dst->bpc + 7) / 8);
+        m.Height = dst->h[i];
+
+        err = cu->cuMemcpy2D(&m);
+        if (err)
+            return AVERROR_EXTERNAL;
+        break;
+    }
+
+    err = cu->cuCtxPopCurrent(NULL);
+    if (err)
+        return AVERROR_EXTERNAL;
+
+    return 0;
+}
+
+static int do_vmaf_cuda(FFFrameSync* fs)
+{
+    AVFilterContext* ctx = fs->parent;
+    LIBVMAFContext* s = ctx->priv;
+    AVFilterLink *inlink = ctx->inputs[0];
+    FilterLink      *inl = ff_filter_link(inlink);
+    AVHWFramesContext *frames_ctx = (AVHWFramesContext*)inl->hw_frames_ctx->data;
+    AVCUDADeviceContext *device_hwctx = frames_ctx->device_ctx->hwctx;
+    VmafPicture pic_ref, pic_dist;
+    AVFrame *ref, *dist;
+
+    int err = 0;
+
+    err = ff_framesync_dualinput_get(fs, &dist, &ref);
+    if (err < 0)
+        return err;
+    if (ctx->is_disabled || !ref)
+        return ff_filter_frame(ctx->outputs[0], dist);
+
+    err = copy_picture_data_cuda(s->vmaf, device_hwctx, ref, &pic_ref,
+                                 frames_ctx->sw_format);
+    if (err) {
+        av_log(s, AV_LOG_ERROR, "problem during copy_picture_data_cuda.\n");
+        return AVERROR(ENOMEM);
+    }
+
+    err = copy_picture_data_cuda(s->vmaf, device_hwctx, dist, &pic_dist,
+                                 frames_ctx->sw_format);
+    if (err) {
+        av_log(s, AV_LOG_ERROR, "problem during copy_picture_data_cuda.\n");
+        return AVERROR(ENOMEM);
+    }
+
+    err = vmaf_read_pictures(s->vmaf, &pic_ref, &pic_dist, s->frame_cnt++);
+    if (err) {
+        av_log(s, AV_LOG_ERROR, "problem during vmaf_read_pictures.\n");
+        return AVERROR(EINVAL);
+    }
+
+    return ff_filter_frame(ctx->outputs[0], dist);
+}
+
+static av_cold int init_cuda(AVFilterContext *ctx)
+{
+    LIBVMAFContext *s = ctx->priv;
+    s->fs.on_event = do_vmaf_cuda;
+    return 0;
+}
+
+static const AVFilterPad libvmaf_outputs_cuda[] = {
+    {
+        .name         = "default",
+        .type         = AVMEDIA_TYPE_VIDEO,
+        .config_props = config_props_cuda,
+    },
+};
+
+const AVFilter ff_vf_libvmaf_cuda = {
+    .name           = "libvmaf_cuda",
+    .description    = NULL_IF_CONFIG_SMALL("Calculate the VMAF between two video streams."),
+    .preinit        = libvmaf_framesync_preinit,
+    .init           = init_cuda,
+    .uninit         = uninit,
+    .activate       = activate,
+    .priv_size      = sizeof(LIBVMAFContext),
+    .priv_class     = &libvmaf_class,
+    FILTER_INPUTS(libvmaf_inputs),
+    FILTER_OUTPUTS(libvmaf_outputs_cuda),
+    FILTER_SINGLE_PIXFMT(AV_PIX_FMT_CUDA),
+    .flags_internal = FF_FILTER_FLAG_HWFRAME_AWARE,
+};
+#endif
