@@ -24,17 +24,18 @@
  * 3D Lookup table filter
  */
 
+#include <float.h>
+
 #include "config_components.h"
 
-#include "float.h"
-
+#include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "libavutil/file_open.h"
 #include "libavutil/intfloat.h"
 #include "libavutil/avassert.h"
 #include "libavutil/avstring.h"
 #include "drawutils.h"
-#include "internal.h"
+#include "filters.h"
 #include "video.h"
 #include "lut3d.h"
 
@@ -1111,6 +1112,8 @@ static const enum AVPixelFormat pix_fmts[] = {
     AV_PIX_FMT_NONE
 };
 
+#if CONFIG_LUT3D_FILTER || CONFIG_HALDCLUT_FILTER
+
 static int config_input(AVFilterLink *inlink)
 {
     int depth, is16bit, isfloat, planar;
@@ -1207,8 +1210,6 @@ static int process_command(AVFilterContext *ctx, const char *cmd, const char *ar
     return config_input(ctx->inputs[0]);
 }
 
-#if CONFIG_LUT3D_FILTER || CONFIG_HALDCLUT_FILTER
-
 /* These options are shared between several filters;
  * &lut3d_haldclut_options[COMMON_OPTIONS_OFFSET] must always
  * point to the first of the COMMON_OPTIONS. */
