@@ -23,12 +23,13 @@
  * Shape Adaptive Blur filter, ported from MPlayer libmpcodecs/vf_sab.c
  */
 
+#include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "libavutil/pixdesc.h"
 #include "libswscale/swscale.h"
 
 #include "avfilter.h"
-#include "internal.h"
+#include "filters.h"
 #include "video.h"
 
 typedef struct FilterParam {
