@@ -37,15 +37,45 @@
  */
 
 #include "avfilter.h"
-#include "formats.h"
-#include "internal.h"
+#include "filters.h"
 #include "video.h"
 #include "libavutil/common.h"
 #include "libavutil/imgutils.h"
 #include "libavutil/mem.h"
 #include "libavutil/opt.h"
 #include "libavutil/pixdesc.h"
-#include "unsharp.h"
+
+#define MIN_MATRIX_SIZE 3
+#define MAX_MATRIX_SIZE 63
+
+typedef struct UnsharpFilterParam {
+    int msize_x;                             ///< matrix width
+    int msize_y;                             ///< matrix height
+    int amount;                              ///< effect amount
+    int steps_x;                             ///< horizontal step count
+    int steps_y;                             ///< vertical step count
+    int scalebits;                           ///< bits to shift pixel
+    int32_t halfscale;                       ///< amount to add to pixel
+    uint32_t *sr;        ///< finite state machine storage within a row
+    uint32_t **sc;       ///< finite state machine storage across rows
+} UnsharpFilterParam;
+
+typedef struct UnsharpContext {
+    const AVClass *class;
+    int lmsize_x, lmsize_y, cmsize_x, cmsize_y;
+    int amsize_x, amsize_y;
+    float lamount, camount;
+    float aamount;
+    UnsharpFilterParam luma;   ///< luma parameters (width, height, amount)
+    UnsharpFilterParam chroma; ///< chroma parameters (width, height, amount)
+    UnsharpFilterParam alpha;  ///< alpha parameters (width, height, amount)
+    int hsub, vsub;
+    int nb_planes;
+    int bitdepth;
+    int bps;
+    int nb_threads;
+    int (* unsharp_slice)(AVFilterContext *ctx, void *arg, int jobnr, int nb_jobs);
+} UnsharpContext;
 
 typedef struct TheadData {
     UnsharpFilterParam *fp;
@@ -142,7 +172,7 @@ static int name##_##nbits(AVFilterContext *ctx, void *arg, int jobnr, int nb_job
 DEF_UNSHARP_SLICE_FUNC(unsharp_slice, 16)
 DEF_UNSHARP_SLICE_FUNC(unsharp_slice, 8)
 
-static int apply_unsharp_c(AVFilterContext *ctx, AVFrame *in, AVFrame *out)
+static int apply_unsharp(AVFilterContext *ctx, AVFrame *in, AVFrame *out)
 {
     AVFilterLink *inlink = ctx->inputs[0];
     UnsharpContext *s = ctx->priv;
@@ -171,7 +201,10 @@ static int apply_unsharp_c(AVFilterContext *ctx, AVFrame *in, AVFrame *out)
     return 0;
 }
 
-static void set_filter_param(UnsharpFilterParam *fp, int msize_x, int msize_y, float amount)
+#define MAX_SCALEBITS 25
+
+static int set_filter_param(AVFilterContext *ctx, const char *name, const char *short_name,
+                            UnsharpFilterParam *fp, int msize_x, int msize_y, float amount)
 {
     fp->msize_x = msize_x;
     fp->msize_y = msize_y;
@@ -181,21 +214,31 @@ static void set_filter_param(UnsharpFilterParam *fp, int msize_x, int msize_y, f
     fp->steps_y = msize_y / 2;
     fp->scalebits = (fp->steps_x + fp->steps_y) * 2;
     fp->halfscale = 1 << (fp->scalebits - 1);
+
+    if (fp->scalebits > MAX_SCALEBITS) {
+        av_log(ctx, AV_LOG_ERROR, "%s matrix size (%sx/2+%sy/2)*2=%d greater than maximum value %d\n",
+               name, short_name, short_name, fp->scalebits, MAX_SCALEBITS);
+        return AVERROR(EINVAL);
+    }
+
+    return 0;
 }
 
 static av_cold int init(AVFilterContext *ctx)
 {
     UnsharpContext *s = ctx->priv;
+    int ret;
 
-    set_filter_param(&s->luma,   s->lmsize_x, s->lmsize_y, s->lamount);
-    set_filter_param(&s->chroma, s->cmsize_x, s->cmsize_y, s->camount);
-    set_filter_param(&s->alpha,  s->amsize_x, s->amsize_y, s->aamount);
+#define SET_FILTER_PARAM(name_, short_)                                 \
+    ret = set_filter_param(ctx, #name_, #short_, &s->name_,             \
+                           s->short_##msize_x, s->short_##msize_y, s->short_##amount); \
+    if (ret < 0)                                                        \
+        return ret;                                                     \
+
+    SET_FILTER_PARAM(luma, l);
+    SET_FILTER_PARAM(chroma, c);
+    SET_FILTER_PARAM(alpha, a);
 
-    if (s->luma.scalebits >= 26 || s->chroma.scalebits >= 26 || s->alpha.scalebits >= 26) {
-        av_log(ctx, AV_LOG_ERROR, "luma or chroma or alpha matrix size too big\n");
-        return AVERROR(EINVAL);
-    }
-    s->apply_unsharp = apply_unsharp_c;
     return 0;
 }
 
@@ -292,7 +335,6 @@ static av_cold void uninit(AVFilterContext *ctx)
 
 static int filter_frame(AVFilterLink *link, AVFrame *in)
 {
-    UnsharpContext *s = link->dst->priv;
     AVFilterLink *outlink   = link->dst->outputs[0];
     AVFrame *out;
     int ret = 0;
@@ -304,7 +346,7 @@ static int filter_frame(AVFilterLink *link, AVFrame *in)
     }
     av_frame_copy_props(out, in);
 
-    ret = s->apply_unsharp(link->dst, in, out);
+    ret = apply_unsharp(link->dst, in, out);
 
     av_frame_free(&in);
 
@@ -352,13 +394,6 @@ static const AVFilterPad avfilter_vf_unsharp_inputs[] = {
     },
 };
 
-static const AVFilterPad avfilter_vf_unsharp_outputs[] = {
-    {
-        .name = "default",
-        .type = AVMEDIA_TYPE_VIDEO,
-    },
-};
-
 const AVFilter ff_vf_unsharp = {
     .name          = "unsharp",
     .description   = NULL_IF_CONFIG_SMALL("Sharpen or blur the input video."),
@@ -367,7 +402,7 @@ const AVFilter ff_vf_unsharp = {
     .init          = init,
     .uninit        = uninit,
     FILTER_INPUTS(avfilter_vf_unsharp_inputs),
-    FILTER_OUTPUTS(avfilter_vf_unsharp_outputs),
+    FILTER_OUTPUTS(ff_video_default_filterpad),
     FILTER_PIXFMTS_ARRAY(pix_fmts),
     .flags         = AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC | AVFILTER_FLAG_SLICE_THREADS,
 };
