@@ -20,7 +20,8 @@
 
 #include "libavutil/common.h"
 #include "libavutil/opt.h"
-#include "internal.h"
+
+#include "filters.h"
 #include "video.h"
 
 typedef struct VFRDETContext {
