@@ -27,6 +27,7 @@
 #include "libavcodec/get_bits.h"
 
 #include "avformat.h"
+#include "demux.h"
 #include "rawdec.h"
 
 static int dts_probe(const AVProbeData *p)
@@ -132,15 +133,15 @@ static int dts_probe(const AVProbeData *p)
     return 0;
 }
 
-const AVInputFormat ff_dts_demuxer = {
-    .name           = "dts",
-    .long_name      = NULL_IF_CONFIG_SMALL("raw DTS"),
+const FFInputFormat ff_dts_demuxer = {
+    .p.name         = "dts",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("raw DTS"),
+    .p.flags        = AVFMT_GENERIC_INDEX,
+    .p.extensions   = "dts",
+    .p.priv_class   = &ff_raw_demuxer_class,
     .read_probe     = dts_probe,
     .read_header    = ff_raw_audio_read_header,
     .read_packet    = ff_raw_read_partial_packet,
-    .flags          = AVFMT_GENERIC_INDEX,
-    .extensions     = "dts",
     .raw_codec_id   = AV_CODEC_ID_DTS,
     .priv_data_size = sizeof(FFRawDemuxerContext),
-    .priv_class     = &ff_raw_demuxer_class,
 };
