@@ -20,6 +20,9 @@
  */
 
 #include "libavutil/channel_layout.h"
+#include "libavutil/mem.h"
+#include "libavcodec/avcodec.h"
+#include "libavcodec/bytestream.h"
 #include "libavcodec/flac.h"
 #include "avformat.h"
 #include "demux.h"
@@ -34,6 +37,8 @@
 typedef struct FLACDecContext {
     FFRawDemuxerContext rawctx;
     int found_seektable;
+
+    AVCodecContext *parser_dec;
 } FLACDecContext;
 
 static void reset_index_position(int64_t metadata_head_size, AVStream *st)
@@ -44,11 +49,16 @@ static void reset_index_position(int64_t metadata_head_size, AVStream *st)
         sti->index_entries[i].pos += metadata_head_size;
 }
 
+static const uint16_t sr_table[16] = {
+    0, 1764, 3528, 3840, 160, 320, 441, 480, 640, 882, 960, 1920, 0, 0, 0, 0
+};
+
 static int flac_read_header(AVFormatContext *s)
 {
     int ret, metadata_last=0, metadata_type, metadata_size, found_streaminfo=0;
     uint8_t header[4];
     uint8_t *buffer=NULL;
+    uint32_t marker;
     FLACDecContext *flac = s->priv_data;
     AVStream *st = avformat_new_stream(s, NULL);
     if (!st)
@@ -59,7 +69,11 @@ static int flac_read_header(AVFormatContext *s)
     /* the parameters will be extracted from the compressed bitstream */
 
     /* if fLaC marker is not found, assume there is no header */
-    if (avio_rl32(s->pb) != MKTAG('f','L','a','C')) {
+    marker = avio_rl32(s->pb);
+    if (marker != MKTAG('f','L','a','C')) {
+        const int sample_rate = 50 * sr_table[(marker >> 16) & 0xF];
+        if (sample_rate)
+            avpriv_set_pts_info(st, 64, 1, sample_rate);
         avio_seek(s->pb, -4, SEEK_CUR);
         return 0;
     }
@@ -258,6 +272,7 @@ static int flac_probe(const AVProbeData *p)
 static av_unused int64_t flac_read_timestamp(AVFormatContext *s, int stream_index,
                                              int64_t *ppos, int64_t pos_limit)
 {
+    FLACDecContext *flac = s->priv_data;
     FFFormatContext *const si = ffformatcontext(s);
     AVPacket *const pkt = si->parse_pkt;
     AVStream *st = s->streams[stream_index];
@@ -268,10 +283,19 @@ static av_unused int64_t flac_read_timestamp(AVFormatContext *s, int stream_inde
     if (avio_seek(s->pb, *ppos, SEEK_SET) < 0)
         return AV_NOPTS_VALUE;
 
-    parser = av_parser_init(st->codecpar->codec_id);
-    if (!parser){
-        return AV_NOPTS_VALUE;
+    if (!flac->parser_dec) {
+        flac->parser_dec = avcodec_alloc_context3(NULL);
+        if (!flac->parser_dec)
+            return AV_NOPTS_VALUE;
+
+        ret = avcodec_parameters_to_context(flac->parser_dec, st->codecpar);
+        if (ret < 0)
+            return ret;
     }
+
+    parser = av_parser_init(st->codecpar->codec_id);
+    if (!parser)
+        return AV_NOPTS_VALUE;
     parser->flags |= PARSER_FLAG_USE_CODEC_TS;
 
     for (;;){
@@ -287,7 +311,7 @@ static av_unused int64_t flac_read_timestamp(AVFormatContext *s, int stream_inde
                 av_assert1(!pkt->size);
             }
         }
-        av_parser_parse2(parser, ffstream(st)->avctx,
+        av_parser_parse2(parser, flac->parser_dec,
                          &data, &size, pkt->data, pkt->size,
                          pkt->pts, pkt->dts, *ppos);
 
@@ -307,6 +331,15 @@ static av_unused int64_t flac_read_timestamp(AVFormatContext *s, int stream_inde
     return pts;
 }
 
+static int flac_close(AVFormatContext *s)
+{
+    FLACDecContext *flac = s->priv_data;
+
+    avcodec_free_context(&flac->parser_dec);
+
+    return 0;
+}
+
 static int flac_seek(AVFormatContext *s, int stream_index, int64_t timestamp, int flags) {
     AVStream *const st  = s->streams[0];
     FFStream *const sti = ffstream(st);
@@ -331,17 +364,18 @@ static int flac_seek(AVFormatContext *s, int stream_index, int64_t timestamp, in
     return -1;
 }
 
-const AVInputFormat ff_flac_demuxer = {
-    .name           = "flac",
-    .long_name      = NULL_IF_CONFIG_SMALL("raw FLAC"),
+const FFInputFormat ff_flac_demuxer = {
+    .p.name         = "flac",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("raw FLAC"),
+    .p.flags        = AVFMT_GENERIC_INDEX,
+    .p.extensions   = "flac",
+    .p.priv_class   = &ff_raw_demuxer_class,
     .read_probe     = flac_probe,
     .read_header    = flac_read_header,
+    .read_close     = flac_close,
     .read_packet    = ff_raw_read_partial_packet,
     .read_seek      = flac_seek,
     .read_timestamp = flac_read_timestamp,
-    .flags          = AVFMT_GENERIC_INDEX,
-    .extensions     = "flac",
     .raw_codec_id   = AV_CODEC_ID_FLAC,
     .priv_data_size = sizeof(FLACDecContext),
-    .priv_class     = &ff_raw_demuxer_class,
 };
