@@ -20,21 +20,46 @@
 
 #include "libavcodec/get_bits.h"
 #include "libavcodec/golomb.h"
-#include "libavcodec/hevc.h"
+#include "libavcodec/hevc/hevc.h"
 #include "libavutil/intreadwrite.h"
+#include "libavutil/mem.h"
 #include "avc.h"
 #include "avio.h"
 #include "avio_internal.h"
 #include "hevc.h"
+#include "nal.h"
 
 #define MAX_SPATIAL_SEGMENTATION 4096 // max. value of u(12) field
 
+enum {
+    VPS_INDEX,
+    SPS_INDEX,
+    PPS_INDEX,
+    SEI_PREFIX_INDEX,
+    SEI_SUFFIX_INDEX,
+    NB_ARRAYS
+};
+
+
+#define FLAG_ARRAY_COMPLETENESS (1 << 0)
+#define FLAG_IS_NALFF           (1 << 1)
+#define FLAG_IS_LHVC            (1 << 2)
+
+typedef struct HVCCNALUnit {
+    uint8_t nuh_layer_id;
+    uint8_t parameter_set_id;
+    uint16_t nalUnitLength;
+    const uint8_t *nalUnit;
+
+    // VPS
+    uint8_t vps_max_sub_layers_minus1;
+} HVCCNALUnit;
+
 typedef struct HVCCNALUnitArray {
     uint8_t  array_completeness;
     uint8_t  NAL_unit_type;
     uint16_t numNalus;
-    uint16_t *nalUnitLength;
-    uint8_t  **nalUnit;
+    HVCCNALUnit *nal;
 } HVCCNALUnitArray;
 
 typedef struct HEVCDecoderConfigurationRecord {
@@ -56,7 +81,7 @@ typedef struct HEVCDecoderConfigurationRecord {
     uint8_t  temporalIdNested;
     uint8_t  lengthSizeMinusOne;
     uint8_t  numOfArrays;
-    HVCCNALUnitArray *array;
+    HVCCNALUnitArray arrays[NB_ARRAYS];
 } HEVCDecoderConfigurationRecord;
 
 typedef struct HVCCProfileTierLevel {
@@ -359,19 +384,17 @@ static void skip_sub_layer_ordering_info(GetBitContext *gb)
     get_ue_golomb_long(gb); // max_latency_increase_plus1
 }
 
-static int hvcc_parse_vps(GetBitContext *gb,
+static int hvcc_parse_vps(GetBitContext *gb, HVCCNALUnit *nal,
                           HEVCDecoderConfigurationRecord *hvcc)
 {
-    unsigned int vps_max_sub_layers_minus1;
-
+    nal->parameter_set_id = get_bits(gb, 4);
     /*
-     * vps_video_parameter_set_id u(4)
      * vps_reserved_three_2bits   u(2)
      * vps_max_layers_minus1      u(6)
      */
-    skip_bits(gb, 12);
+    skip_bits(gb, 8);
 
-    vps_max_sub_layers_minus1 = get_bits(gb, 3);
+    nal->vps_max_sub_layers_minus1 = get_bits(gb, 3);
 
     /*
      * numTemporalLayers greater than 1 indicates that the stream to which this
@@ -382,7 +405,7 @@ static int hvcc_parse_vps(GetBitContext *gb,
      * that it is unknown whether the stream is temporally scalable.
      */
     hvcc->numTemporalLayers = FFMAX(hvcc->numTemporalLayers,
-                                    vps_max_sub_layers_minus1 + 1);
+                                    nal->vps_max_sub_layers_minus1 + 1);
 
     /*
      * vps_temporal_id_nesting_flag u(1)
@@ -390,7 +413,7 @@ static int hvcc_parse_vps(GetBitContext *gb,
      */
     skip_bits(gb, 17);
 
-    hvcc_parse_ptl(gb, hvcc, vps_max_sub_layers_minus1);
+    hvcc_parse_ptl(gb, hvcc, nal->vps_max_sub_layers_minus1);
 
     /* nothing useful for hvcC past this point */
     return 0;
@@ -482,15 +505,38 @@ static int parse_rps(GetBitContext *gb, unsigned int rps_idx,
     return 0;
 }
 
-static int hvcc_parse_sps(GetBitContext *gb,
+static int hvcc_parse_sps(GetBitContext *gb, HVCCNALUnit *nal,
                           HEVCDecoderConfigurationRecord *hvcc)
 {
     unsigned int i, sps_max_sub_layers_minus1, log2_max_pic_order_cnt_lsb_minus4;
     unsigned int num_short_term_ref_pic_sets, num_delta_pocs[HEVC_MAX_SHORT_TERM_REF_PIC_SETS];
+    unsigned int sps_ext_or_max_sub_layers_minus1, multi_layer_ext_sps_flag;
 
-    skip_bits(gb, 4); // sps_video_parameter_set_id
+    unsigned int sps_video_parameter_set_id = get_bits(gb, 4);
 
-    sps_max_sub_layers_minus1 = get_bits (gb, 3);
+    if (nal->nuh_layer_id == 0) {
+        sps_ext_or_max_sub_layers_minus1 = 0;
+        sps_max_sub_layers_minus1 = get_bits(gb, 3);
+    } else {
+        sps_ext_or_max_sub_layers_minus1 = get_bits(gb, 3);
+        if (sps_ext_or_max_sub_layers_minus1 == 7) {
+            const HVCCNALUnitArray *array = &hvcc->arrays[VPS_INDEX];
+            const HVCCNALUnit *vps = NULL;
+
+            for (i = 0; i < array->numNalus; i++)
+                if (sps_video_parameter_set_id == array->nal[i].parameter_set_id) {
+                    vps = &array->nal[i];
+                    break;
+                }
+            if (!vps)
+                return AVERROR_INVALIDDATA;
+
+            sps_max_sub_layers_minus1 = vps->vps_max_sub_layers_minus1;
+        } else
+            sps_max_sub_layers_minus1 = sps_ext_or_max_sub_layers_minus1;
+    }
+    multi_layer_ext_sps_flag = nal->nuh_layer_id &&
+                               sps_ext_or_max_sub_layers_minus1 == 7;
 
     /*
      * numTemporalLayers greater than 1 indicates that the stream to which this
@@ -503,35 +549,43 @@ static int hvcc_parse_sps(GetBitContext *gb,
     hvcc->numTemporalLayers = FFMAX(hvcc->numTemporalLayers,
                                     sps_max_sub_layers_minus1 + 1);
 
-    hvcc->temporalIdNested = get_bits1(gb);
-
-    hvcc_parse_ptl(gb, hvcc, sps_max_sub_layers_minus1);
-
-    get_ue_golomb_long(gb); // sps_seq_parameter_set_id
-
-    hvcc->chromaFormat = get_ue_golomb_long(gb);
-
-    if (hvcc->chromaFormat == 3)
-        skip_bits1(gb); // separate_colour_plane_flag
-
-    get_ue_golomb_long(gb); // pic_width_in_luma_samples
-    get_ue_golomb_long(gb); // pic_height_in_luma_samples
-
-    if (get_bits1(gb)) {        // conformance_window_flag
-        get_ue_golomb_long(gb); // conf_win_left_offset
-        get_ue_golomb_long(gb); // conf_win_right_offset
-        get_ue_golomb_long(gb); // conf_win_top_offset
-        get_ue_golomb_long(gb); // conf_win_bottom_offset
+    if (!multi_layer_ext_sps_flag) {
+        hvcc->temporalIdNested = get_bits1(gb);
+        hvcc_parse_ptl(gb, hvcc, sps_max_sub_layers_minus1);
     }
 
-    hvcc->bitDepthLumaMinus8          = get_ue_golomb_long(gb);
-    hvcc->bitDepthChromaMinus8        = get_ue_golomb_long(gb);
+    nal->parameter_set_id = get_ue_golomb_long(gb);
+
+    if (multi_layer_ext_sps_flag) {
+        if (get_bits1(gb)) // update_rep_format_flag
+            skip_bits(gb, 8); // sps_rep_format_idx
+    } else {
+        hvcc->chromaFormat = get_ue_golomb_long(gb);
+
+        if (hvcc->chromaFormat == 3)
+            skip_bits1(gb); // separate_colour_plane_flag
+
+        get_ue_golomb_long(gb); // pic_width_in_luma_samples
+        get_ue_golomb_long(gb); // pic_height_in_luma_samples
+
+        if (get_bits1(gb)) {        // conformance_window_flag
+            get_ue_golomb_long(gb); // conf_win_left_offset
+            get_ue_golomb_long(gb); // conf_win_right_offset
+            get_ue_golomb_long(gb); // conf_win_top_offset
+            get_ue_golomb_long(gb); // conf_win_bottom_offset
+        }
+
+        hvcc->bitDepthLumaMinus8          = get_ue_golomb_long(gb);
+        hvcc->bitDepthChromaMinus8        = get_ue_golomb_long(gb);
+    }
     log2_max_pic_order_cnt_lsb_minus4 = get_ue_golomb_long(gb);
 
-    /* sps_sub_layer_ordering_info_present_flag */
-    i = get_bits1(gb) ? 0 : sps_max_sub_layers_minus1;
-    for (; i <= sps_max_sub_layers_minus1; i++)
-        skip_sub_layer_ordering_info(gb);
+    if (!multi_layer_ext_sps_flag) {
+        /* sps_sub_layer_ordering_info_present_flag */
+        i = get_bits1(gb) ? 0 : sps_max_sub_layers_minus1;
+        for (; i <= sps_max_sub_layers_minus1; i++)
+            skip_sub_layer_ordering_info(gb);
+    }
 
     get_ue_golomb_long(gb); // log2_min_luma_coding_block_size_minus3
     get_ue_golomb_long(gb); // log2_diff_max_min_luma_coding_block_size
@@ -540,9 +594,15 @@ static int hvcc_parse_sps(GetBitContext *gb,
     get_ue_golomb_long(gb); // max_transform_hierarchy_depth_inter
     get_ue_golomb_long(gb); // max_transform_hierarchy_depth_intra
 
-    if (get_bits1(gb) && // scaling_list_enabled_flag
-        get_bits1(gb))   // sps_scaling_list_data_present_flag
-        skip_scaling_list_data(gb);
+    if (get_bits1(gb)) { // scaling_list_enabled_flag
+        int sps_infer_scaling_list_flag = 0;
+        if (multi_layer_ext_sps_flag)
+            sps_infer_scaling_list_flag = get_bits1(gb);
+        if (sps_infer_scaling_list_flag)
+            skip_bits(gb, 6);   // sps_scaling_list_ref_layer_id
+        else if (get_bits1(gb)) // sps_scaling_list_data_present_flag
+            skip_scaling_list_data(gb);
+    }
 
     skip_bits1(gb); // amp_enabled_flag
     skip_bits1(gb); // sample_adaptive_offset_enabled_flag
@@ -586,12 +646,12 @@ static int hvcc_parse_sps(GetBitContext *gb,
     return 0;
 }
 
-static int hvcc_parse_pps(GetBitContext *gb,
+static int hvcc_parse_pps(GetBitContext *gb, HVCCNALUnit *nal,
                           HEVCDecoderConfigurationRecord *hvcc)
 {
     uint8_t tiles_enabled_flag, entropy_coding_sync_enabled_flag;
 
-    get_ue_golomb_long(gb); // pps_pic_parameter_set_id
+    nal->parameter_set_id = get_ue_golomb_long(gb); // pps_pic_parameter_set_id
     get_ue_golomb_long(gb); // pps_seq_parameter_set_id
 
     /*
@@ -643,79 +703,51 @@ static int hvcc_parse_pps(GetBitContext *gb,
     return 0;
 }
 
-static void nal_unit_parse_header(GetBitContext *gb, uint8_t *nal_type)
+static void nal_unit_parse_header(GetBitContext *gb, uint8_t *nal_type,
+                                  uint8_t *nuh_layer_id)
 {
     skip_bits1(gb); // forbidden_zero_bit
 
     *nal_type = get_bits(gb, 6);
+    *nuh_layer_id = get_bits(gb, 6);
 
     /*
-     * nuh_layer_id          u(6)
      * nuh_temporal_id_plus1 u(3)
      */
-    skip_bits(gb, 9);
+    skip_bits(gb, 3);
 }
 
-static int hvcc_array_add_nal_unit(uint8_t *nal_buf, uint32_t nal_size,
-                                   uint8_t nal_type, int ps_array_completeness,
-                                   HEVCDecoderConfigurationRecord *hvcc)
+static int hvcc_array_add_nal_unit(const uint8_t *nal_buf, uint32_t nal_size,
+                                   HVCCNALUnitArray *array)
 {
+    HVCCNALUnit *nal;
     int ret;
-    uint8_t index;
-    uint16_t numNalus;
-    HVCCNALUnitArray *array;
+    uint16_t numNalus = array->numNalus;
 
-    for (index = 0; index < hvcc->numOfArrays; index++)
-        if (hvcc->array[index].NAL_unit_type == nal_type)
-            break;
-
-    if (index >= hvcc->numOfArrays) {
-        uint8_t i;
-
-        ret = av_reallocp_array(&hvcc->array, index + 1, sizeof(HVCCNALUnitArray));
-        if (ret < 0)
-            return ret;
-
-        for (i = hvcc->numOfArrays; i <= index; i++)
-            memset(&hvcc->array[i], 0, sizeof(HVCCNALUnitArray));
-        hvcc->numOfArrays = index + 1;
-    }
-
-    array    = &hvcc->array[index];
-    numNalus = array->numNalus;
-
-    ret = av_reallocp_array(&array->nalUnit, numNalus + 1, sizeof(uint8_t*));
+    ret = av_reallocp_array(&array->nal, numNalus + 1, sizeof(*array->nal));
     if (ret < 0)
         return ret;
 
-    ret = av_reallocp_array(&array->nalUnitLength, numNalus + 1, sizeof(uint16_t));
-    if (ret < 0)
-        return ret;
-
-    array->nalUnit      [numNalus] = nal_buf;
-    array->nalUnitLength[numNalus] = nal_size;
-    array->NAL_unit_type           = nal_type;
+    nal = &array->nal[numNalus];
+    nal->nalUnit       = nal_buf;
+    nal->nalUnitLength = nal_size;
     array->numNalus++;
 
-    /*
-     * When the sample entry name is ‘hvc1’, the default and mandatory value of
-     * array_completeness is 1 for arrays of all types of parameter sets, and 0
-     * for all other arrays. When the sample entry name is ‘hev1’, the default
-     * value of array_completeness is 0 for all arrays.
-     */
-    if (nal_type == HEVC_NAL_VPS || nal_type == HEVC_NAL_SPS || nal_type == HEVC_NAL_PPS)
-        array->array_completeness = ps_array_completeness;
-
     return 0;
 }
 
-static int hvcc_add_nal_unit(uint8_t *nal_buf, uint32_t nal_size,
-                             int ps_array_completeness,
-                             HEVCDecoderConfigurationRecord *hvcc)
+static int hvcc_add_nal_unit(const uint8_t *nal_buf, uint32_t nal_size,
+                             HEVCDecoderConfigurationRecord *hvcc,
+                             int flags, unsigned array_idx)
 {
     int ret = 0;
+    int is_nalff = !!(flags & FLAG_IS_NALFF);
+    int is_lhvc = !!(flags & FLAG_IS_LHVC);
+    int ps_array_completeness = !!(flags & FLAG_ARRAY_COMPLETENESS);
+    HVCCNALUnitArray *const array = &hvcc->arrays[array_idx];
+    HVCCNALUnit *nal;
     GetBitContext gbc;
-    uint8_t nal_type;
+    uint8_t nal_type, nuh_layer_id;
     uint8_t *rbsp_buf;
     uint32_t rbsp_size;
 
@@ -729,37 +761,49 @@ static int hvcc_add_nal_unit(uint8_t *nal_buf, uint32_t nal_size,
     if (ret < 0)
         goto end;
 
-    nal_unit_parse_header(&gbc, &nal_type);
+    nal_unit_parse_header(&gbc, &nal_type, &nuh_layer_id);
+    if (!is_lhvc && nuh_layer_id > 0)
+        goto end;
 
     /*
      * Note: only 'declarative' SEI messages are allowed in
      * hvcC. Perhaps the SEI playload type should be checked
      * and non-declarative SEI messages discarded?
      */
-    switch (nal_type) {
-    case HEVC_NAL_VPS:
-    case HEVC_NAL_SPS:
-    case HEVC_NAL_PPS:
-    case HEVC_NAL_SEI_PREFIX:
-    case HEVC_NAL_SEI_SUFFIX:
-        ret = hvcc_array_add_nal_unit(nal_buf, nal_size, nal_type,
-                                      ps_array_completeness, hvcc);
-        if (ret < 0)
-            goto end;
-        else if (nal_type == HEVC_NAL_VPS)
-            ret = hvcc_parse_vps(&gbc, hvcc);
-        else if (nal_type == HEVC_NAL_SPS)
-            ret = hvcc_parse_sps(&gbc, hvcc);
-        else if (nal_type == HEVC_NAL_PPS)
-            ret = hvcc_parse_pps(&gbc, hvcc);
-        if (ret < 0)
-            goto end;
-        break;
-    default:
-        ret = AVERROR_INVALIDDATA;
+    ret = hvcc_array_add_nal_unit(nal_buf, nal_size, array);
+    if (ret < 0)
         goto end;
+    if (array->numNalus == 1) {
+        hvcc->numOfArrays++;
+        array->NAL_unit_type = nal_type;
+
+        /*
+         * When the sample entry name is ‘hvc1’, the default and mandatory value of
+         * array_completeness is 1 for arrays of all types of parameter sets, and 0
+         * for all other arrays. When the sample entry name is ‘hev1’, the default
+         * value of array_completeness is 0 for all arrays.
+         */
+        if (nal_type == HEVC_NAL_VPS || nal_type == HEVC_NAL_SPS ||
+            nal_type == HEVC_NAL_PPS)
+            array->array_completeness = ps_array_completeness;
     }
 
+    nal = &array->nal[array->numNalus-1];
+    nal->nuh_layer_id = nuh_layer_id;
+
+    /* Don't parse parameter sets. We already have the needed information*/
+    if (is_nalff)
+        goto end;
+
+    if (nal_type == HEVC_NAL_VPS)
+        ret = hvcc_parse_vps(&gbc, nal, hvcc);
+    else if (nal_type == HEVC_NAL_SPS)
+        ret = hvcc_parse_sps(&gbc, nal, hvcc);
+    else if (nal_type == HEVC_NAL_PPS)
+        ret = hvcc_parse_pps(&gbc, nal, hvcc);
+    if (ret < 0)
+        goto end;
+
 end:
     av_free(rbsp_buf);
     return ret;
@@ -787,22 +831,19 @@ static void hvcc_init(HEVCDecoderConfigurationRecord *hvcc)
 
 static void hvcc_close(HEVCDecoderConfigurationRecord *hvcc)
 {
-    uint8_t i;
-
-    for (i = 0; i < hvcc->numOfArrays; i++) {
-        hvcc->array[i].numNalus = 0;
-        av_freep(&hvcc->array[i].nalUnit);
-        av_freep(&hvcc->array[i].nalUnitLength);
+    for (unsigned i = 0; i < FF_ARRAY_ELEMS(hvcc->arrays); i++) {
+        HVCCNALUnitArray *const array = &hvcc->arrays[i];
+        array->numNalus = 0;
+        av_freep(&array->nal);
     }
-
-    hvcc->numOfArrays = 0;
-    av_freep(&hvcc->array);
 }
 
-static int hvcc_write(AVIOContext *pb, HEVCDecoderConfigurationRecord *hvcc)
+static int hvcc_write(AVIOContext *pb, HEVCDecoderConfigurationRecord *hvcc,
+                      int flags)
 {
-    uint8_t i;
-    uint16_t j, vps_count = 0, sps_count = 0, pps_count = 0;
+    uint16_t numNalus[NB_ARRAYS] = { 0 };
+    int is_lhvc = !!(flags & FLAG_IS_LHVC);
+    int numOfArrays = 0;
 
     /*
      * We only support writing HEVCDecoderConfigurationRecord version 1.
@@ -828,36 +869,61 @@ static int hvcc_write(AVIOContext *pb, HEVCDecoderConfigurationRecord *hvcc)
      * let's always set them to values meaning 'unspecified'.
      */
     hvcc->avgFrameRate      = 0;
-    hvcc->constantFrameRate = 0;
+    /*
+     * lhvC doesn't store this field. It instead reserves the bits, setting them
+     * to '11'b.
+     */
+    hvcc->constantFrameRate = is_lhvc * 0x3;
 
+    /*
+     * Skip all NALUs with nuh_layer_id == 0 if writing lhvC. We do it here and
+     * not before parsing them as some parameter sets with nuh_layer_id > 0
+     * may reference base layer parameters sets.
+     */
+    for (unsigned i = 0; i < FF_ARRAY_ELEMS(hvcc->arrays); i++) {
+        const HVCCNALUnitArray *const array = &hvcc->arrays[i];
+
+        if (array->numNalus == 0)
+            continue;
+
+        for (unsigned j = 0; j < array->numNalus; j++)
+            numNalus[i] += !is_lhvc || (array->nal[j].nuh_layer_id != 0);
+        numOfArrays += (numNalus[i] > 0);
+    }
+
+    av_log(NULL, AV_LOG_TRACE,  "%s\n", is_lhvc ? "lhvC" : "hvcC");
     av_log(NULL, AV_LOG_TRACE,  "configurationVersion:                %"PRIu8"\n",
             hvcc->configurationVersion);
-    av_log(NULL, AV_LOG_TRACE,  "general_profile_space:               %"PRIu8"\n",
-            hvcc->general_profile_space);
-    av_log(NULL, AV_LOG_TRACE,  "general_tier_flag:                   %"PRIu8"\n",
-            hvcc->general_tier_flag);
-    av_log(NULL, AV_LOG_TRACE,  "general_profile_idc:                 %"PRIu8"\n",
-            hvcc->general_profile_idc);
-    av_log(NULL, AV_LOG_TRACE, "general_profile_compatibility_flags: 0x%08"PRIx32"\n",
-            hvcc->general_profile_compatibility_flags);
-    av_log(NULL, AV_LOG_TRACE, "general_constraint_indicator_flags:  0x%012"PRIx64"\n",
-            hvcc->general_constraint_indicator_flags);
-    av_log(NULL, AV_LOG_TRACE,  "general_level_idc:                   %"PRIu8"\n",
-            hvcc->general_level_idc);
+    if (!is_lhvc) {
+        av_log(NULL, AV_LOG_TRACE,  "general_profile_space:               %"PRIu8"\n",
+                hvcc->general_profile_space);
+        av_log(NULL, AV_LOG_TRACE,  "general_tier_flag:                   %"PRIu8"\n",
+                hvcc->general_tier_flag);
+        av_log(NULL, AV_LOG_TRACE,  "general_profile_idc:                 %"PRIu8"\n",
+                hvcc->general_profile_idc);
+        av_log(NULL, AV_LOG_TRACE, "general_profile_compatibility_flags: 0x%08"PRIx32"\n",
+                hvcc->general_profile_compatibility_flags);
+        av_log(NULL, AV_LOG_TRACE, "general_constraint_indicator_flags:  0x%012"PRIx64"\n",
+                hvcc->general_constraint_indicator_flags);
+        av_log(NULL, AV_LOG_TRACE,  "general_level_idc:                   %"PRIu8"\n",
+                hvcc->general_level_idc);
+    }
     av_log(NULL, AV_LOG_TRACE,  "min_spatial_segmentation_idc:        %"PRIu16"\n",
             hvcc->min_spatial_segmentation_idc);
     av_log(NULL, AV_LOG_TRACE,  "parallelismType:                     %"PRIu8"\n",
             hvcc->parallelismType);
-    av_log(NULL, AV_LOG_TRACE,  "chromaFormat:                        %"PRIu8"\n",
-            hvcc->chromaFormat);
-    av_log(NULL, AV_LOG_TRACE,  "bitDepthLumaMinus8:                  %"PRIu8"\n",
-            hvcc->bitDepthLumaMinus8);
-    av_log(NULL, AV_LOG_TRACE,  "bitDepthChromaMinus8:                %"PRIu8"\n",
-            hvcc->bitDepthChromaMinus8);
-    av_log(NULL, AV_LOG_TRACE,  "avgFrameRate:                        %"PRIu16"\n",
-            hvcc->avgFrameRate);
-    av_log(NULL, AV_LOG_TRACE,  "constantFrameRate:                   %"PRIu8"\n",
-            hvcc->constantFrameRate);
+    if (!is_lhvc) {
+        av_log(NULL, AV_LOG_TRACE,  "chromaFormat:                        %"PRIu8"\n",
+                hvcc->chromaFormat);
+        av_log(NULL, AV_LOG_TRACE,  "bitDepthLumaMinus8:                  %"PRIu8"\n",
+                hvcc->bitDepthLumaMinus8);
+        av_log(NULL, AV_LOG_TRACE,  "bitDepthChromaMinus8:                %"PRIu8"\n",
+                hvcc->bitDepthChromaMinus8);
+        av_log(NULL, AV_LOG_TRACE,  "avgFrameRate:                        %"PRIu16"\n",
+                hvcc->avgFrameRate);
+        av_log(NULL, AV_LOG_TRACE,  "constantFrameRate:                   %"PRIu8"\n",
+                hvcc->constantFrameRate);
+    }
     av_log(NULL, AV_LOG_TRACE,  "numTemporalLayers:                   %"PRIu8"\n",
             hvcc->numTemporalLayers);
     av_log(NULL, AV_LOG_TRACE,  "temporalIdNested:                    %"PRIu8"\n",
@@ -865,99 +931,105 @@ static int hvcc_write(AVIOContext *pb, HEVCDecoderConfigurationRecord *hvcc)
     av_log(NULL, AV_LOG_TRACE,  "lengthSizeMinusOne:                  %"PRIu8"\n",
             hvcc->lengthSizeMinusOne);
     av_log(NULL, AV_LOG_TRACE,  "numOfArrays:                         %"PRIu8"\n",
-            hvcc->numOfArrays);
-    for (i = 0; i < hvcc->numOfArrays; i++) {
-        av_log(NULL, AV_LOG_TRACE, "array_completeness[%"PRIu8"]:               %"PRIu8"\n",
-                i, hvcc->array[i].array_completeness);
-        av_log(NULL, AV_LOG_TRACE, "NAL_unit_type[%"PRIu8"]:                    %"PRIu8"\n",
-                i, hvcc->array[i].NAL_unit_type);
-        av_log(NULL, AV_LOG_TRACE, "numNalus[%"PRIu8"]:                         %"PRIu16"\n",
-                i, hvcc->array[i].numNalus);
-        for (j = 0; j < hvcc->array[i].numNalus; j++)
+            numOfArrays);
+    for (unsigned i = 0, j = 0; i < FF_ARRAY_ELEMS(hvcc->arrays); i++) {
+        const HVCCNALUnitArray *const array = &hvcc->arrays[i];
+
+        if (numNalus[i] == 0)
+            continue;
+
+        av_log(NULL, AV_LOG_TRACE, "array_completeness[%u]:               %"PRIu8"\n",
+               j, array->array_completeness);
+        av_log(NULL, AV_LOG_TRACE, "NAL_unit_type[%u]:                    %"PRIu8"\n",
+               j, array->NAL_unit_type);
+        av_log(NULL, AV_LOG_TRACE, "numNalus[%u]:                         %"PRIu16"\n",
+               j, numNalus[i]);
+        for (unsigned k = 0; k < array->numNalus; k++) {
+            if (is_lhvc && array->nal[k].nuh_layer_id == 0)
+                continue;
+
             av_log(NULL, AV_LOG_TRACE,
-                    "nalUnitLength[%"PRIu8"][%"PRIu16"]:                 %"PRIu16"\n",
-                    i, j, hvcc->array[i].nalUnitLength[j]);
+                    "nalUnitLength[%u][%u]:                 %"PRIu16"\n",
+                   j, k, array->nal[k].nalUnitLength);
+        }
+        j++;
     }
 
     /*
      * We need at least one of each: VPS, SPS and PPS.
      */
-    for (i = 0; i < hvcc->numOfArrays; i++)
-        switch (hvcc->array[i].NAL_unit_type) {
-        case HEVC_NAL_VPS:
-            vps_count += hvcc->array[i].numNalus;
-            break;
-        case HEVC_NAL_SPS:
-            sps_count += hvcc->array[i].numNalus;
-            break;
-        case HEVC_NAL_PPS:
-            pps_count += hvcc->array[i].numNalus;
-            break;
-        default:
-            break;
-        }
-    if (!vps_count || vps_count > HEVC_MAX_VPS_COUNT ||
-        !sps_count || sps_count > HEVC_MAX_SPS_COUNT ||
-        !pps_count || pps_count > HEVC_MAX_PPS_COUNT)
+    if ((flags & FLAG_ARRAY_COMPLETENESS) &&
+        (!numNalus[VPS_INDEX] || numNalus[VPS_INDEX] > HEVC_MAX_VPS_COUNT) && !is_lhvc)
+        return AVERROR_INVALIDDATA;
+    if ((flags & FLAG_ARRAY_COMPLETENESS) &&
+        (!numNalus[SPS_INDEX] || numNalus[SPS_INDEX] > HEVC_MAX_SPS_COUNT ||
+         !numNalus[PPS_INDEX] || numNalus[PPS_INDEX] > HEVC_MAX_PPS_COUNT))
         return AVERROR_INVALIDDATA;
 
     /* unsigned int(8) configurationVersion = 1; */
     avio_w8(pb, hvcc->configurationVersion);
 
-    /*
-     * unsigned int(2) general_profile_space;
-     * unsigned int(1) general_tier_flag;
-     * unsigned int(5) general_profile_idc;
-     */
-    avio_w8(pb, hvcc->general_profile_space << 6 |
-                hvcc->general_tier_flag     << 5 |
-                hvcc->general_profile_idc);
+    if (!is_lhvc) {
+        /*
+         * unsigned int(2) general_profile_space;
+         * unsigned int(1) general_tier_flag;
+         * unsigned int(5) general_profile_idc;
+         */
+        avio_w8(pb, hvcc->general_profile_space << 6 |
+                    hvcc->general_tier_flag     << 5 |
+                    hvcc->general_profile_idc);
 
-    /* unsigned int(32) general_profile_compatibility_flags; */
-    avio_wb32(pb, hvcc->general_profile_compatibility_flags);
+        /* unsigned int(32) general_profile_compatibility_flags; */
+        avio_wb32(pb, hvcc->general_profile_compatibility_flags);
 
-    /* unsigned int(48) general_constraint_indicator_flags; */
-    avio_wb32(pb, hvcc->general_constraint_indicator_flags >> 16);
-    avio_wb16(pb, hvcc->general_constraint_indicator_flags);
+        /* unsigned int(48) general_constraint_indicator_flags; */
+        avio_wb32(pb, hvcc->general_constraint_indicator_flags >> 16);
+        avio_wb16(pb, hvcc->general_constraint_indicator_flags);
 
-    /* unsigned int(8) general_level_idc; */
-    avio_w8(pb, hvcc->general_level_idc);
+        /* unsigned int(8) general_level_idc; */
+        avio_w8(pb, hvcc->general_level_idc);
+    }
 
     /*
-     * bit(4) reserved = ‘1111’b;
+     * bit(4) reserved = '1111'b;
      * unsigned int(12) min_spatial_segmentation_idc;
      */
     avio_wb16(pb, hvcc->min_spatial_segmentation_idc | 0xf000);
 
     /*
-     * bit(6) reserved = ‘111111’b;
+     * bit(6) reserved = '111111'b;
      * unsigned int(2) parallelismType;
      */
     avio_w8(pb, hvcc->parallelismType | 0xfc);
 
-    /*
-     * bit(6) reserved = ‘111111’b;
-     * unsigned int(2) chromaFormat;
-     */
-    avio_w8(pb, hvcc->chromaFormat | 0xfc);
+    if (!is_lhvc) {
+        /*
+         * bit(6) reserved = '111111'b;
+         * unsigned int(2) chromaFormat;
+         */
+        avio_w8(pb, hvcc->chromaFormat | 0xfc);
+
+        /*
+         * bit(5) reserved = '11111'b;
+         * unsigned int(3) bitDepthLumaMinus8;
+         */
+        avio_w8(pb, hvcc->bitDepthLumaMinus8 | 0xf8);
+
+        /*
+         * bit(5) reserved = '11111'b;
+         * unsigned int(3) bitDepthChromaMinus8;
+         */
+        avio_w8(pb, hvcc->bitDepthChromaMinus8 | 0xf8);
+
+        /* bit(16) avgFrameRate; */
+        avio_wb16(pb, hvcc->avgFrameRate);
+    }
 
     /*
-     * bit(5) reserved = ‘11111’b;
-     * unsigned int(3) bitDepthLumaMinus8;
-     */
-    avio_w8(pb, hvcc->bitDepthLumaMinus8 | 0xf8);
-
-    /*
-     * bit(5) reserved = ‘11111’b;
-     * unsigned int(3) bitDepthChromaMinus8;
-     */
-    avio_w8(pb, hvcc->bitDepthChromaMinus8 | 0xf8);
-
-    /* bit(16) avgFrameRate; */
-    avio_wb16(pb, hvcc->avgFrameRate);
-
-    /*
-     * bit(2) constantFrameRate;
+     * if (!is_lhvc)
+     *     bit(2) constantFrameRate;
+     * else
+     *     bit(2) reserved = '11'b;
      * bit(3) numTemporalLayers;
      * bit(1) temporalIdNested;
      * unsigned int(2) lengthSizeMinusOne;
@@ -968,27 +1040,35 @@ static int hvcc_write(AVIOContext *pb, HEVCDecoderConfigurationRecord *hvcc)
                 hvcc->lengthSizeMinusOne);
 
     /* unsigned int(8) numOfArrays; */
-    avio_w8(pb, hvcc->numOfArrays);
+    avio_w8(pb, numOfArrays);
 
-    for (i = 0; i < hvcc->numOfArrays; i++) {
+    for (unsigned i = 0; i < FF_ARRAY_ELEMS(hvcc->arrays); i++) {
+        const HVCCNALUnitArray *const array = &hvcc->arrays[i];
+
+        if (!numNalus[i])
+            continue;
         /*
          * bit(1) array_completeness;
          * unsigned int(1) reserved = 0;
          * unsigned int(6) NAL_unit_type;
          */
-        avio_w8(pb, hvcc->array[i].array_completeness << 7 |
-                    hvcc->array[i].NAL_unit_type & 0x3f);
+        avio_w8(pb, array->array_completeness << 7 |
+                    array->NAL_unit_type & 0x3f);
 
         /* unsigned int(16) numNalus; */
-        avio_wb16(pb, hvcc->array[i].numNalus);
+        avio_wb16(pb, numNalus[i]);
+
+        for (unsigned j = 0; j < array->numNalus; j++) {
+            HVCCNALUnit *nal = &array->nal[j];
+
+            if (is_lhvc && nal->nuh_layer_id == 0)
+                continue;
 
-        for (j = 0; j < hvcc->array[i].numNalus; j++) {
             /* unsigned int(16) nalUnitLength; */
-            avio_wb16(pb, hvcc->array[i].nalUnitLength[j]);
+            avio_wb16(pb, nal->nalUnitLength);
 
             /* bit(8*nalUnitLength) nalUnit; */
-            avio_write(pb, hvcc->array[i].nalUnit[j],
-                       hvcc->array[i].nalUnitLength[j]);
+            avio_write(pb, nal->nalUnit, nal->nalUnitLength);
         }
     }
 
@@ -1002,11 +1082,11 @@ int ff_hevc_annexb2mp4(AVIOContext *pb, const uint8_t *buf_in,
     uint8_t *buf, *end, *start = NULL;
 
     if (!filter_ps) {
-        ret = ff_avc_parse_nal_units(pb, buf_in, size);
+        ret = ff_nal_parse_units(pb, buf_in, size);
         goto end;
     }
 
-    ret = ff_avc_parse_nal_units_buf(buf_in, &start, &size);
+    ret = ff_nal_parse_units_buf(buf_in, &start, &size);
     if (ret < 0)
         goto end;
 
@@ -1064,26 +1144,105 @@ int ff_hevc_annexb2mp4_buf(const uint8_t *buf_in, uint8_t **buf_out,
     return 0;
 }
 
-int ff_isom_write_hvcc(AVIOContext *pb, const uint8_t *data,
-                       int size, int ps_array_completeness)
+static int hvcc_parse_nal_unit(const uint8_t *buf, uint32_t len, int type,
+                               HEVCDecoderConfigurationRecord *hvcc,
+                               int flags)
+{
+    for (unsigned i = 0; i < FF_ARRAY_ELEMS(hvcc->arrays); i++) {
+        static const uint8_t array_idx_to_type[] =
+            { HEVC_NAL_VPS, HEVC_NAL_SPS, HEVC_NAL_PPS,
+              HEVC_NAL_SEI_PREFIX, HEVC_NAL_SEI_SUFFIX };
+
+        if (type == array_idx_to_type[i]) {
+            int ret = hvcc_add_nal_unit(buf, len, hvcc, flags, i);
+            if (ret < 0)
+                return ret;
+            break;
+        }
+    }
+
+    return 0;
+}
+
+static int write_configuration_record(AVIOContext *pb, const uint8_t *data,
+                                      int size, int flags)
 {
     HEVCDecoderConfigurationRecord hvcc;
-    uint8_t *buf, *end, *start;
+    uint8_t *buf, *end, *start = NULL;
     int ret;
 
     if (size < 6) {
         /* We can't write a valid hvcC from the provided data */
         return AVERROR_INVALIDDATA;
     } else if (*data == 1) {
-        /* Data is already hvcC-formatted */
-        avio_write(pb, data, size);
-        return 0;
+        /* Data is already hvcC-formatted. Parse the arrays to skip any NALU
+           with nuh_layer_id > 0 */
+        GetBitContext gbc;
+        int num_arrays;
+
+        if (size < 23)
+            return AVERROR_INVALIDDATA;
+
+        ret = init_get_bits8(&gbc, data, size);
+        if (ret < 0)
+            return ret;
+
+        hvcc_init(&hvcc);
+        skip_bits(&gbc, 8); // hvcc.configurationVersion
+        hvcc.general_profile_space = get_bits(&gbc, 2);
+        hvcc.general_tier_flag = get_bits1(&gbc);
+        hvcc.general_profile_idc = get_bits(&gbc, 5);
+        hvcc.general_profile_compatibility_flags = get_bits_long(&gbc, 32);
+        hvcc.general_constraint_indicator_flags = get_bits64(&gbc, 48);
+        hvcc.general_level_idc = get_bits(&gbc, 8);
+        skip_bits(&gbc, 4); // reserved
+        hvcc.min_spatial_segmentation_idc = get_bits(&gbc, 12);
+        skip_bits(&gbc, 6); // reserved
+        hvcc.parallelismType = get_bits(&gbc, 2);
+        skip_bits(&gbc, 6); // reserved
+        hvcc.chromaFormat = get_bits(&gbc, 2);
+        skip_bits(&gbc, 5); // reserved
+        hvcc.bitDepthLumaMinus8 = get_bits(&gbc, 3);
+        skip_bits(&gbc, 5); // reserved
+        hvcc.bitDepthChromaMinus8 = get_bits(&gbc, 3);
+        hvcc.avgFrameRate = get_bits(&gbc, 16);
+        hvcc.constantFrameRate = get_bits(&gbc, 2);
+        hvcc.numTemporalLayers = get_bits(&gbc, 3);
+        hvcc.temporalIdNested = get_bits1(&gbc);
+        hvcc.lengthSizeMinusOne = get_bits(&gbc, 2);
+
+        flags |= FLAG_IS_NALFF;
+
+        num_arrays = get_bits(&gbc, 8);
+        for (int i = 0; i < num_arrays; i++) {
+            int type, num_nalus;
+
+            skip_bits(&gbc, 2);
+            type = get_bits(&gbc, 6);
+            num_nalus = get_bits(&gbc, 16);
+            for (int j = 0; j < num_nalus; j++) {
+                int len = get_bits(&gbc, 16);
+
+                if (len > (get_bits_left(&gbc) / 8))
+                    goto end;
+
+                ret = hvcc_parse_nal_unit(data + get_bits_count(&gbc) / 8,
+                                          len, type, &hvcc, flags);
+                if (ret < 0)
+                    goto end;
+
+                skip_bits_long(&gbc, len * 8);
+            }
+        }
+
+        ret = hvcc_write(pb, &hvcc, flags);
+        goto end;
     } else if (!(AV_RB24(data) == 1 || AV_RB32(data) == 1)) {
         /* Not a valid Annex B start code prefix */
         return AVERROR_INVALIDDATA;
     }
 
-    ret = ff_avc_parse_nal_units_buf(data, &start, &size);
+    ret = ff_nal_parse_units_buf(data, &start, &size);
     if (ret < 0)
         return ret;
 
@@ -1098,27 +1257,31 @@ int ff_isom_write_hvcc(AVIOContext *pb, const uint8_t *data,
 
         buf += 4;
 
-        switch (type) {
-        case HEVC_NAL_VPS:
-        case HEVC_NAL_SPS:
-        case HEVC_NAL_PPS:
-        case HEVC_NAL_SEI_PREFIX:
-        case HEVC_NAL_SEI_SUFFIX:
-            ret = hvcc_add_nal_unit(buf, len, ps_array_completeness, &hvcc);
-            if (ret < 0)
-                goto end;
-            break;
-        default:
-            break;
-        }
+        ret = hvcc_parse_nal_unit(buf, len, type, &hvcc, flags);
+        if (ret < 0)
+            goto end;
 
         buf += len;
     }
 
-    ret = hvcc_write(pb, &hvcc);
+    ret = hvcc_write(pb, &hvcc, flags);
 
 end:
     hvcc_close(&hvcc);
     av_free(start);
     return ret;
 }
+
+int ff_isom_write_hvcc(AVIOContext *pb, const uint8_t *data,
+                       int size, int ps_array_completeness)
+{
+    return write_configuration_record(pb, data, size,
+                                      !!ps_array_completeness * FLAG_ARRAY_COMPLETENESS);
+}
+
+int ff_isom_write_lhvc(AVIOContext *pb, const uint8_t *data,
+                       int size, int ps_array_completeness)
+{
+    return write_configuration_record(pb, data, size,
+                                      (!!ps_array_completeness * FLAG_ARRAY_COMPLETENESS) | FLAG_IS_LHVC);
+}
