@@ -22,6 +22,7 @@
 #include "libavutil/intreadwrite.h"
 #include "libavutil/intfloat.h"
 #include "avformat.h"
+#include "demux.h"
 #include "riff.h"
 
 static int read_probe(const AVProbeData *p)
@@ -96,11 +97,11 @@ static int read_packet(AVFormatContext *s, AVPacket *pkt)
     return ret;
 }
 
-const AVInputFormat ff_mgsts_demuxer = {
-    .name        = "mgsts",
-    .long_name   = NULL_IF_CONFIG_SMALL("Metal Gear Solid: The Twin Snakes"),
+const FFInputFormat ff_mgsts_demuxer = {
+    .p.name      = "mgsts",
+    .p.long_name = NULL_IF_CONFIG_SMALL("Metal Gear Solid: The Twin Snakes"),
+    .p.flags     = AVFMT_GENERIC_INDEX,
     .read_probe  = read_probe,
     .read_header = read_header,
     .read_packet = read_packet,
-    .flags       = AVFMT_GENERIC_INDEX,
 };
