@@ -20,8 +20,10 @@
  */
 
 #include "avformat.h"
+#include "demux.h"
 #include "libavutil/avstring.h"
 #include "libavutil/intreadwrite.h"
+#include "libavutil/mem.h"
 #include "network.h"
 #include "os_support.h"
 #include "internal.h"
@@ -196,6 +198,9 @@ static int sap_fetch_packet(AVFormatContext *s, AVPacket *pkt)
     struct pollfd p = {fd, POLLIN, 0};
     uint8_t recvbuf[RTP_MAX_PACKET_LENGTH];
 
+    if (fd < 0)
+        return fd;
+
     if (sap->eof)
         return AVERROR_EOF;
 
@@ -232,13 +237,13 @@ static int sap_fetch_packet(AVFormatContext *s, AVPacket *pkt)
     return ret;
 }
 
-const AVInputFormat ff_sap_demuxer = {
-    .name           = "sap",
-    .long_name      = NULL_IF_CONFIG_SMALL("SAP input"),
+const FFInputFormat ff_sap_demuxer = {
+    .p.name         = "sap",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("SAP input"),
+    .p.flags        = AVFMT_NOFILE,
     .priv_data_size = sizeof(struct SAPState),
     .read_probe     = sap_probe,
     .read_header    = sap_read_header,
     .read_packet    = sap_fetch_packet,
     .read_close     = sap_read_close,
-    .flags          = AVFMT_NOFILE,
 };
