@@ -19,10 +19,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "config.h"
+
 #include <stdint.h>
 
-#include "avutil.h"
-#include "common.h"
+#include "attributes.h"
+#include "error.h"
 #include "intreadwrite.h"
 #include "mem.h"
 #include "des.h"
