@@ -60,21 +60,18 @@ int av_file_map(const char *filename, uint8_t **bufptr, size_t *size,
     struct stat st;
     av_unused void *ptr;
     off_t off_size;
-    char errbuf[128];
     *bufptr = NULL;
     *size = 0;
 
     if (fd < 0) {
         err = AVERROR(errno);
-        av_strerror(err, errbuf, sizeof(errbuf));
-        av_log(&file_log_ctx, AV_LOG_ERROR, "Cannot read file '%s': %s\n", filename, errbuf);
+        av_log(&file_log_ctx, AV_LOG_ERROR, "Cannot read file '%s': %s\n", filename, av_err2str(err));
         return err;
     }
 
     if (fstat(fd, &st) < 0) {
         err = AVERROR(errno);
-        av_strerror(err, errbuf, sizeof(errbuf));
-        av_log(&file_log_ctx, AV_LOG_ERROR, "Error occurred in fstat(): %s\n", errbuf);
+        av_log(&file_log_ctx, AV_LOG_ERROR, "Error occurred in fstat(): %s\n", av_err2str(err));
         close(fd);
         return err;
     }
@@ -97,8 +94,7 @@ int av_file_map(const char *filename, uint8_t **bufptr, size_t *size,
     ptr = mmap(NULL, *size, PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0);
     if (ptr == MAP_FAILED) {
         err = AVERROR(errno);
-        av_strerror(err, errbuf, sizeof(errbuf));
-        av_log(&file_log_ctx, AV_LOG_ERROR, "Error occurred in mmap(): %s\n", errbuf);
+        av_log(&file_log_ctx, AV_LOG_ERROR, "Error occurred in mmap(): %s\n", av_err2str(err));
         close(fd);
         *size = 0;
         return err;
@@ -155,9 +151,3 @@ void av_file_unmap(uint8_t *bufptr, size_t size)
     av_free(bufptr);
 #endif
 }
-
-#if FF_API_AV_FOPEN_UTF8
-int av_tempfile(const char *prefix, char **filename, int log_offset, void *log_ctx) {
-    return avpriv_tempfile(prefix, filename, log_offset, log_ctx);
-}
-#endif
