@@ -22,6 +22,7 @@
 #include "tx.h"
 #include "thread.h"
 #include "mem_internal.h"
+#include "common.h"
 #include "attributes.h"
 
 #ifdef TX_FLOAT
@@ -74,7 +75,8 @@ typedef void TXComplex;
         .function   = TX_FN_NAME(fn, suffix),                                  \
         .type       = TX_TYPE(tx_type),                                        \
         .flags      = FF_TX_ALIGNED | FF_TX_OUT_OF_PLACE | cd_flags,           \
-        .factors    = { f1, f2 },                                              \
+        .factors    = { (f1), (f2) },                                          \
+        .nb_factors = !!(f1) + !!(f2),                                         \
         .min_len    = len_min,                                                 \
         .max_len    = len_max,                                                 \
         .init       = init_fn,                                                 \
@@ -133,7 +135,7 @@ typedef void TXComplex;
     } while (0)
 
 #define UNSCALE(x) ((double)(x)/2147483648.0)
-#define RESCALE(x) (av_clip64(lrintf((x) * 2147483648.0), INT32_MIN, INT32_MAX))
+#define RESCALE(x) (av_clip64(llrintf((x) * 2147483648.0), INT32_MIN, INT32_MAX))
 
 #define FOLD(x, y) ((int32_t)((x) + (unsigned)(y) + 32) >> 6)
 
@@ -154,6 +156,7 @@ typedef void TXComplex;
 #define FF_TX_PRESHUFFLE   (1ULL << 61) /* Codelet expects permuted coeffs            */
 #define FF_TX_INVERSE_ONLY (1ULL << 60) /* For non-orthogonal inverse-only transforms */
 #define FF_TX_FORWARD_ONLY (1ULL << 59) /* For non-orthogonal forward-only transforms */
+#define FF_TX_ASM_CALL     (1ULL << 58) /* For asm->asm functions only                */
 
 typedef enum FFTXCodeletPriority {
     FF_TX_PRIO_BASE = 0,               /* Baseline priority */
@@ -165,15 +168,34 @@ typedef enum FFTXCodeletPriority {
     FF_TX_PRIO_MAX          =  32768,  /* For custom implementations/ASICs */
 } FFTXCodeletPriority;
 
+typedef enum FFTXMapDirection {
+    /* No map. Make a map up. */
+    FF_TX_MAP_NONE = 0,
+
+    /* Lookup table must be applied via dst[i] = src[lut[i]]; */
+    FF_TX_MAP_GATHER,
+
+    /* Lookup table must be applied via dst[lut[i]] = src[i]; */
+    FF_TX_MAP_SCATTER,
+} FFTXMapDirection;
+
 /* Codelet options */
 typedef struct FFTXCodeletOptions {
-    int invert_lookup;     /* If codelet is flagged as FF_TX_CODELET_PRESHUFFLE,
-                              invert the lookup direction for the map generated */
+    /* Request a specific lookup table direction. Codelets MUST put the
+     * direction in AVTXContext. If the codelet does not respect this, a
+     * conversion will be performed. */
+    FFTXMapDirection map_dir;
 } FFTXCodeletOptions;
 
+/* Maximum number of factors a codelet may have. Arbitrary. */
+#define TX_MAX_FACTORS 16
+
 /* Maximum amount of subtransform functions, subtransforms and factors. Arbitrary. */
 #define TX_MAX_SUB 4
 
+/* Maximum number of returned results for ff_tx_decompose_length. Arbitrary. */
+#define TX_MAX_DECOMPOSITIONS 512
+
 typedef struct FFTXCodelet {
     const char    *name;          /* Codelet name, for debugging */
     av_tx_fn       function;      /* Codelet function, != NULL */
@@ -183,13 +205,16 @@ typedef struct FFTXCodelet {
     uint64_t flags;               /* A combination of AVTXFlags and codelet
                                    * flags that describe its properties. */
 
-    int factors[TX_MAX_SUB];      /* Length factors */
+    int factors[TX_MAX_FACTORS];  /* Length factors. MUST be coprime. */
 #define TX_FACTOR_ANY -1          /* When used alone, signals that the codelet
                                    * supports all factors. Otherwise, if other
                                    * factors are present, it signals that whatever
                                    * remains will be supported, as long as the
                                    * other factors are a component of the length */
 
+    int nb_factors;               /* Minimum number of factors that have to
+                                   * be a modulo of the length. Must not be 0. */
+
     int min_len;                  /* Minimum length of transform, must be >= 1 */
     int max_len;                  /* Maximum length of transform */
 #define TX_LEN_UNLIMITED -1       /* Special length value to permit all lengths */
@@ -217,7 +242,8 @@ struct AVTXContext {
     int                len;             /* Length of the transform */
     int                inv;             /* If transform is inverse */
     int               *map;             /* Lookup table(s) */
-    TXComplex         *exp;             /* Any non-pre-baked multiplication factors needed */
+    TXComplex         *exp;             /* Any non-pre-baked multiplication factors,
+                                         * or extra temporary buffer */
     TXComplex         *tmp;             /* Temporary buffer, if needed */
 
     AVTXContext       *sub;             /* Subtransform context(s), if needed */
@@ -235,11 +261,32 @@ struct AVTXContext {
     enum AVTXType      type;            /* Type of transform */
     uint64_t           flags;           /* A combination of AVTXFlags and
                                          * codelet flags used when creating */
+    FFTXMapDirection   map_dir;         /* Direction of AVTXContext->map */
     float              scale_f;
     double             scale_d;
     void              *opaque;          /* Free to use by implementations */
 };
 
+/* This function embeds a Ruritanian PFA input map into an existing lookup table
+ * to avoid double permutation. This allows for compound factors to be
+ * synthesized as fast PFA FFTs and embedded into either other or standalone
+ * transforms.
+ * The output CRT map must still be pre-baked into the transform. */
+#define TX_EMBED_INPUT_PFA_MAP(map, tot_len, d1, d2)                             \
+    do {                                                                         \
+        int mtmp[(d1)*(d2)];                                                     \
+        for (int k = 0; k < tot_len; k += (d1)*(d2)) {                           \
+            memcpy(mtmp, &map[k], (d1)*(d2)*sizeof(*mtmp));                      \
+            for (int m = 0; m < (d2); m++)                                       \
+                for (int n = 0; n < (d1); n++)                                   \
+                    map[k + m*(d1) + n] = mtmp[(m*(d1) + n*(d2)) % ((d1)*(d2))]; \
+        }                                                                        \
+    } while (0)
+
+/* This function generates a Ruritanian PFA input map into s->map. */
+int ff_tx_gen_pfa_input_map(AVTXContext *s, FFTXCodeletOptions *opts,
+                            int d1, int d2);
+
 /* Create a subtransform in the current context with the given parameters.
  * The flags parameter from FFTXCodelet.init() should be preserved as much
  * as that's possible.
@@ -248,11 +295,26 @@ int ff_tx_init_subtx(AVTXContext *s, enum AVTXType type,
                      uint64_t flags, FFTXCodeletOptions *opts,
                      int len, int inv, const void *scale);
 
+/* Clear the context by freeing all tables, maps and subtransforms. */
+void ff_tx_clear_ctx(AVTXContext *s);
+
+/* Attempt to factorize a length into 2 integers such that
+ * len / dst1 == dst2, where dst1 and dst2 are coprime. */
+int ff_tx_decompose_length(int dst[TX_MAX_DECOMPOSITIONS], enum AVTXType type,
+                           int len, int inv);
+
+/* Generate a default map (0->len or 0, (len-1)->1 for inverse transforms)
+ * for a context. */
+int ff_tx_gen_default_map(AVTXContext *s, FFTXCodeletOptions *opts);
+
 /*
  * Generates the PFA permutation table into AVTXContext->pfatab. The end table
  * is appended to the start table.
+ * The `inv` flag should only be enabled if the lookup tables of subtransforms
+ * won't get flattened.
  */
-int ff_tx_gen_compound_mapping(AVTXContext *s, int n, int m);
+int ff_tx_gen_compound_mapping(AVTXContext *s, FFTXCodeletOptions *opts,
+                               int inv, int n, int m);
 
 /*
  * Generates a standard-ish (slightly modified) Split-Radix revtab into
@@ -260,14 +322,14 @@ int ff_tx_gen_compound_mapping(AVTXContext *s, int n, int m);
  * If it's set to 0, it has to be applied like out[map[i]] = in[i], otherwise
  * if it's set to 1, has to be applied as out[i] = in[map[i]]
  */
-int ff_tx_gen_ptwo_revtab(AVTXContext *s, int invert_lookup);
+int ff_tx_gen_ptwo_revtab(AVTXContext *s, FFTXCodeletOptions *opts);
 
 /*
  * Generates an index into AVTXContext->inplace_idx that if followed in the
  * specific order, allows the revtab to be done in-place. The sub-transform
  * and its map should already be initialized.
  */
-int ff_tx_gen_ptwo_inplace_revtab_idx(AVTXContext *s);
+int ff_tx_gen_inplace_map(AVTXContext *s, int len);
 
 /*
  * This generates a parity-based revtab of length len and direction inv.
@@ -296,8 +358,12 @@ int ff_tx_gen_ptwo_inplace_revtab_idx(AVTXContext *s);
  * functions in AVX mode.
  *
  * If length is smaller than basis/2 this function will not do anything.
+ *
+ * If inv_lookup is set to 1, it will flip the lookup from out[map[i]] = src[i]
+ * to out[i] = src[map[i]].
  */
-int ff_tx_gen_split_radix_parity_revtab(AVTXContext *s, int invert_lookup,
+int ff_tx_gen_split_radix_parity_revtab(AVTXContext *s, int len, int inv,
+                                        FFTXCodeletOptions *opts,
                                         int basis, int dual_stride);
 
 /* Typed init function to initialize shared tables. Will initialize all tables
@@ -306,14 +372,18 @@ void ff_tx_init_tabs_float (int len);
 void ff_tx_init_tabs_double(int len);
 void ff_tx_init_tabs_int32 (int len);
 
-/* Typed init function to initialize an MDCT exptab in a context. */
-int  ff_tx_mdct_gen_exp_float (AVTXContext *s);
-int  ff_tx_mdct_gen_exp_double(AVTXContext *s);
-int  ff_tx_mdct_gen_exp_int32 (AVTXContext *s);
+/* Typed init function to initialize an MDCT exptab in a context.
+ * If pre_tab is set, duplicates the entire table, with the first
+ * copy being shuffled according to pre_tab, and the second copy
+ * being the original. */
+int ff_tx_mdct_gen_exp_float (AVTXContext *s, int *pre_tab);
+int ff_tx_mdct_gen_exp_double(AVTXContext *s, int *pre_tab);
+int ff_tx_mdct_gen_exp_int32 (AVTXContext *s, int *pre_tab);
 
 /* Lists of codelets */
 extern const FFTXCodelet * const ff_tx_codelet_list_float_c       [];
 extern const FFTXCodelet * const ff_tx_codelet_list_float_x86     [];
+extern const FFTXCodelet * const ff_tx_codelet_list_float_aarch64 [];
 
 extern const FFTXCodelet * const ff_tx_codelet_list_double_c      [];
 
